@@ -1,7 +1,7 @@
 /*
     WN: A Server for the HTTP
     File: wn/auth.h
-    Version 1.15.7
+    Version 1.19.9
     
     Copyright (C) 1995, 1996  <by John Franks>
 
@@ -24,6 +24,7 @@
 #define AUTH_GRANTED	(0)
 #define AUTH_DENIED	(1)
 #define AUTH_EXPIRED	(2)
+#define AUTH_UNTESTED	(-2)
 
 static
 WN_CONST
@@ -33,7 +34,7 @@ char * WN_CONST autherr_m[] = {
 	/* 2 */ "",	
 	/* 3 */ "Badly formed user info string",
 	/* 4 */ "Can't open passwd file",
-	/* 5 */ "Can't init dbm file",
+	/* 5 */ "Can't open dbm file",
 	/* 6 */ "Can't open group file",
 	/* 7 */ "No password file listed on command line",
 	/* 8 */ "DBM code for authorization not installed",
@@ -43,8 +44,13 @@ char * WN_CONST autherr_m[] = {
 	/* 12 */ "Improper usage: bad options",
 	/* 13 */ "Authorization denied",
 	/* 14 */ "Authorization expired",
-	/* 15 */ "",
-	/* 16 */ "Authentication timed out"
+	/* 15 */ "Unsupported algorithm for digest",
+	/* 16 */ "Authentication timed out",
+	/* 17 */ "No realm available"
 };
 
 	
+
+
+
+
