@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/cgi.c
-    Version 1.17.12
+    Version 2.0.4
     
-    Copyright (C) 1996  <by John Franks>
+    Copyright (C) 1996-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,14 +23,14 @@
 
 #include "../config.h"
 #include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
 #include <string.h>
 #include <errno.h>
 #include "wn.h"
 #include "version.h"
 #include "cgi.h"
 
-extern char	*malloc();
-
 static void	cgi_headers();
 
 static CGI_data	*cgip = NULL;
@@ -63,10 +63,11 @@ Request	*ip;
 	location[0] = '\0';
 	*ip->length = '\0';  /* Don't send length of script!! */
 
-	/* if ( NOT CHUNKING ) */
-		this_conp->keepalive = FALSE;
-
 	if ( ip->type == RTYPE_CGI_HANDLER) {
+		if ( (inheadp->method != GET) && (inheadp->method != POST) ) {
+			senderr( SERV_ERR, err_m[117], ip->filepath);
+			return;
+		}
 		mystrncpy( ip->pathinfo, ip->relpath, MIDLEN);
 		getfpath( buf, ip->handler, ip);
 		mystrncpy( command, buf, MIDLEN - SMALLLEN);
@@ -93,7 +94,8 @@ Request	*ip;
 		strcat( command, inheadp->tmpfile);
 	}
 
-	if ( !*ip->query || (strchr( ip->query, '=') != NULL)) {
+	if ( !*ip->query || (strchr( ip->query, '=') != NULL)
+	     		|| ( ip->type == RTYPE_CGI_HANDLER) ) {
 		/* '=' in query so don't use on command line */
 		if ((fp = popen( command, "r")) == (FILE *) NULL ) {
 			senderr( SERV_ERR, err_m[55], ip->filepath);
@@ -120,8 +122,10 @@ Request	*ip;
 
 		pclose( fp);
 		writelog( ip, log_m[7], ip->filepath);
-		if ( *inheadp->tmpfile)
+		if ( *inheadp->tmpfile) {
 			unlink( inheadp->tmpfile);
+			*inheadp->tmpfile = '\0';
+		}
 		return; /* to end of process_url */
 	}
 
@@ -140,17 +144,21 @@ Request	*ip;
 
 		pclose( fp);
 		writelog( ip, log_m[8], ip->filepath);
-		if ( *inheadp->tmpfile)
+		if ( *inheadp->tmpfile) {
 			unlink( inheadp->tmpfile);
+			*inheadp->tmpfile = '\0';
+		}
 		return;  /* to end of process_url */
 	}
 	else {
 		writelog( ip, log_m[9], location);
 		sprintf( ip->request, log_m[12], location);
-		dolocation( location, ip);
+		dolocation( location, ip, 302); /* should be 303 */
 		pclose( fp);
-		if ( *inheadp->tmpfile)
+		if ( *inheadp->tmpfile) {
 			unlink( inheadp->tmpfile);
+			*inheadp->tmpfile = '\0';
+		}
 		return; /* to end of process_url */
 	}
 #endif
@@ -180,6 +188,7 @@ int	auth;
 		}
 	}
 
+	bzero( (char *)cgip, sizeof( CGI_data)); 
 	strcpy( cgip->method, "REQUEST_METHOD=");
 	switch ( inheadp->method) {
 	case GET:
@@ -220,6 +229,36 @@ int	auth;
 		putenv( cgip->authtype);
 	}
 
+#ifdef OPEN_BIG_SECURITY_HOLE
+	if ( *(inheadp->authorization)) {
+		strcpy( cgip->authorization, "HTTP_AUTHORIZATION=");
+		mystrncat( cgip->authorization, inheadp->authorization,
+						MIDLEN - 20);
+		putenv( cgip->authorization);
+	}
+#endif
+
+#ifdef DIGEST_AUTHENTICATION
+	if ( strncasecmp( inheadp->authorization, "Digest", 6) == 0 &&
+			dir_p->authtype &&
+			strcasecmp( dir_p->authtype, "Digest") == 0 ) {
+		if ( !(*cgip->authorization)) {
+			strcpy( cgip->authorization, "HTTP_AUTHORIZATION=");
+			mystrncat( cgip->authorization, inheadp->authorization,
+							MIDLEN - 20);
+			putenv( cgip->authorization);
+		}
+		if ( (cp = strchr( outheadp->md5, ':')) != NULL ) {
+			strcpy( cgip->md5, "CONTENT_MD5=");
+			cp++;
+			while ( isspace ( *cp))
+				cp++;
+			mystrncat( cgip->md5, cp, TINYLEN);
+			putenv( cgip->md5);
+		}
+	}
+#endif
+
 	if ( *(inheadp->host_head) ) {
 		strcpy( cgip->http_myhost, "HTTP_HOST=");
 		mystrncat( cgip->http_myhost, inheadp->host_head, SMALLLEN);
@@ -231,12 +270,16 @@ int	auth;
 	putenv( cgip->lochost);
 
 	strcpy( cgip->dataroot, "WN_ROOT=");
-	strcat( cgip->dataroot, ip->rootdir);
+	mystrncat( cgip->dataroot, ip->rootdir, SMALLLEN - 20);
 	putenv( cgip->dataroot);
 
+	strcpy( cgip->dataroot2, "DOCUMENT_ROOT=");
+	mystrncat( cgip->dataroot2, ip->rootdir, SMALLLEN - 20);
+	putenv( cgip->dataroot2);
+
 	if ( !(this_conp->con_status & WN_CON_CGI_SET)) {
 		strcpy( cgip->raddr, "REMOTE_ADDR=");
-		mystrncat( cgip->raddr, remaddr, TINYLEN);
+		mystrncat( cgip->raddr, this_conp->remaddr, TINYLEN);
 		putenv( cgip->raddr);
 
 		sprintf( cgip->lport,"SERVER_PORT=%d", port);
@@ -252,15 +295,15 @@ int	auth;
 
 		this_conp->con_status |=  WN_CON_CGI_SET;
 
-		if ( !*remotehost )
+		if ( !*this_conp->remotehost )
 				/* Get remote hostname if not already done */
 			get_remote_info( );
 
 		strcpy( cgip->rhost, "REMOTE_HOST=");
-		mystrncat( cgip->rhost, remotehost, MAXHOSTNAMELEN);
+		mystrncat( cgip->rhost, this_conp->remotehost, MAXHOSTNAMELEN);
 		putenv( cgip->rhost);
 
-		sprintf( cgip->scheme,"URL_SCHEME=%s", this_conp->scheme);
+		sprintf( cgip->scheme,"URL_SCHEME=%.32s", this_conp->scheme);
 		putenv( cgip->scheme);
 
 		putenv("GATEWAY_INTERFACE=CGI/1.1");
@@ -269,12 +312,15 @@ int	auth;
 		strcat( cgip->servsoft, VERSION);
 		putenv( cgip->servsoft);
 
+#ifdef RFC931_TIMEOUT
+		get_rfc931();
 		if ( *(this_conp->rfc931name)) {
 			strcpy( cgip->rident, "REMOTE_IDENT=");
 			mystrncat( cgip->rident,
 					this_conp->rfc931name, SMALLLEN);
 			putenv( cgip->rident);
 		}
+#endif                    
 	}
 
 	if ( *(ip->query)) {
@@ -305,18 +351,34 @@ int	auth;
 	}
 
 	strcpy( cgip->tpath, "PATH_TRANSLATED=");
-	strcat( cgip->tpath, ip->rootdir);
-	mystrncat( cgip->tpath,	ip->pathinfo, MIDLEN);
+	if ( *ip->pathinfo) {
+		mystrncat( cgip->tpath, ip->rootdir, SMALLLEN);
+		mystrncat( cgip->tpath,	ip->pathinfo, MIDLEN);
+	}
 	putenv( cgip->tpath);
 
 	strcpy( cgip->scrname, "SCRIPT_NAME=");
-	if ( (ip->type == RTYPE_CGI_HANDLER) && (*ip->handler))
-		mystrncat( cgip->scrname, ip->handler, MIDLEN - 20);
+	if ( (ip->type == RTYPE_CGI_HANDLER) && (*ip->handler)) {
+		cp = strrchr( ip->handler, '/');
+		if ( cp)
+			mystrncat( cgip->scrname, cp+1, MIDLEN - 20);
+		else 
+			*(cgip->scrname) = '\0';
+	}
 	else {
 		cp = ip->filepath + strlen( ip->rootdir);
 		mystrncat( cgip->scrname, cp, MIDLEN - 20);
 	}
-	putenv( cgip->scrname);
+	if ( *(cgip->scrname) )
+		putenv( cgip->scrname);
+
+	strcpy( cgip->filescrname, "SCRIPT_FILENAME=");
+	if ( (ip->type == RTYPE_CGI_HANDLER) && (*ip->handler))
+		mystrncat( cgip->filescrname, ip->handler, MIDLEN - 20);
+	else {
+		mystrncat( cgip->filescrname, ip->filepath, MIDLEN - 20);
+	}
+	putenv( cgip->filescrname);
 
 	if ( *(inheadp->content) ) {
 		strcpy( cgip->content, "CONTENT_TYPE=");
