@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/cgi.h
-    Version 1.17.4
+    Version 2.0.3
     
-    Copyright (C) 1995-7  <by John Franks>
+    Copyright (C) 1995-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,18 +21,19 @@
 
 */
 #define CGI_BYTECHUNK	(128*1024)
-#define WN_CON_CGI_SET	(1)
 
 typedef struct CGI_data {
 char	cgi_content_type[SMALLLEN],  /* output content type */
 	serv_protocol[TINYLEN],
 	servsoft[TINYLEN],
 	dataroot[TINYLEN + SMALLLEN],
+	dataroot2[TINYLEN + SMALLLEN],
 	dirpath[TINYLEN + SMALLLEN],
 	query[MIDLEN + SMALLLEN],
 	pathinfo[MIDLEN + SMALLLEN],
 	tpath[MIDLEN + SMALLLEN],
 	scrname[MIDLEN + SMALLLEN],
+	filescrname[MIDLEN + SMALLLEN],
 	http_accept[ACCEPTLEN + TINYLEN],
 	http_lang[ACCEPTLEN/4 + TINYLEN],
 	http_charset[ACCEPTLEN/4 + TINYLEN],
@@ -43,6 +44,8 @@ char	cgi_content_type[SMALLLEN],  /* output content type */
 	http_myhost[TINYLEN + SMALLLEN],
 	lochost[TINYLEN + SMALLLEN],
 	authtype[TINYLEN],
+	authorization[MIDLEN],
+	md5[2*TINYLEN],		/* Content-MD5 header */
 	lport[TINYLEN],
 	scheme[TINYLEN],
 	rhost[MAXHOSTNAMELEN + TINYLEN],
