@@ -1,7 +1,7 @@
 /*
     Wn: A Server for the HTTP
     File: wn/chkauth.c
-    Version 1.17.9
+    Version 2.0.4
 
     Copyright (C) 1996, 1997  <by John Franks>
 
@@ -59,7 +59,7 @@ char	*bufcoded,
 
 	register char	*in;
 	char		buf[SMALLLEN + TINYLEN];
-    
+
 	
 	while( isspace(*bufcoded))
 		bufcoded++;
@@ -67,7 +67,7 @@ char	*bufcoded,
 	mystrncpy( buf, bufcoded, SMALLLEN);
 	in = buf;
 
-	while( *in && (tr[*in &= 0177] >= 0))
+	while( *in && (tr[(unsigned) (*in &= 0177)] >= 0))
 		in++;
 	*in++ = 0;
 	*in++ = 0;
@@ -77,23 +77,29 @@ char	*bufcoded,
 	in = buf;
     
 	while ( in[3] ) {
-        	*out++ = (unsigned char) (tr[in[0]] << 2 | tr[in[1]] >> 4);
-	        *out++ = (unsigned char) (tr[in[1]] << 4 | tr[in[2]] >> 2);
-        	*out++ = (unsigned char) (tr[in[2]] << 6 | tr[in[3]]);
+        	*out++ = (unsigned char) (tr[(unsigned)in[0]] << 2 
+						| tr[(unsigned)in[1]] >> 4);
+	        *out++ = (unsigned char) (tr[(unsigned)in[1]] << 4 
+						| tr[(unsigned)in[2]] >> 2);
+        	*out++ = (unsigned char) (tr[(unsigned)in[2]] << 6 
+						| tr[(unsigned)in[3]]);
 		in += 4;
 	}
 	
     	if ( in[2] ) {
-        	*out++ = (unsigned char) (tr[in[0]] << 2 | tr[in[1]] >> 4);
-	        *out++ = (unsigned char) (tr[in[1]] << 4 | tr[in[2]] >> 2);
-        	*out++ = (unsigned char) tr[in[2]] << 6;
+        	*out++ = (unsigned char) (tr[(unsigned)in[0]] << 2 
+						| tr[(unsigned)in[1]] >> 4);
+	        *out++ = (unsigned char) (tr[(unsigned)in[1]] << 4 
+						| tr[(unsigned)in[2]] >> 2);
+        	*out++ = (unsigned char) tr[(unsigned)in[2]] << 6;
 	}
 	else if ( in[1]) {
-        	*out++ = (unsigned char) (tr[in[0]] << 2 | tr[in[1]] >> 4);
-	        *out++ = (unsigned char) tr[in[1]] << 4;
+        	*out++ = (unsigned char) (tr[(unsigned)in[0]] << 2 
+						| tr[(unsigned)in[1]] >> 4);
+	        *out++ = (unsigned char) tr[(unsigned)in[1]] << 4;
 	}
 	else if ( in[0] ) {
-        	*out++ = (unsigned char) tr[in[0]] << 2;
+        	*out++ = (unsigned char) tr[(unsigned)in[0]] << 2;
 	}
 	*out = '\0';
 }
@@ -113,41 +119,33 @@ Request	*ip;
 
 	char	*authheadp = NULL,
 		authcmd[MIDLEN + 2*SMALLLEN],
-		buf[SMALLLEN];
+		buf[MIDLEN];
 
 	int	status,
 		result;
 
-	FILE	*fp;
+	FILE	*fp = NULL;
 
+	signal( _WN_SIGCHLD, SIG_DFL);
 
-
-        signal( _WN_SIGCHLD, SIG_DFL);
-
-	
+	result = AUTH_UNTESTED;
 	mystrncpy( authcmd, dir_p->authmod, SMALLLEN);
 
-#ifdef DIGEST_AUTHENTICATION
-	if ( strcasecmp( dir_p->authtype, "Digest") == 0) {
-		strcat( authcmd, " -r ");
-		mystrncat( authcmd, dir_p->authrealm, SMALLLEN);
-	}
-#endif
+	cp = inheadp->authorization;
 
-	if ( *(inheadp->authorization)) {
+	if ( *cp ) {
 		if ( ip->attributes & WN_CGI )
 			cgi_env( ip, FALSE);
 		else
 			cgi_env( ip, TRUE);  /* Small (auth) CGI environ */
 
-		if ((fp = popen( authcmd, "w"))  == (FILE *) NULL ) {
-			senderr( SERV_ERR, err_m[14], authcmd);
-			wn_exit( 2);
-		}
+		if ( strncasecmp( cp, "Basic", 5) == 0 &&
+				strcasecmp( dir_p->authtype, "Basic") == 0) {
 
-		cp = inheadp->authorization;
-
-		if ( strncasecmp( cp, "Basic", 5) == 0) {
+			if ((fp = popen( authcmd, "w"))  == (FILE *) NULL ) {
+				senderr( SERV_ERR, err_m[14], authcmd);
+				wn_exit( 2);   /* senderr: SERV_ERR */
+			}
 			strcpy( buf, "Basic ");
 			cp += 5;
 			cp2 = buf + 6;
@@ -156,9 +154,32 @@ Request	*ip;
 			if ( ( cp = strchr( ip->authuser, ':')) != NULL)
 				*cp = '\0';
 			authheadp = buf;
+			fprintf( fp, "%s\n", authheadp);
+		}
+		else if ( strncasecmp( cp, "cert", 4) == 0 &&
+			strcasecmp( dir_p->authtype, "Certificate") == 0) {
+
+			strcpy(  inheadp->authorization, "cert");
+			if ((fp = popen( authcmd, "w"))  == (FILE *) NULL ) {
+				senderr( SERV_ERR, err_m[14], authcmd);
+				wn_exit( 2);   /* senderr: SERV_ERR */
+			}
+			strcpy( buf, "Client Cert");
+			authheadp = buf;
+			fprintf( fp, "%s\n", authheadp);
 		}
 #ifdef DIGEST_AUTHENTICATION
-		else if ( strncasecmp( cp, "Digest", 6) == 0) {
+		else if ( strncasecmp( cp, "Digest", 6) == 0 &&
+				strcasecmp( dir_p->authtype, "Digest") == 0 ) {
+			char	auth_uri[MIDLEN];
+
+			strcat( authcmd, " -r ");
+			mystrncat( authcmd, dir_p->authrealm,SMALLLEN);
+
+			if ((fp = popen( authcmd, "r"))  == (FILE *) NULL ) {
+				senderr( SERV_ERR, err_m[14], authcmd);
+				wn_exit( 2);   /* senderr: SERV_ERR */
+			}
 			cp2 = inheadp->authorization;
 			if ( (cp = strstr( cp2, "username")) == NULL)
 				cp = strstr( cp2, "Username");
@@ -171,8 +192,17 @@ Request	*ip;
 					*cp = '\0';
 			}
 			authheadp = inheadp->authorization;
+			/* For digest compare the URI and auth header URI */
+			if ( ( cp = strstr( authheadp, "uri=\"")) ||
+					(cp = strstr( authheadp, "URI=\""))) {
+				cp += 5;
+				mystrncpy( auth_uri, cp, MIDLEN);
+				if ( (cp = strchr( auth_uri, '"')) != NULL ) 
+					*cp = '\0';
+				if ( !streq( auth_uri, inheadp->auth_url_path))
+					result = AUTH_DENIED;
+			}
 		}
-
 #endif
 		else {
 			mystrncpy( buf, inheadp->authorization, SMALLLEN);
@@ -181,33 +211,34 @@ Request	*ip;
 				cp++;
 			*cp = '\0';
 			senderr( SERV_ERR, autherr_m[9], buf);
-			wn_exit( 2);
+			wn_exit( 2);   /* senderr: SERV_ERR */
 		}
 
 
 		if ( authheadp == NULL) {
 			sendauth( ip, "-s false", autherr_m[10]);
-			wn_exit( 2);
+			return FALSE;
 		}
 
-		fprintf( fp, "%s\n", authheadp);
 
-		status = pclose( fp);
+		if ( (result == AUTH_UNTESTED) && fp != NULL) {
+			status = pclose( fp);
 
 #ifdef NEXT
-                if ( (status != 0) && WIFEXITED( (union wait) status))
-                        result = ((status >> 8) & 0377);
+			if ( (status != 0) && WIFEXITED( (union wait) status))
+				result = ((status >> 8) & 0377);
 #else
-		if ( (status != 0) && WIFEXITED( status))
-			result = WEXITSTATUS( status);
+			if ( (status != 0) && WIFEXITED( status))
+				result = WEXITSTATUS( status);
 #endif
-		else
-			result = status;
+			else
+				result = status;
+		}
 
 		switch (result) {
 		case (-1):
 			sendauth( ip, "-s false", autherr_m[11]);
-			wn_exit( 2);
+			return FALSE;
 		case AUTH_GRANTED:
 			return TRUE;
 		case AUTH_DENIED:
@@ -218,34 +249,36 @@ Request	*ip;
 			return FALSE;
 		case (3):
 			sendauth( ip, "-s false", autherr_m[3]);
-			wn_exit( 2);
+			return FALSE;
 		case (4):
 			sendauth( ip, "-s false", autherr_m[4]);
-			wn_exit( 2);
+			return FALSE;
 		case (5):
 			sendauth( ip, "-s false", autherr_m[5]);
-			wn_exit( 2);
+			return FALSE;
 		case (6):
 			sendauth( ip, "-s false", autherr_m[6]);
-			wn_exit( 2);
+			return FALSE;
 		case (7):
 			sendauth( ip, "-s false", autherr_m[7]);
-			wn_exit( 2);
+			return FALSE;
 		case (8):
 			sendauth( ip, "-s false", autherr_m[8]);
-			wn_exit( 2);
+			return FALSE;
 		case (9):
-			sendauth( ip, "-s false", autherr_m[9]);
-			wn_exit( 2);
+			senderr( "406", autherr_m[9], "");
+			return FALSE;
 		case (10):
 			sendauth( ip, "-s false", autherr_m[10]);
-			wn_exit( 2);
-
+			return FALSE;
+		case (15):
+			senderr( "406", autherr_m[15], "");
+			return FALSE;
 		case (16):
 			logerr( autherr_m[16], "");
-			wn_exit( 2);
+			wn_exit( 2);  /* timed out */
 		default:
-			sprintf( buf, "%s %d", autherr_m[0], result);
+			sprintf( buf, "%.100s %d", autherr_m[0], result);
 			logerr( err_m[42], buf);
 			sendauth( ip, "-s false", buf);
 			return FALSE;
@@ -262,11 +295,13 @@ Request	*ip;
 char	*noncearg,
 	*logmsg;
 {
-	char	authcmd[MIDLEN],
-		buf[MIDLEN];
-	FILE	*fp;
+	char	buf[MIDLEN];
+
+	if ( strcasecmp( dir_p->authtype, "Certificate") == 0)
+		strcpy( outheadp->status, "403 Forbidden");
+	else
+		strcpy( outheadp->status, "401 Unauthorized");
 
-	strcpy( outheadp->status, "401 Unauthorized");
 	if ( strcasecmp( dir_p->authtype, "basic") == 0) {
 		sprintf( outheadp->list, 
 				"WWW-Authenticate: Basic realm=\"%.200s\"\r\n",
@@ -274,21 +309,24 @@ char	*noncearg,
 	}
 #ifdef DIGEST_AUTHENTICATION
 	else if ( strcasecmp( dir_p->authtype, "Digest") == 0) {
+		char	authcmd[MIDLEN];
+		FILE	*fp;
+
 		if ( ip->attributes & WN_CGI )
 			cgi_env( ip, FALSE);
 		else
 			cgi_env( ip, TRUE);  /* Small (auth) CGI environ */
 
-		sprintf( authcmd, "%.200s -r %.200s %s", dir_p->authmod,
+		sprintf( authcmd, "%.200s -r %.200s %.100s", dir_p->authmod,
 			dir_p->authrealm, noncearg);
 		if ((fp = popen( authcmd, "r"))  == (FILE *) NULL ) {
 			senderr( SERV_ERR, err_m[14], authcmd);
-			wn_exit( 2);
+			wn_exit( 2);   /* senderr: SERV_ERR */
 		}
 		if ( fgets( outheadp->list, MIDLEN, fp) == NULL) {
 			senderr( SERV_ERR, err_m[50], authcmd);
 			pclose( fp);
-			wn_exit( 2);
+			wn_exit( 2);   /* senderr: SERV_ERR */
 		}
 		pclose( fp);
 	}
@@ -298,23 +336,22 @@ char	*noncearg,
 	ip->mod_time = 0;
 	ip->content_type = "text/html";
 
-
 	if ( *(dir_p->authdenied_file) ) {
-		if ( send_noauth()) {
+		if ( send_noauth( ) ) {
 			writelog( ip, log_m[1], logmsg);
 			return;
 		}
 	}
 		
-	sprintf( buf, "<head>\n<title>%s</title>\n</head>\n<body>\n",
+	sprintf( buf, "<head>\n<title>%.100s</title>\n</head>\n<body>\n",
 			autherr_m[1]);
-	sprintf( buf + strlen(buf), "<h2>%s</h2>\n", autherr_m[1]);
+	sprintf( buf + strlen(buf), "<h2>%.100s</h2>\n", autherr_m[1]);
 
-	sprintf( buf + strlen(buf), "%s\n%s\n </body>\n", 
+	sprintf( buf + strlen(buf), "%.200s\n%.32s\n </body>\n", 
 			 logmsg, SERVER_LOGO);
 	ip->datalen = strlen( buf);
-	sprintf( ip->length, "%d", ip->datalen);
-
+	sprintf( ip->length, "%ld", ip->datalen);
+	ip->status |= WN_HAS_BODY;
 	http_prolog( );
 	send_text_line(buf);
 
