@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/chkcntrl.c
-    Version 1.18.3
+    Version 2.0.4
 
-    Copyright (C) 1995, 1996  <by John Franks>
+    Copyright (C) 1995-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include "../config.h"
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -67,8 +68,6 @@ Request	*ip;
 	Cache_entry	*cep,
 			entry;
 
-
-
 #ifdef ACCESS_DENIED_URL
 	mystrncpy( dir_p->noaccess_url, ACCESS_DENIED_URL, MIDLEN/2);
 #else
@@ -89,7 +88,6 @@ Request	*ip;
 	if ( ip->type == RTYPE_DENIED )  /* done in parse_request */
 		return;
 
-
 	if ( lstat( ip->cachepath, &stat_buf) != 0 ) {
 		writelog( ip,  err_m[2], ip->cachepath);
 		ip->type = RTYPE_DENIED;
@@ -97,6 +95,7 @@ Request	*ip;
 		return;
 	}
 
+
 #ifndef S_ISREG
 #define	S_ISREG(m)	(((m)&S_IFMT) == S_IFREG)
 #endif
@@ -105,16 +104,17 @@ Request	*ip;
 		senderr( SERV_ERR, err_m[13], ip->cachepath);
 		return;
 	}
-	ip->cache_uid = (unsigned) stat_buf.st_uid;
-	ip->cache_gid = (unsigned) stat_buf.st_gid;
+	dir_p->cache_uid = (unsigned) stat_buf.st_uid;
+	dir_p->cache_gid = (unsigned) stat_buf.st_gid;
+	dir_p->cmod_time =  stat_buf.st_mtime;
 
-	if ( (!WN_OPT_U) && (WN_OPT_T) && (!is_trusted( ip)) ) {
+	if ( (!WN_OPT_U) && (WN_OPT_T) && (!IS_TRUSTED) ) {
 		logerr( err_m[7], ip->cachepath);
 		ip->type = RTYPE_DENIED;
 		return;
 	}
 
-	if ( stat_buf.st_uid == (uid_t) USERID) {
+	if ( stat_buf.st_uid == (uid_t) user_id) {
 		logerr( err_m[8], ip->cachepath);
 		ip->type = RTYPE_DENIED;
 		return;
@@ -126,7 +126,6 @@ Request	*ip;
 		return;
 	}
 
-
 	read_cache_dirinfo( fp, dir_p);
 
 	access_status = chkaccess( ip->cachepath,  dir_p->accessfile);
@@ -147,13 +146,13 @@ Request	*ip;
 
 	if ( *(dir_p->defdoc) && ( ip->filetype & WN_DEFAULT_DOC)) {
 		fclose( fp);
-		dolocation( dir_p->defdoc, ip);
+		dolocation( dir_p->defdoc, ip, 301);
 		ip->type = RTYPE_FINISHED;
 		return;
 	}
 
 	if ( *(dir_p->authmod) && (access_status != ACCESS_PRIVILEGED) ) {
-		if ( !(is_trusted( ip) || is_atrusted( ip)) ) {
+		if ( !(IS_TRUSTED || IS_ATRUSTED) ) {
 			senderr( DENYSTATUS, err_m[90], dir_p->authmod);
 			ip->type = RTYPE_FINISHED;
 			fclose( fp);
@@ -173,7 +172,12 @@ Request	*ip;
 		/* anything requiring authentication should not be cached */
 		ip->attributes |= WN_NOCACHE;
 
-		if ( !chkauth( ip)) {
+		if ( !(strcasecmp( dir_p->authtype, "Digest") == 0) &&
+						 !chkauth( ip)) {
+			/*
+			 * chkauth must be delayed for Digest until we know the
+			   md5 digest.
+			 */
 			ip->type = RTYPE_NO_AUTH;
 			fclose( fp);
 			return;
@@ -187,8 +191,11 @@ Request	*ip;
 			return; /* to process_url */
 		}
 		dir_p->filemod = dir_p->filemodule;
+
 		if ( WN_OPT_U )
 			check_perm( ip, dir_p->filemodule);
+
+		ip->attrib2 |= WN_FILEMOD;
 	}
 
 	if ( *(dir_p->cachemod)) {
@@ -218,9 +225,10 @@ Request	*ip;
 
 	if ( iswndir( ip)) {
 		/* It's a title, keyword, grep, or index search of this dir */
+		ip->allowed |= WN_M_GET + WN_M_TRACE;
 		fclose( fp);
 		ip->title = ip->encoding = ip->keywords 
-			= ip->filter = ip->handler = NULL;
+			= ip->filter = ip->handler = wn_empty;
 		ip->content_type = "text/html";
 		return;  
 	}
@@ -240,6 +248,10 @@ Request	*ip;
 				(entry.attributes ? entry.attributes :
 						dir_p->defattributes);
 
+			ip->allowed |= WN_M_GET + WN_M_HEAD
+						+ WN_M_TRACE + WN_M_OPTIONS;
+
+
 			ip->title = entry.title;
 			ip->content_type = entry.content; 
 			ip->encoding = entry.encoding; 
@@ -257,7 +269,7 @@ Request	*ip;
 				ip->attrib2 &= ~(WN_LIST_INCL);
 			}
 			else
-				ip->includes = empty;
+				ip->includes = wn_empty;
 
 			if ( *entry.wrappers)
 				ip->wrappers = entry.wrappers; 
@@ -267,7 +279,7 @@ Request	*ip;
 				ip->attrib2 &= ~(WN_LIST_INCL);
 			}
 			else
-				ip->wrappers = empty;
+				ip->wrappers = wn_empty;
 
 			ip->inclptr = ip->wrappers; 
 
@@ -279,15 +291,14 @@ Request	*ip;
 				ip->attrib2 |= WN_LIST_INCL;
 				/* shut off any includes */
 				ip->attrib2 &= ~(WN_INCLUDE + WN_WRAPPED);
-				ip->includes = ip->wrappers = empty;
+				ip->includes = ip->wrappers = wn_empty;
 			}
 			else
-				ip->list_incl = empty;
+				ip->list_incl = wn_empty;
 
 			ip->swrapper = entry.swrapper; 
 			ip->nomatchsub = entry.nomatchsub; 
 			ip->filter = entry.filter;
-			ip->handler = entry.handler;
 
 			ip->expires = entry.expires;
 			ip->maxage = ((*entry.maxage) 
@@ -298,20 +309,51 @@ Request	*ip;
 
 			if ( *ip->handler) 
 				if ( streq( ip->handler, "<none>")) {
-					ip->handler = empty;
+					ip->handler = wn_empty;
 					ip->attributes &= ~WN_CGI;
 				}
 				else {
-					ip->attributes |= WN_CGI;
+					ip->attributes |= 
+					(WN_CGI + WN_DYNAMIC +
+					 WN_NOCACHE + WN_NOSEARCH);
 					ip->type = RTYPE_CGI_HANDLER;
 				}
 
+			/* By default CGI is dynamic non-cachable */
+			if ( (ip->type == RTYPE_CGI ) ||
+						(ip->type == RTYPE_NPH_CGI ))
+				ip->attributes |= 
+					(WN_CGI + WN_DYNAMIC +
+					 WN_NOCACHE + WN_NOSEARCH);
+
+			/* But it can be non-dynamic  */
+			if ( ip->attributes & WN_NONDYNAMIC)
+				ip->attributes &= ~(WN_DYNAMIC);
+
+			if ( ip->attributes & WN_CACHEABLE)
+				ip->attributes &= ~(WN_NOCACHE);
+
+			if ( ip->attributes & WN_CGI ) {
+				ip->allowed |= (WN_M_GET + WN_M_POST);
+			}
+
+			if ( ip->attributes & WN_POST_ONLY ) {
+				ip->allowed |= WN_M_POST;
+				ip->allowed &= ~WN_M_GET;
+			}
+
+			if ( ip->attributes & WN_NO_POST)
+				ip->allowed &= ~WN_M_POST;
+
+			if ( ip->attributes & WN_PUT_OK)
+				ip->allowed |= WN_M_PUT;
+
 			ip->filter = ((*entry.filter) 
 				? entry.filter : dir_p->def_filter);
 
 			if ( *ip->filter) 
 				if ( streq( ip->filter, "<none>")) {
-					ip->filter = empty;
+					ip->filter = wn_empty;
 					ip->attributes &= ~WN_FILTERED;
 				}
 				else {
@@ -338,32 +380,6 @@ Request	*ip;
 			if ( ip->attributes & WN_NOPARSE)
 				ip->attributes &= ~(WN_PARSE);
 
-			/* By default CGI is dynamic non-cachable */
-			if ( (ip->type == RTYPE_CGI ) ||
-						(ip->type == RTYPE_NPH_CGI ))
-				ip->attributes |= (WN_DYNAMIC + WN_NOCACHE);
-
-			/* But it can be non-dynamic if dynamic = false */
-			if ( ip->attributes & WN_NONDYNAMIC)
-				ip->attributes &= ~(WN_DYNAMIC);
-
-			if ( ip->attributes & WN_CACHEABLE)
-				ip->attributes &= ~(WN_NOCACHE);
-
-			if ( *(entry.dynamic)){
-				if ( streq(entry.dynamic, "true"))
-					ip->attributes |= WN_DYNAMIC;
-				else
-					ip->attributes &= ~(WN_DYNAMIC);
-			}
-
-			if ( *(entry.cgi)){
-				if ( streq(entry.cgi, "true"))
-					ip->attributes |= WN_CGI;
-				else
-					ip->attributes &= ~(WN_CGI);
-			}
-
 			if ( strncasecmp( ip->content_type, "text", 4) == 0 )
 				ip->filetype |= WN_TEXT;
 			if ( streq( ip->content_type, "text/html"))
@@ -372,6 +388,17 @@ Request	*ip;
 			if ( *entry.headerlines)
 				strcpy( outheadp->list, entry.headerlines);
 				/* both have size BIGLEN */
+
+			if ( *entry.status)
+				mystrncpy( outheadp->status, 
+						entry.status, SMALLLEN);
+
+			if ( *entry.md5) {
+				mystrncpy( outheadp->md5, "Content-MD5: ", 20);
+				mystrncat( outheadp->md5, entry.md5, TINYLEN + 2);
+				mystrncat( outheadp->md5, "\r\n", 3);
+			}
+
 			if ( *entry.redirect) {
 				char 	*cp;
 
@@ -388,25 +415,43 @@ Request	*ip;
 						*cp = '\0';
 				}
 			}
-			else if ( (ip->status & WN_CANT_STAT) &&
-						(*dir_p->filemod == '\0'))
-				ip->type = RTYPE_DENIED;
-
+			else if ( (ip->status & WN_CANT_STAT) ) {
+				if (*dir_p->filemod == '\0')
+					; /* filemod handles it */
+				else if ( (ip->allowed & WN_M_PUT) &&
+						(inheadp->method == PUT))
+					; /* put file may not exist */
+				else
+					ip->type = RTYPE_DENIED;
+			}
+			if ( strcasecmp( dir_p->authtype, "Digest") == 0 &&
+							!chkauth( ip)) {
+				ip->type = RTYPE_NO_AUTH;
+			}
 			fclose( fp);
 			return;
 		}
 	}
+
 #ifdef NO_SERVEALL
 	ip->type = RTYPE_DENIED;
 #else
-	if ( dir_p->attributes & WN_SERVEALL)
+	if ( dir_p->attributes & WN_SERVEALL) {
+		if ( strcasecmp( dir_p->authtype, "Digest") == 0 &&
+							!chkauth( ip)) {
+			ip->type = RTYPE_NO_AUTH;
+			fclose( fp);
+			return;
+		}
 		do_serveall( ip);
+	}
 	else
 		ip->type = RTYPE_DENIED;
 #endif
-
 	fclose( fp);
 	return;
+
+
 }
 
 
@@ -430,10 +475,13 @@ Request	*ip;
 
 	if ( !*(dir_p->default_content))
 		dir_p->default_content = DEFAULT_CONTENT_TYPE;
+	ip->allowed |= WN_M_GET + WN_M_HEAD + WN_M_TRACE + WN_M_OPTIONS;
 	ip->content_type = dir_p->default_content;
 	ip->maxage = dir_p->default_maxage;
 	ip->handler = dir_p->def_handler;
 	ip->filter = dir_p->def_filter;
+	if ( *ip->handler || *ip->filter)
+		ip->attributes |= WN_NOSEARCH;
 	ip->attributes |= dir_p->defattributes;
 
 	if ( *(dir_p->filemod))
@@ -451,7 +499,7 @@ Request	*ip;
 		ip->type = RTYPE_FILE;
 
 	if ( (ip->type == RTYPE_CGI ) || (ip->type == RTYPE_NPH_CGI ))
-		ip->attributes |= WN_DYNAMIC;
+		ip->attributes |= (WN_DYNAMIC + WN_NOSEARCH);
 
 	if ( (cp = strrchr( ip->basename, '.')) == NULL )
 		return;
@@ -470,6 +518,14 @@ Request	*ip;
 		strlower( suffix);
 		*cp = '.';
 	}
+
+#ifdef PARSE_EXT
+	if ( streq( suffix, PARSE_EXT) && !(ip->attributes  & WN_NOPARSE))
+		ip->attributes |= WN_PARSE;
+#endif
+	if ( streq( suffix, CGI_EXT + 1) )
+		ip->attributes |= WN_NOSEARCH;
+
 	get_mtype( suffix);
 	if ( strncasecmp( ip->content_type, "text", 4) == 0 )
 		ip->filetype |= WN_TEXT;
@@ -543,7 +599,8 @@ char	*cachepath,
 
 		strlower( cp);
 
-		if ( streq( cp, remaddr) || streq( cp, remotehost)) {
+		if ( streq(cp, this_conp->remaddr)
+				|| streq(cp, this_conp->remotehost)) {
 			match = TRUE;
 		}
 		else if ( (cp2 = strchr( cp, '/')) != NULL) {
@@ -552,11 +609,11 @@ char	*cachepath,
 		}
 		else {
 			if ( (cp[len - 1] == '.') && 
-					( strncmp( cp, remaddr, len) == 0 ))
+				( strncmp( cp, this_conp->remaddr, len) == 0 ))
 				match = TRUE;
 		}
 
-		if ( match || wild_match( remotehost, cp)) {
+		if ( match || wild_match( this_conp->remotehost, cp)) {
 			if ( notflag)
 				break;
 			else {
@@ -593,7 +650,7 @@ char	*net,
 			&ipnet[0], &ipnet[1], &ipnet[2], &ipnet[3]);
 	sscanf( mask, "%d.%d.%d.%d", 
 			&ipmask[0], &ipmask[1], &ipmask[2], &ipmask[3]);
-	sscanf( remaddr, "%d.%d.%d.%d", 
+	sscanf( this_conp->remaddr, "%d.%d.%d.%d", 
 			&iprem[0], &iprem[1], &iprem[2], &iprem[3]);
 
 	return ( (ipnet[0] == (ipmask[0] & iprem[0]))
@@ -675,7 +732,7 @@ Dir_info	*dep;
 
 	if ( strrchr( dep->dirline, '\n') == NULL) {
 		senderr( SERV_ERR, err_m[63], "");
-		wn_exit( 2);
+		wn_exit( 2);  /* senderr: SERV_ERR */
 	}
 
 	dep->attributes = dep->defattributes = 0;
@@ -903,13 +960,13 @@ char		*key;
 
 		if ((lfp = popen( dir_p->cachemod, "r")) == (FILE *)NULL ) {
 			senderr( SERV_ERR, err_m[40], dir_p->cachemod);
-			wn_exit( 2);
+			wn_exit( 2);  /* senderr: SERV_ERR */
 		}
 
 		if ( (c = getc( lfp)) == EOF ) {
 			senderr( SERV_ERR, err_m[43], dir_p->cachemod);
 			pclose( lfp);
-			wn_exit( 2);
+			wn_exit( 2);  /* senderr: SERV_ERR */
 		}
 		else
 			ungetc( c, lfp);
@@ -931,16 +988,17 @@ char		*key;
 
 	if ( strrchr( cp, '\n') == NULL) {
 		senderr( SERV_ERR, err_m[63], "");
-		wn_exit( 2);
+		wn_exit( 2);  /* senderr: SERV_ERR */
 	}
 
 	while ( *cp)
 		cp++;
-	cep->end = cep->basename = cep->title = cep->keywords = cep->dynamic
+	cep->end = cep->basename = cep->title = cep->keywords 
 	= cep->content = cep->encoding = cep->includes = cep->list_incl 
-	= cep->cgi = cep->wrappers = cep->swrapper = cep->nomatchsub
-	= cep->filter = cep->handler = cep->expires = cep->maxage
-	= cep->url = cep->redirect = cep->handler = cp;
+	= cep->wrappers = cep->swrapper = cep->nomatchsub
+	= cep->filter = cep->expires = cep->maxage
+	= cep->status = cep->md5 = cep->url = cep->redirect 
+	= cep->handler = cp;
 
 	for ( i = 0; i < NUMFIELDS; i++) {
 		cep->field[i] = cp;
@@ -976,11 +1034,15 @@ char		*key;
 		case '\n':
 			*cp = '\0';
 			setvalue( field, value, cp, cep);
-			return TRUE;
+			break;
 		default:
 			cp++;
 		}
 	}
+	cp = strrchr( cep->basename, '.');
+	if ( (cp && streq( cp, CGI_EXT)) || (this_rp->attrib2 & WN_ISACGIBIN)) {
+		cep->attributes |= WN_NOSEARCH;
+	}
 	return TRUE;
 }
 
@@ -992,7 +1054,10 @@ char	*field,
 Cache_entry	*cep;
 {
 	char	buf[SMALLLEN];
-	int	i;
+	int	i,
+		errflg;
+
+	errflg = 0;
 
 	if ( !*value) {
 		mystrncpy( buf, field, 28);
@@ -1010,17 +1075,14 @@ Cache_entry	*cep;
 	switch (*field) {
 	case 'a':		/* attributes */
 		cep->attributes |= (unsigned) atol( value);
+		if ( cep->attributes & WN_DYNAMIC) {
+			cep->attributes |= WN_NOSEARCH;
+		}
 		break;
 	case 'c':
-		if ( field[1] == 'g' )		/* cgi */
-			cep->cgi = value;
-		else
-			cep->content = value;
+		cep->content = value;
 		break;
 
-	case 'd':
-		cep->dynamic = value;
-		break;
 	case 'e':
 		switch( field[1]) {
 		case 'n':
@@ -1029,6 +1091,8 @@ Cache_entry	*cep;
 		case 'x':
 			cep->expires = value;
 			break;
+		default:
+			errflg++;
 		}
 		break;
 	case 'f':
@@ -1045,33 +1109,34 @@ Cache_entry	*cep;
 		case 't':		/* filter */
 			cep->filter = value;
 			break;
+		default:
+			errflg++;
 		}
 		break;
 	case 'h':
-		if ( streq( field, "handler"))
+		if ( streq( field, "handler")) {
 			cep->handler = value;
+			if ( !streq( value, "<none>")) {
+				cep->attributes |= WN_NOSEARCH;
+			}
+		}
+
 
 		else if ( streq( field, "header")) {
 			if ( strlen( cep->headerlines) + 
 						strlen( value) > BIGLEN ) {
 				senderr( SERV_ERR, err_m[70], value);
-				wn_exit( 2);
+				wn_exit( 2);  /* senderr: SERV_ERR */
 			}
 			strcat( cep->headerlines, value);
 			strcat( cep->headerlines, "\n");
 		}
+		else
+			errflg++;
 		break;
 	case 'i':
-		switch( field[1]) {
-		case 'n':
-			cep->includes = value;
-			cep->attrib2 |= WN_INCLUDE;
-			break;
-		case 'm':
-			if ( streq( value, "true"))
-				cep->filetype |= WN_IMAGEMAP;
-			break;
-		}
+		cep->includes = value;
+		cep->attrib2 |= WN_INCLUDE;
 		break;
 	case 'k':
 		cep->keywords = value;
@@ -1081,20 +1146,38 @@ Cache_entry	*cep;
 		cep->attrib2 |= WN_LIST_INCL;
 		break;
 	case 'm':
-		cep->maxage = value;
+		if ( streq( field, "md5"))
+			cep->md5 =  value;
+		else 	if ( streq( field, "maxage"))
+			cep->maxage = value;
+		else
+			errflg++;
 		break;
 	case 'n':
 		if ( *(field+2) == 'm')  /* nomatchsub */
 			cep->nomatchsub = value;
 		else if ( streq( value, "true")) /* nosearch=true */
 			cep->attributes |= WN_NOSEARCH;
+		else
+			errflg++;
 		break;
 	case 'r':
 		cep->redirect = value;
 		break;
 	case 's':
-		cep->swrapper = value;
-		cep->attributes |= WN_SWRAPPED;
+		if ( streq( field, "swrapper")) {
+			cep->swrapper = value;
+			cep->attrib2 |= WN_SWRAPPED;
+		}
+		else if ( streq( field, "status")) {
+			if ( strlen( value) > SMALLLEN ) {
+				senderr( SERV_ERR, err_m[70], value);
+				wn_exit( 2);  /* senderr: SERV_ERR */
+			}
+			cep->status = value;
+		}
+		else
+			errflg++;
 		break;
 	case 't':		/* title */
 		cep->title = value;
@@ -1108,8 +1191,11 @@ Cache_entry	*cep;
 		cep->attrib2 |= WN_WRAPPED;
 		break;
 	default:
-		logerr( err_m[6], field);
+		errflg++;
 	}
+
+	if ( errflg) 
+		logerr( err_m[6], field);
 }
 
 
@@ -1117,7 +1203,7 @@ Cache_entry	*cep;
  * check_perm( ip, buf)
  * This stats the file named in buf, then checks if the index.cache is
  * owned by trusted user or group (options -t & -T) or if the index.cache
- * owner must own the file ( option -u). If sends an error and quits
+ * owner must own the file ( option -u). It sends an error and quits
  * if permssion is not allowed.
  */
 
@@ -1139,9 +1225,8 @@ char	*buf;
 		}
 		else
 			return;
-
 	}
-	else if ( is_trusted( ip) )
+	else if ( IS_TRUSTED )
 		return;
 
 	if ( WN_OPT_U ) {
@@ -1155,14 +1240,14 @@ char	*buf;
 			return;
 		}
 
-		if (( ip->cache_uid == stat_buf.st_uid )
+		if (( dir_p->cache_uid == stat_buf.st_uid )
 				 || ( cache_id == stat_buf.st_uid )
 				 || ( cache_id == stat_buf.st_gid ))
 			return;
 	}
 		
 	senderr( DENYSTATUS, err_m[3], filebuf);
-	wn_exit( 2);
+	wn_exit( 2); /* senderr: DENYSTATUS */
 }
 
 
@@ -1172,13 +1257,14 @@ Request	*ip;
 {
 	
 #ifdef FORBID_CGI
+	ip->allowed &= ~(WN_M_POST + WN_M_PUT);
 	senderr( DENYSTATUS, err_m[4], ip->cachepath);
-	wn_exit( 2);
+	wn_exit( 2);  /* senderr: DENYSTATUS*/
 #else
 	if ((serv_perm & WN_FORBID_EXEC) ||
-			((serv_perm & WN_RESTRICT_EXEC) && !is_trusted( ip))) {
+			((serv_perm & WN_RESTRICT_EXEC) && !IS_TRUSTED)) {
 		senderr( DENYSTATUS, err_m[4], ip->cachepath);
-		wn_exit( 2);
+		wn_exit( 2);  /* senderr: DENYSTATUS*/
 	}
 #endif
 }
