@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/content.h
-    Version 1.17.8
+    Version 2.0.3
     
-    Copyright (C) 1995  <by John Franks>
+    Copyright (C) 1998  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,8 +22,6 @@
 */
 
 
-#define MAXMIME		(150)
-
 #ifdef WNDEX
 static char * 
 #else
@@ -34,27 +32,36 @@ WN_CONST
 mimelist[MAXMIME][2] = {
 		{ "html", "text/html"},
 		{ "htm", "text/html"},
-		{ "gif", "image/gif"},
 		{ "txt", "text/plain"},
-		{ "ps", "application/postscript"},
-		{ "jpeg", "image/jpeg"},
-		{ "jpg", "image/jpeg"},
-		{ "jpe", "image/jpeg"},
-		{ "tex", "text/plain"},
-		{ "css", "text/css"},
+		{ "au", "audio/basic"},
+		{ "snd", "audio/basic"},
+		{ "mid", "audio/midi"},
+		{ "midi", "audio/midi"},
+		{ "aif", "audio/x-aiff"},
+		{ "aiff", "audio/x-aiff"},
+		{ "aifc", "audio/x-aiff"},
+		{ "ra", "audio/x-pn-realaudio"},
+		{ "rm", "audio/x-pn-realaudio"},
+		{ "ram", "audio/x-pn-realaudio"},
+		{ "wav", "audio/x-wav"},
+		{ "qt", "video/quicktime"},
+		{ "mov", "video/quicktime"},
+		{ "mpeg", "video/mpeg"},
+		{ "mpg", "video/mpeg"},
+		{ "mpe", "video/mpeg"},
+		{ "mws", "application/maple-v-r4"},
 		{ "csm", "application/cu-seeme"},
 		{ "cu", "application/cu-seeme"},
 		{ "tsp", "application/dsptype"},
 		{ "xls", "application/excel"},
+		{ "hqx", "application/mac-binhex40"},
 		{ "doc", "application/msword"},
 		{ "dot", "application/msword"},
-		{ "wrd", "application/msword"},
 		{ "bin", "application/octet-stream"},
-		{ "class", "application/octet-stream"},
-		{ "ms", "application/octet-stream"},
 		{ "oda", "application/oda"},
 		{ "pdf", "application/pdf"},
 		{ "pgp", "application/pgp-signature"},
+		{ "ps", "application/postscript"},
 		{ "ai", "application/postscript"},
 		{ "eps", "application/postscript"},
 		{ "ppt", "application/powerpoint"},
@@ -63,24 +70,25 @@ mimelist[MAXMIME][2] = {
 		{ "wk", "application/x-123"},
 		{ "wz", "application/x-Wingz"},
 		{ "bcpio", "application/x-bcpio"},
-		{ "z", "application/x-compress"},
-		{ "Z", "application/x-compress"},
 		{ "cpio", "application/x-cpio"},
 		{ "csh", "application/x-csh"},
 		{ "deb", "application/x-debian-package"},
-		{ "com", "application/x-msdos-program"},
-		{ "exe", "application/x-msdos-program"},
-		{ "bat", "application/x-msdos-program"},
 		{ "dvi", "application/x-dvi"},
+		{ "pfa", "application/x-font"},
+		{ "pfb", "application/x-font"},
+		{ "gsf", "application/x-font"},
+		{ "pcf", "application/x-font"},
 		{ "gtar", "application/x-gtar"},
 		{ "tgz", "application/x-gtar"},
-		{ "gz", "application/x-gzip"},
 		{ "hdf", "application/x-hdf"},
 		{ "phtml", "application/x-httpd-php"},
 		{ "pht", "application/x-httpd-php"},
 		{ "php", "application/x-httpd-php"},
+		{ "php3", "application/x-httpd-php3"},
+		{ "phps", "application/x-httpd-php3-source"},
+		{ "php3p", "application/x-httpd-php3-preprocessed"},
+		{ "class", "application/x-java"},
 		{ "latex", "application/x-latex"},
-		{ "hqx", "application/mac-binhex40"},
 		{ "frm", "application/x-maker"},
 		{ "maker", "application/x-maker"},
 		{ "frame", "application/x-maker"},
@@ -88,10 +96,13 @@ mimelist[MAXMIME][2] = {
 		{ "fb", "application/x-maker"},
 		{ "book", "application/x-maker"},
 		{ "fbdoc", "application/x-maker"},
-		{ "mws", "application/maple-v-r4"},
 		{ "mif", "application/x-mif"},
+		{ "com", "application/x-msdos-program"},
+		{ "exe", "application/x-msdos-program"},
+		{ "bat", "application/x-msdos-program"},
 		{ "nc", "application/x-netcdf"},
 		{ "cdf", "application/x-netcdf"},
+		{ "o", "application/x-object"},
 		{ "pl", "application/x-perl"},
 		{ "pm", "application/x-perl"},
 		{ "sh", "application/x-sh"},
@@ -100,8 +111,14 @@ mimelist[MAXMIME][2] = {
 		{ "sv4crc", "application/x-sv4crc"},
 		{ "tar", "application/x-tar"},
 		{ "tcl", "application/x-tcl"},
+		{ "tex", "application/x-tex"},
+		{ "gf", "application/x-tex-gf"},
+		{ "pk", "application/x-tex-pk"},
 		{ "texinfo", "application/x-texinfo"},
 		{ "texi", "application/x-texinfo"},
+		{ "bak", "application/x-trash"},
+		{ "old", "application/x-trash"},
+		{ "sik", "application/x-trash"},
 		{ "t", "application/x-troff"},
 		{ "tr", "application/x-troff"},
 		{ "roff", "application/x-troff"},
@@ -111,19 +128,16 @@ mimelist[MAXMIME][2] = {
 		{ "ustar", "application/x-ustar"},
 		{ "src", "application/x-wais-source"},
 		{ "zip", "application/zip"},
-		{ "au", "audio/basic"},
-		{ "snd", "audio/basic"},
-		{ "aif", "audio/x-aiff"},
-		{ "aiff", "audio/x-aiff"},
-		{ "aifc", "audio/x-aiff"},
-		{ "ra", "audio/x-pn-realaudio"},
-		{ "ram", "audio/x-pn-realaudio"},
-		{ "wav", "audio/x-wav"},
+		{ "gif", "image/gif"},
 		{ "ief", "image/ief"},
+		{ "jpeg", "image/jpeg"},
+		{ "jpg", "image/jpeg"},
+		{ "jpe", "image/jpeg"},
 		{ "png", "image/png"},
 		{ "tiff", "image/tiff"},
 		{ "tif", "image/tiff"},
 		{ "ras", "image/x-cmu-raster"},
+		{ "bmp", "image/x-ms-bmp"},
 		{ "pnm", "image/x-portable-anymap"},
 		{ "pbm", "image/x-portable-bitmap"},
 		{ "pgm", "image/x-portable-graymap"},
@@ -132,25 +146,33 @@ mimelist[MAXMIME][2] = {
 		{ "xbm", "image/x-xbitmap"},
 		{ "xpm", "image/x-xpixmap"},
 		{ "xwd", "image/x-xwindowdump"},
-		{ "c", "text/plain"},
-		{ "h", "text/plain"},
-		{ "cc", "text/plain"},
-		{ "hh", "text/plain"},
-		{ "cpp", "text/plain"},
-		{ "hpp", "text/plain"},
+		{ "csv", "text/comma-separated-values"},
 		{ "rtx", "text/richtext"},
 		{ "tsv", "text/tab-separated-values"},
+		{ "hpp", "text/x-c++hdr"},
+		{ "hxx", "text/x-c++hdr"},
+		{ "hh", "text/x-c++hdr"},
+		{ "cpp", "text/x-c++src"},
+		{ "cxx", "text/x-c++src"},
+		{ "cc", "text/x-c++src"},
+		{ "h", "text/x-chdr"},
+		{ "c", "text/x-csrc"},
+		{ "java", "text/x-java"},
+		{ "moc", "text/x-moc"},
+		{ "p", "text/x-pascal"},
+		{ "pas", "text/x-pascal"},
 		{ "etx", "text/x-setext"},
+		{ "tcl", "text/x-tcl"},
+		{ "tk", "text/x-tcl"},
+		{ "tex", "text/x-tex"},
+		{ "ltx", "text/x-tex"},
+		{ "sty", "text/x-tex"},
+		{ "cls", "text/x-tex"},
 		{ "vcs", "text/x-vCalendar"},
 		{ "vcf", "text/x-vCard"},
 		{ "dl", "video/dl"},
 		{ "fli", "video/fli"},
 		{ "gl", "video/gl"},
-		{ "mpeg", "video/mpeg"},
-		{ "mpg", "video/mpeg"},
-		{ "mpe", "video/mpeg"},
-		{ "qt", "video/quicktime"},
-		{ "mov", "video/quicktime"},
 		{ "avi", "video/x-msvideo"},
 		{ "movie", "video/x-sgi-movie"},
 		{ "vrm", "x-world/x-vrml"},
