@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/csearch.c
-    Version 1.18.0
+    Version 2.0.4
     
-    Copyright (C) 1996  <by John Franks>
+    Copyright (C) 1996-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,8 +22,9 @@
 */
 
 #include "../config.h"
-#include <sys/types.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <sys/types.h>
 #include <string.h>
 #include "wn.h"
 #include "parse.h"
@@ -74,15 +75,16 @@ struct regprog	**rp,
 	if ( !*ip->query) {
 		bzero( (char *) outheadp, sizeof( Outheader));
 		ip->content_type = "text/html";
+		ip->status |= WN_HAS_BODY;
 		http_prolog( );
 		cp = ( *dir_p->dir_owner ? dir_p->dir_owner : MAINTAINER);
 		strcpy( owner, "<link rev=\"made\" href=\"");
 		mystrncat( owner, cp, SMALLLEN - 30);
 		strcat( owner, "\">\n");
-		sprintf(buf,"<html>\n<head>\n<title> %s </title>\n", 
+		sprintf(buf,"<html>\n<head>\n<title> %.100s </title>\n", 
 							search_m[1]);
 		send_text_line( buf);
-		sprintf(buf,"%s</head>\n<body>\n<h2>%s</h2>\n",
+		sprintf(buf,"%.100s</head>\n<body>\n<h2>%.100s</h2>\n",
 							owner, search_m[1]);
 		send_text_line( buf);
 		send_text_line( search_m[7]);
@@ -136,6 +138,7 @@ Request	*ip;
 		return;
 
 	*ip->length = '\0';
+
 	mystrncpy( cdirpath, ip->cachepath, MIDLEN);
 	if ( (cp = strrchr( cdirpath, '/')) != NULL)
 		*cp = '\0';
@@ -217,16 +220,17 @@ char	*str;
 		buf[BIGLEN + 2*SMALLLEN];
 
 	if ((iswndir( ip)) ?  isdirwrapped( dir_p) :
-			( ip->attributes & WN_SWRAPPED))
+			( ip->attrib2 & WN_SWRAPPED))
 		return;
 
+	ip->status |= WN_HAS_BODY;
 	http_prolog( );
 		
 	cp = ( *dir_p->dir_owner ? dir_p->dir_owner : MAINTAINER);
 	sprintf( owner, "<link rev=\"made\" href=\"%.200s\">\n", cp);
-	sprintf(buf,"<html>\n<head>\n<title> %s </title>\n", str);
+	sprintf(buf,"<html>\n<head>\n<title> %.500s </title>\n", str);
 	send_text_line( buf);
-	sprintf(buf,"%s</head>\n<body>\n<h2>%s</h2>\n", owner, str);
+	sprintf(buf,"%.100s</head>\n<body>\n<h2>%.500s</h2>\n", owner, str);
 	send_text_line( buf);
 }	
 
@@ -300,7 +304,7 @@ int	depth;
 	}
 	/* endrelpath now points AFTER the '/' at the end */
 
-	sprintf( cfpath, "%s%s", cdirbuf, cfname);
+	sprintf( cfpath, "%.500s%.100s", cdirbuf, cfname);
 
 	if ( (cfp = fopen( cfpath, "r")) == (FILE *) NULL ) {
 		logerr( err_m[41], cfpath);
@@ -341,8 +345,10 @@ int	depth;
 
 
 	while ( read_cache_file( cep, cfp, (char *) NULL)) {
+		/*
 		if ( cep->attributes & WN_NOSEARCH)
 			continue;
+		*/
 		if ( *(cep->redirect))
 			continue;
 
@@ -421,11 +427,10 @@ Request	*ip;
 Cache_entry	*cep;
 char		*srelpath;
 {
-	static int	first_matched_line = TRUE;
 	char		buf[BIGLEN];
 
-	if ( first_matched_line) {
-		first_matched_line = FALSE;
+	if ( !(ip->status & WN_MATCH_SENT)) {  /* first matching line */
+		ip->status |= WN_MATCH_SENT;
 		if ( isdirwrapped( dir_p)) {
 			do_swrap( ip);
 			if ( ip->status & WN_ERROR)
@@ -444,12 +449,12 @@ char		*srelpath;
 
 	}
 	if ( *cep->basename) {
-		sprintf( buf, "<li><a href=\"%s%s\"> %s </a>\n",
+		sprintf( buf, "<li><a href=\"%.200s%.500s\"> %.500s </a>\n",
 				srelpath, cep->basename, cep->title);
 		send_text_line( buf);
 	}
 	else {
-		sprintf( buf, "<li><a href=\"%s\"> %s </a>\n",
+		sprintf( buf, "<li><a href=\"%.400s\"> %.400s </a>\n",
 				cep->url, cep->title);
 		send_text_line( buf);
 	}
@@ -465,7 +470,6 @@ Request	*ip;
 	int	found = FALSE;
 
 	check_query( ip, &regp, &regp);
-
 	if ( ip->type == RTYPE_FINISHED)
 		return;
 
@@ -476,6 +480,7 @@ Request	*ip;
 	}
 
 	*ip->length = '\0';
+
 	search_prolog( ip, out_m[6]);
 
 	check_perm( ip, ip->filepath);
@@ -493,7 +498,7 @@ Request	*ip;
 		return;
 	}
 	
-	if ( ip->attributes & WN_SWRAPPED)
+	if ( ip->attrib2 & WN_SWRAPPED)
 		do_swrap( ip);
 	else
 		search_epilog( );
@@ -530,7 +535,7 @@ FILE	*fp;
 		*cp = '\0';
 		strlower( cp2);
 		if ( regfind( rp, cp2) ) {
-			if ( first && ip->attributes & WN_SWRAPPED) {
+			if ( first && ip->attrib2 & WN_SWRAPPED) {
 				do_swrap( ip);
 				if ( ip->status & WN_ERROR)
 					return FALSE;
