@@ -1,9 +1,10 @@
 /*
+
     Wn: A Server for the HTTP
     File: wn/evalif.c
-    Version 1.17.9
+    Version 2.0.3
     
-    Copyright (C) 1996-7  <by John Franks>
+    Copyright (C) 1996-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +24,7 @@
 
 #include "../config.h"
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include "wn.h"
 #include "access.h"
@@ -31,81 +33,318 @@
 #include "regi.h"
 
 static int	fieldnum = 0,
+		eval_ifterm(),
+		if_op(),
+		gettoken(),
 		chk_accessfile( ),
 		chk_file_match(),
 		chk_match();
 
 int
-eval_if( cp, pdp)
-char		*cp;
+do_ifexpr(  exprp,  pdp)
+char		**exprp;
 Parsedata	*pdp;
 {
 
-	while ( *cp && isspace( *cp))
-		cp++;
-	if ( strncasecmp( cp, "true ", 5) == 0 )
-		return (IF_TRUE);
-	if ( strncasecmp( cp, "false ", 6) == 0 )
-		return (IF_FALSE);
-	if ( strncasecmp( cp, "accessfile", 10) == 0 )
-		return (chk_accessfile( this_rp, cp));
+	char	token[SMALLLEN];
+	int	value,
+		val2,
+		notflg,
+		ttype;
 
-	if ( strncasecmp( cp, "accept", 6) == 0 )
-		return (chk_match( cp + 6, MATCH_ACCEPT, pdp));
 
-	if ( strncasecmp( cp, "cookie", 6) == 0 )
-		return (chk_match( cp + 6, MATCH_COOKIE, pdp));
+	value = IF_ERR;
 
-	if ( strncasecmp( cp, "ua", 2) == 0 )
-		return (chk_match( cp + 2, MATCH_UA, pdp));
+	ttype = gettoken( exprp, token, pdp, &notflg);
 
-	if ( strncasecmp( cp, "refer", 5) == 0 ) {
-		while ( isalpha( *cp)) /* get referer, and referrer */
-			cp++;
-		return (chk_match( cp, MATCH_REFERRER, pdp));
+	if ( ttype == IFCLAUSE_TERM) {
+		value = eval_ifterm ( token, exprp, pdp);
+		if ( notflg) {
+			if ( value == IF_TRUE)
+				value = IF_FALSE;
+			else if ( value == IF_FALSE)
+				value = IF_TRUE;
+		}
+		ttype = gettoken( exprp, token, pdp, &notflg);
+		if ( notflg)
+			return IF_ERR;
 	}
 
-	if ( strncasecmp( cp, "host_header", 11) == 0 )
-		return (chk_match( cp + 11, MATCH_HOST_HEAD, pdp));
+	switch ( ttype) {
+	case IFCLAUSE_ERR:
+		return IF_ERR;
 
-	if ( strncasecmp( cp, "query", 5) == 0 )
-		return (chk_match( cp + 5, MATCH_QUERY, pdp));
+	case IFCLAUSE_TERM:
+	case IFCLAUSE_END:
+		return value;
 
-	if ( strncasecmp( cp, "param_field", 11) == 0 )
-		return (chk_match( cp + 11, MATCH_PARAM_FIELD, pdp));
+	case IFCLAUSE_AND:
+		val2 = do_ifexpr( exprp, pdp);
+		return (if_op( value, val2, IFCLAUSE_AND));
 
-	if ( strncasecmp( cp, "param_value", 11) == 0 )
-		return (chk_match( cp + 11, MATCH_PARAM_VALUE, pdp));
+	case IFCLAUSE_OR:
+		val2 = do_ifexpr( exprp, pdp);
+		return (if_op( value, val2, IFCLAUSE_OR));
+		
+	}
+	return 	IF_ERR;
+}
 
-	if ( strncasecmp( cp, "request", 7) == 0 )
-		return (chk_match( cp + 7, MATCH_REQUEST, pdp));
+/*
+ * static int if_op( val1, val2, op)
+ * op must be IFCLAUSE_AND or IFCLAUSE_OR.  val1 and val2 must
+ * be IF_TRUE, IF_FALSE, or IF_ERR.
+ */
 
-	if ( strncasecmp( cp, "hostname", 8) == 0 )
-		return (chk_match( cp + 8, MATCH_HOST, pdp));
+static int
+if_op( val1, val2, op)
+int	val1,
+	val2,
+	op;
+{
 
-	if ( strncasecmp( cp, "ip", 2) == 0 )
-		return (chk_match( cp + 2, MATCH_IP, pdp));
+	switch (val1) {
+	case IF_TRUE:
+		val1 = TRUE;
+		break;
+	case IF_FALSE:
+		val1 = FALSE;
+		break;
+	default:
+		return IF_ERR;
+	}
 
-	if ( strncasecmp( cp, "before", 6) == 0 )
-		return ( date_cmp( this_rp, cp + 6, FALSE) ? 
+	switch (val2) {
+	case IF_TRUE:
+		val2 = TRUE;
+		break;
+	case IF_FALSE:
+		val2 = FALSE;
+		break;
+	default:
+		return IF_ERR;
+	}
+
+	switch ( op) {
+	case IFCLAUSE_AND:
+		return ( (val1 & val2) ? IF_TRUE : IF_FALSE);
+	case IFCLAUSE_OR:
+		return ( (val1 | val2) ? IF_TRUE : IF_FALSE);
+	}
+	return IF_ERR;
+}
+		
+static int
+gettoken( exprp, tokstart, pdp, flg)
+char	**exprp,
+	*tokstart;
+Parsedata	*pdp;
+int	*flg;
+{
+	char	*tok,
+		*cp;
+	int	val,
+		type;
+
+	tok = tokstart;
+	*flg = FALSE;
+	cp = *exprp;
+
+	while ( isspace( *cp))
+		cp++;
+
+	if ( strncmp( cp , "-->", 3) == 0 ) {	/* <!-- something --> */
+		mystrncpy( tokstart, cp, 4);
+		*exprp = cp + 3;
+		return IFCLAUSE_END;
+	}
+
+	if ( *cp == '>' ) {	/* <?WN something > */
+		mystrncpy( tokstart, cp, 2);
+		*exprp = ++cp;
+		return IFCLAUSE_END;
+	}
+
+	if ( strncmp( cp , "&&", 2) == 0 ) {
+		mystrncpy( tokstart, cp, 3);
+		*exprp = cp + 2;
+		return IFCLAUSE_AND;
+	}
+
+	if ( strncmp( cp , "||", 2) == 0 ) {
+		mystrncpy( tokstart, cp, 3);
+		*exprp = cp + 2;
+		return IFCLAUSE_OR;
+	}
+
+	type = IFCLAUSE_TERM;
+
+	if ( *cp == '!' ) {
+		*exprp = ++cp;
+		while ( isspace( *cp))
+			cp++;
+		*flg = TRUE;
+	}
+
+	if ( *cp == '(' ) {
+		*exprp = ++cp;
+		val = do_ifexpr( exprp, pdp);
+
+		switch ( val) {
+		case IF_TRUE:
+			strcpy( tokstart, "true");
+			break;
+		case IF_FALSE:
+			strcpy( tokstart, "false");
+			break;
+		case IF_ERR:
+			return IFCLAUSE_ERR;
+		}
+
+		cp = *exprp;
+		while ( isspace( *cp))
+			cp++;
+		if ( *cp++ == ')' ) {
+			*exprp = cp;
+			return IFCLAUSE_TERM;
+		}
+		else
+			return IFCLAUSE_ERR;
+	}
+
+	while ( *cp && ( tok - tokstart < SMALLLEN) ) {
+		int	end = FALSE;
+		switch ( *cp) {
+		case '&':
+			if ( *(cp+1) == '&') {
+				*exprp = cp;
+				end = TRUE;
+			}
+			else
+				*tok++ = *cp++;
+			break;
+		case '|':
+			if ( *(cp+1) == '|') {
+				*exprp = cp;
+				end = TRUE;
+			}
+			else
+				*tok++ = *cp++;
+			break;
+
+		case '-':
+			if ( ( *(cp+1) == '-') && (*(cp+2) == '>') ) {
+				*exprp = cp;
+				end = TRUE;
+			}
+			else
+				*tok++ = *cp++;
+			break;
+
+		case ')':
+		case '>':
+			*exprp = cp;
+			end = TRUE;
+			break;
+		default:
+			*tok++ = *cp++;
+			break;
+		}
+
+		if ( end == TRUE )
+			break;
+	}
+
+	*tok-- = '\0';
+	while (  (tok >= tokstart) && isspace( *tok) )
+		tok--; /* eliminate trailing whitespace */
+	*++tok = '\0';
+
+	if ( !*cp)
+		return IFCLAUSE_ERR;
+	return (type);
+}
+
+
+
+
+/*
+ * static int eval_ifterm( token, exprp, pdp)
+ *  returns IF_TRUE, IF_FALSE, or IF_ERR 
+ */
+
+static int
+eval_ifterm( token, exprp, pdp)
+char   *token,
+       **exprp;
+Parsedata *pdp;
+{
+	if ( strncasecmp( token, "true", 4) == 0 )
+		return (IF_TRUE);
+	if ( strncasecmp( token, "false", 6) == 0 )
+		return (IF_FALSE);
+	if ( strncasecmp( token, "accessfile", 10) == 0 )
+		return (chk_accessfile( this_rp, token));
+
+	if ( strncasecmp( token, "accept", 6) == 0 )
+		return (chk_match( token + 6, MATCH_ACCEPT, pdp));
+
+	if ( strncasecmp( token, "cookie", 6) == 0 )
+		return (chk_match( token + 6, MATCH_COOKIE, pdp));
+
+	if ( strncasecmp( token, "ua", 2) == 0 )
+		return (chk_match( token + 2, MATCH_UA, pdp));
+
+	if ( strncasecmp( token, "refer", 5) == 0 ) {
+		while ( isalpha( *token)) /* get referer, and referrer */
+			token++;
+		return (chk_match( token, MATCH_REFERRER, pdp));
+	}
+
+	if ( strncasecmp( token, "host_header", 11) == 0 )
+		return (chk_match( token + 11, MATCH_HOST_HEAD, pdp));
+
+	if ( strncasecmp( token, "query", 5) == 0 )
+		return (chk_match( token + 5, MATCH_QUERY, pdp));
+
+	if ( strncasecmp( token, "param_field", 11) == 0 )
+		return (chk_match( token + 11, MATCH_PARAM_FIELD, pdp));
+
+	if ( strncasecmp( token, "param_value", 11) == 0 )
+		return (chk_match( token + 11, MATCH_PARAM_VALUE, pdp));
+
+	if ( strncasecmp( token, "request", 7) == 0 )
+		return (chk_match( token + 7, MATCH_REQUEST, pdp));
+
+	if ( strncasecmp( token, "hostname", 8) == 0 )
+		return (chk_match( token + 8, MATCH_HOST, pdp));
+
+	if ( strncasecmp( token, "ip", 2) == 0 )
+		return (chk_match( token + 2, MATCH_IP, pdp));
+	
+	if ( strncasecmp( token, "language", 8) == 0 )
+		return (chk_match( token + 8, MATCH_LANGUAGE, pdp));
+
+	if ( strncasecmp( token, "before", 6) == 0 )
+		return ( date_cmp( this_rp, token + 6, FALSE) ? 
 				IF_FALSE : IF_TRUE);
 
-	if ( strncasecmp( cp, "after", 5) == 0 )
-		return ( date_cmp( this_rp, cp + 5, FALSE) ? 
+	if ( strncasecmp( token, "after", 5) == 0 )
+		return ( date_cmp( this_rp, token + 5, FALSE) ? 
 				IF_TRUE : IF_FALSE);
 
-	if ( strncasecmp( cp, "field", 5) == 0 )  {
+	if ( strncasecmp( token, "field", 5) == 0 )  {
 		/* Get the field number */
-                cp += 5;
-                while ( isspace( *cp) || (*cp == '#'))
-                        cp++;
+                token += 5;
+                while ( isspace( *token) || (*token == '#'))
+                        token++;
 
-		fieldnum = atoi( cp);
-		return (chk_match( cp, MATCH_FIELD, pdp));
+		fieldnum = atoi( token);
+		return (chk_match( token, MATCH_FIELD, pdp));
 	}
 
-	parse_html_err( NULLTOKEN, pdp);
-	return (NULLTOKEN);
+	parse_html_err( TEXT_TOKEN, pdp);
+	return (TEXT_TOKEN);
 }
 
 
@@ -216,11 +455,11 @@ Parsedata	*pdp;
 		val = regfind( rp, this_rp->request);
 		break;
 	case MATCH_IP:
-		val = regfind( rp, remaddr);
+		val = regfind( rp, this_conp->remaddr);
 		break;
 	case MATCH_HOST:
 		get_remote_info();
-		val = regfind( rp, remotehost);
+		val = regfind( rp, this_conp->remotehost);
 		break;
 	case MATCH_QUERY:
 		val = regfind( rp, this_rp->query);
@@ -234,8 +473,11 @@ Parsedata	*pdp;
 	case MATCH_FIELD:
 		val = regfind( rp, this_rp->field[fieldnum]);
 		break;
+	case MATCH_LANGUAGE:
+		val = regfind( rp, inheadp->lang);
+		break;		
 	default:
-		parse_html_err( NULLTOKEN, pdp);
+		parse_html_err( TEXT_TOKEN, pdp);
 		return (IF_FALSE);
 	}
 	if ( notflg )
@@ -277,7 +519,7 @@ Parsedata 	*pdp;
 		item = inheadp->referrer;
 		break;
 	case MATCH_IP:
-		item = remaddr;
+		item = this_conp->remaddr;
 		break;
 	case MATCH_QUERY:
 		item = this_rp->query;
@@ -293,13 +535,16 @@ Parsedata 	*pdp;
 		break;
 	case MATCH_HOST:
 		get_remote_info();
-		item = remotehost;
+		item = this_conp->remotehost;
 		break;
 	case MATCH_FIELD:
 		item = this_rp->field[fieldnum];
 		break;
+	case MATCH_LANGUAGE:
+		item = inheadp->lang;
+		break;
 	default:
-		parse_html_err( NULLTOKEN, pdp);
+		parse_html_err( TEXT_TOKEN, pdp);
 		return (IF_ERR);
 	}
 
@@ -356,3 +601,5 @@ Parsedata 	*pdp;
 	fclose( fp);
 	return IF_FALSE;
 }
+
+
