@@ -23,9 +23,13 @@
 
 /* Any extra headers go here */
 
+
 #define WN_EXTRA_INIT
 
 #define WN_EXTRA_OPTS
 
 #define WN_EXTRA_SETUP
 
+#ifndef USE_WN_READ
+#define WN_read(a,b,c)		read(a,b,c)
+#endif
