@@ -1,7 +1,7 @@
 /*
     Wn: A Server for the HTTP
     File: wn/gsearch.c
-    Version 1.18.2
+    Version 2.0.4
     
     Copyright (C) 1996-7  <by John Franks>
 
@@ -224,10 +224,12 @@ Parsedata	*pdp;
 	int		token;
 
 	char	linebuf[MIDLEN],
+		svbuf[MIDLEN],
 		dummy,
 		*cp,
 		*startmatch,
-		*endmatch;
+		*endmatch,
+		*line;
 
 
 	searchoff = FALSE;
@@ -235,12 +237,20 @@ Parsedata	*pdp;
 
 	if ( wrapped)
 		ok2mark = TRUE;
-	while ( fgets( linebuf, MIDLEN, fp)) {
+	svbuf[0] = '\0';
+	while ( TRUE) {
+		if ( ishtml )
+			line = get_parse_line( linebuf, svbuf, MIDLEN, fp);
+		else
+			line = fgets( linebuf,  MIDLEN, fp);
+
+		if ( !line)
+			break;
 		curr_line_num++;
 		pdp->currline++;
 		if ( ishtml ) {
 			token = get_parse_token( linebuf, &dummy, pdp);
-			if ( token != NULLTOKEN) {
+			if ( token != TEXT_TOKEN) {
 				switch ( token) {
 				case SEARCH_ON:
 					searchoff = FALSE;
@@ -349,7 +359,7 @@ Parsedata	*pdp;
 		pdp->currline++;
 		if ( ishtml) {
 			token = get_parse_token( linebuf, &dummy, pdp);
-			if ( token != NULLTOKEN) {
+			if ( token != TEXT_TOKEN) {
 				switch ( token) {
 				case SEARCH_ON:
 					searchoff = FALSE;
@@ -380,14 +390,13 @@ Request	*ip;
 char	*name,
 	*title;
 {
-	static int	first_matched_line = TRUE;
 	char		buf[BIGLEN];
 
-	if ( first_matched_line) {
-		first_matched_line = FALSE;
+	if ( !(ip->status & WN_MATCH_SENT)) {  /* first matching line */
+		ip->status |= WN_MATCH_SENT;
 		if ( iswndir( ip) && dir_p->attributes & WN_DIRWRAPPED)
 			do_swrap( ip);
-		else if ( ip->attributes & WN_SWRAPPED)
+		else if ( ip->attrib2 & WN_SWRAPPED)
 			do_swrap( ip);
 		else {
 			send_text_line( search_m[4]);
@@ -411,7 +420,7 @@ char	*name,
 				send_text_line("\t</ul>\n");
 			mystrncpy( prev_filep, name, MIDLEN);
 			sprintf( buf,
-				"<li><b>Title:</b> <a href=\"%s\">%s</a>\n",
+				"<li><b>Title:</b> <a href=\"%.200s\">%.500s</a>\n",
 					name, title);
 			send_text_line( buf);
 			send_text_line(
@@ -420,7 +429,7 @@ char	*name,
 		}
 	}
 	else {
-		sprintf( buf, "<li><a href=\"%s\">%s</a>\n", name, title);
+		sprintf( buf, "<li><a href=\"%.200s\">%.500s</a>\n", name, title);
 		send_text_line( buf);
 	}
 }
@@ -445,12 +454,12 @@ char	*base;
 	strcpy( buf, "\t<li> ");
 	cp += 6;
 	if ( ok2mark && ishtml)
-		sprintf( mline, ";mark=%d,%d,%d#%s", curr_line_num,
+		sprintf( mline, ";mark=%d,%d,%d#%.32s", curr_line_num,
 				begin_offset, end_offset, WN_HTML_MARK);
 
 	cp = remove_tags( cp, curr_linep, beginmatch);
 
-	sprintf( cp, "<a href=\"%s%s\">",
+	sprintf( cp, "<a href=\"%.400s%.200s\">",
 		 base, mline);
 
 	while ( *cp)
@@ -559,10 +568,22 @@ Request	*ip;
 	ishtml = (ip->filetype & WN_ISHTML);
 
 	check_perm( ip, ip->filepath);
-        if ( (gfp = fopen( ip->filepath, "r")) == (FILE *) NULL ) {
+
+	if ( ip->attributes & WN_FILTERED) {
+		char commandbuf[SMALLLEN];
+		mystrncpy( commandbuf, ip->filter, SMALLLEN);
+		strcat( commandbuf, " < ");
+		strcat( commandbuf, ip->filepath);
+		if ((gfp = popen( commandbuf, "r")) == (FILE *)NULL) {
+		senderr( SERV_ERR, err_m[1], ip->relpath);
+		return;
+		}
+	}
+        else if ( (gfp = fopen( ip->filepath, "r")) == (FILE *) NULL ) {
 		senderr( SERV_ERR, err_m[1], ip->relpath);
 		return;
 	}
+
 	curr_line_num = 0;
 	if ( ip->attributes & WN_PARSE )
 		reset_parse_err( ip->filepath, 0, &pdata);
@@ -578,7 +599,10 @@ Request	*ip;
 	else
 		found = file_grep( ip, (ishtml ? htmlregp : regp), gfp);
 
-	fclose( gfp);
+	if ( ip->attributes & WN_FILTERED)
+		pclose( gfp);
+	else
+		fclose( gfp);
 
 	if ( !found) {
 		send_nomatch( ip, 'f');
@@ -588,7 +612,7 @@ Request	*ip;
 	if ( ip->type == RTYPE_CONTEXTSEARCH)
 		send_text_line( "\t</ul>\n</ul>\n");
 
-	if ( ip->attributes & WN_SWRAPPED)
+	if ( ip->attrib2 & WN_SWRAPPED)
 		do_swrap( ip);
 	else
 		search_epilog( );
@@ -612,7 +636,7 @@ FILE	*fp;
 		strcpy( curr_linep, linebuf);
 		strlower( linebuf);
 		if ( regfind( rp, linebuf) ) {
-			if ( first && ip->attributes & WN_SWRAPPED) {
+			if ( first && ip->attrib2 & WN_SWRAPPED) {
 				do_swrap( ip);
 				if ( ip->status & WN_ERROR)
 					return FALSE;
