@@ -1,7 +1,7 @@
 /*
     Wn: A Server for the HTTP
     File: wn/image.c
-    Version 1.18.0
+    Version 1.19.10
     
     Copyright (C) 1995  <by John Franks>
 
@@ -22,6 +22,7 @@
 */
 
 #include "../config.h"
+#include <string.h>
 #include "wn.h"
 #include "image.h"
 #define HIT_IT	(111)	/* anything bigger than 2 works fine here */
@@ -152,7 +153,7 @@ image( )
 		return;
 
 	if ( *default_url && !res )
-		dolocation( default_url, this_rp);
+		dolocation( default_url, this_rp, 302);
 	else if ( !res) {
 		if ( !coords_ok)
 			senderr( SERV_ERR, IMAGE_ERR1, this_rp->query);
@@ -243,7 +244,7 @@ char	*url,
 
 	if ( (x1 <= xclick) && (x2 >= xclick) &&
 				(y1 <= yclick) && (y2 >= yclick) ) {
-		dolocation( url, this_rp);
+		dolocation( url, this_rp, 302);
 		return TRUE;
 	}
 	return FALSE;
@@ -281,7 +282,7 @@ char	*url,
 			(y1 - yclick) * (y1 - yclick));
 
 	if ( dist <= radius) {
-		dolocation( url, this_rp);
+		dolocation( url, this_rp, 302);
 		return TRUE;
 	}
 	return FALSE;
@@ -358,13 +359,13 @@ char	*url,
  * initializes the previous point in segment().
  */
 	if ( segment( x - xclick, y - yclick) == HIT_IT) {
-		dolocation( url, this_rp);  /* clicked on a vertex */
+		dolocation( url, this_rp, 302);  /* clicked on a vertex */
 		return TRUE;
 	}
 
 	while ( (cp = getcoords( &x, &y, cp)) != NULL) {
 		if ( (cross = segment( x - xclick, y - yclick)) == HIT_IT ) {
-			dolocation( url, this_rp); /* clicked on a vertex */
+			dolocation( url, this_rp, 302); /* clicked on vertex */
 			return TRUE;
 		}
 		crossnum += cross;
@@ -374,12 +375,12 @@ char	*url,
 		return FALSE;
 
 	if ( (cross = segment( firstx - xclick, firsty - yclick)) == HIT_IT ) {
-		dolocation( url, this_rp); /* clicked on a vertex */
+		dolocation( url, this_rp, 302); /* clicked on a vertex */
 		return TRUE;
 	}
 	crossnum += cross;
 	if ( crossnum != 0 ) {
-		dolocation( url, this_rp);
+		dolocation( url, this_rp, 302);
 		return TRUE;
 	}
 	return FALSE;
