@@ -1,7 +1,7 @@
 /*
     Wn: A Server for the HTTP
     File: wn/init.c
-    Version 1.18.7
+    Version 2.0.4
     
     Copyright (C) 1995-8  <by John Franks>
 
@@ -23,10 +23,12 @@
 
 #include "../config.h"
 #include <stdio.h>
+#include <stdlib.h>
 #include <sys/types.h>
-#include <sys/file.h>
 #include <sys/stat.h>
+#include <sys/file.h>
 #include <sys/signal.h>
+#include <sys/socket.h>
 #include <string.h>
 #include <errno.h>
 #include <netdb.h>
@@ -46,7 +48,7 @@
 #include "content.h"
 
 #ifdef USE_VIRTUAL_HOSTS
-extern char	*vhostlist[][3];
+extern char	*vhostlist[][4];
 #endif
 
 /* this may be needed if unistd.h isn't included... */
@@ -58,6 +60,14 @@ extern char	*vhostlist[][3];
 extern void	get_rfc931();
 #endif
 
+#ifdef STANDALONE
+#define PRINT_ERROR( x, y)	fprintf( stderr, x, y);
+#define PRINT1_ERROR( x)	fprintf( stderr, x);
+#else
+#define PRINT_ERROR( x, y)	printf( x, y);
+#define PRINT1_ERROR( x)	printf( x);
+#endif
+
 #ifdef NO_VOID_PTR
 #define	void	int
 #endif
@@ -66,6 +76,8 @@ extern void	get_rfc931();
 extern long	timezone;
 #endif
 
+extern void	end_chunking();
+
 extern char	*optarg;
 
 extern int	optind;
@@ -74,31 +86,38 @@ extern int	optind;
 #define		WN_VERBOSE	(1)
 #define		NCSA_LOG	(2)
 
+#ifdef SET_UID
+#ifndef USERID
+#define USERID	(-2)
+#define GROUPID	(-2)
+#endif
+#endif
+
 #ifdef VERBOSELOG
 static int	log_type = WN_VERBOSE;
 #else
 static int	log_type = WN_COMMON;
 #endif
 
-
 static void	dump_log();
 
 char	rootdir[SMALLLEN],
 	wn_tmpdir[sizeof(TEMPDIR) + 20] = {TEMPDIR},
+	wn_empty[] = { "" },	/* safe place to park pointers */
 	wnlogfile[SMALLLEN],
 	errlogfile[SMALLLEN],
 	pid_file[SMALLLEN],
 	cfname[SMALLLEN],
-	empty[1],	/* safe place to park pointers */
 	hostname[MAXHOSTNAMELEN],
-	remotehost[MAXHOSTNAMELEN],
-	remaddr[20],
 	**mtypelist = NULL,
 	**suflist = NULL;
 
 
 int	admin_mode = FALSE,
-	debug_log = FALSE;
+	debug_log = FALSE,
+ 	user_id = USERID,
+	group_id = GROUPID;
+
 
 unsigned	serv_perm = 0,
 		interface_num,
@@ -116,10 +135,12 @@ static void	locklog(),
 		unlocklog();
 #endif
 
-static void	log_logfile(),
-		start_log(),
+#ifdef STANDALONE
+static void	start_log(),
 		restart();
+#endif
 
+static void	log_logfile();
 
 static FILE	*logfp = NULL,
 		*errlogfp = NULL;
@@ -138,26 +159,7 @@ char	*argv[];
 	int	c,
 		errflg = 0;
 
-/*
- * Let's run as specified user and group if SET_UID defined.
- * This is imperative if inetd cannot change UID from root.
- */
 
-#ifdef SET_UID
-#ifndef USERID
-#define USERID	(-2)
-#define GROUPID	(-2)
-#endif
-
-#ifndef STANDALONE  /* Otherwise this is done in standalone.c */
- 	setegid( (gid_t) GROUPID );
- 	setgid(  (gid_t) GROUPID );   
- 	seteuid( (uid_t) USERID );
- 	setuid(  (uid_t) USERID );  /* This pulls rug out from under us */
-#endif
-#endif
-
-	empty[0] = '\0';
 	port = DEFAULT_PORT;
 	mystrncpy (wnlogfile, WN_LOGFILE, SMALLLEN);
 	mystrncpy (errlogfile, WN_ERRLOGFILE, SMALLLEN);
@@ -180,7 +182,7 @@ char	*argv[];
 #endif
 
 	while ((c = getopt(argc, argv, 
-				"a:A:eEh:L:l:p:q:St:T:uv:V:dxz:")) != -1) {
+				"a:A:eEh:L:l:n:N:p:q:St:T:uv:V:dxz:")) != -1) {
 		switch ((char) c) {
 			case 'a':
 				acache_id = (unsigned) atoi( optarg);
@@ -208,6 +210,12 @@ char	*argv[];
 				mystrncpy( errlogfile, optarg, SMALLLEN);
 				break;
 #ifdef STANDALONE
+			case 'n':
+				user_id = (unsigned) atoi( optarg);
+				break;
+			case 'N':
+				group_id = (unsigned) atoi( optarg);
+				break;
 			case 'p':
 				port = atoi( optarg);
 				break;
@@ -225,6 +233,7 @@ char	*argv[];
 				exit( 2);
 #else
 				wnlogfile[0] = '\0';
+				errlogfile[0] = '\0';
 				log_func = LSYSLOG;
 #endif /* NO_SYSLOG */
 				break;
@@ -252,8 +261,7 @@ char	*argv[];
 					log_type = NCSA_LOG;
 					break;
 				}
-				fprintf( stderr, "Unknown log type %s\n", 
-					optarg);
+				PRINT_ERROR( "Unknown log type %.20s\n", optarg);
 				break;
 #ifdef USE_VIRTUAL_HOSTS
 #ifdef VIRTUAL_HOSTS_FILE
@@ -279,16 +287,34 @@ char	*argv[];
 	}
 	if (errflg) {
 #ifdef STANDALONE
-		fprintf( stderr, "Usage: wn [-L logfile | -S] ");
-		fprintf( stderr, "[-a uid |-A gid]\n");
-		fprintf( stderr, "[-e | -E] [-p port] [-t uid | -T gid ] ");
-		fprintf( stderr, "[-v log_type] [-V virtual host file]\n");
-		fprintf( stderr, "[-u] [-q pid_file] [-h host] [topdir]\n");
+		PRINT1_ERROR("Usage: wn [-L logfile | -S] [-a uid |-A gid]\n");
+		PRINT1_ERROR( "[-e | -E] [-p port] [-t uid | -T gid ] ");
+		PRINT1_ERROR( "[-v log_type] [-V virtual host file]\n");
+		PRINT1_ERROR( "[-u] [-q pid_file] [-h host] [topdir]\n");
 #else
-		fprintf( stderr, "Unknown option given to server");
+		PRINT1_ERROR( "Unknown option given to server");
 #endif
 		exit (2);
 	}
+
+
+#ifdef SET_UID
+#ifndef STANDALONE  /* Otherwise this is done in standalone.c */
+/*
+ * Let's run as specified user and group if SET_UID defined.
+ * This is only used with inetd and if inetd cannot change UID from root.
+ * (i.e. Ultrix)
+ */
+
+ 	setegid( (gid_t) group_id );
+ 	setgid(  (gid_t) group_id );   
+ 	seteuid( (uid_t) user_id );
+ 	setuid(  (uid_t) user_id );  /* This pulls rug out from under us */
+#endif
+#endif
+
+
+
 	if ( *wnlogfile && !*errlogfile) {
 		strcpy( errlogfile, wnlogfile);
 	}
@@ -345,6 +371,7 @@ open_wnlog( logfile, errlog)
 char	*logfile,
 	*errlog;
 {
+
 #ifndef FORBID_CGI
 	char		buf[TINYLEN];
 	struct stat	stat_buf;
@@ -355,24 +382,21 @@ char	*logfile,
 	strcat( wn_tmpdir, buf);
 	if ( lstat( wn_tmpdir, &stat_buf) == -1) {
 		if ( (errno != ENOENT) || (mkdir( wn_tmpdir, 0700) != 0) ) {
-			fprintf( stderr, err_m[0], wn_tmpdir);
+			PRINT_ERROR( err_m[122], wn_tmpdir);
 			exit( 2);
 		}
 	}
 	else if ( (!S_ISDIR(stat_buf.st_mode))
 				|| (stat_buf.st_uid != my_id)
 				|| (chmod( wn_tmpdir, 0700) != 0) ) {
-		fprintf( stderr, err_m[0], wn_tmpdir);
+		PRINT_ERROR( err_m[122], wn_tmpdir);
 		exit( 2);
 	}
 #endif
+
 	if ( *logfile != '\0')
 		if ( (logfp = fopen( logfile, "a")) == NULL ) {
-#ifdef STANDALONE
-			fprintf( stderr, "Can't open logfile: %s\n", logfile);
-#else
-			printf( "Can't open logfile %s\n", logfile);
-#endif
+			PRINT_ERROR( err_m[0], logfile);
 			exit( 2);
 		}
 	if ( *errlog != '\0') {
@@ -380,12 +404,7 @@ char	*logfile,
 			errlogfp = logfp;
 
 		else if ( (errlogfp = fopen( errlog, "a")) == NULL ) {
-#ifdef STANDALONE
-			fprintf( stderr, 
-				"Can't open error logfile: %s\n", errlog);
-#else
-			printf( "Can't open error logfile %s\n", errlog);
-#endif
+			PRINT_ERROR( err_m[0], errlog);
 			exit( 2);
 		}
 	}
@@ -396,12 +415,10 @@ char	*logfile,
 }
 
 
-
+#ifdef STANDALONE
 static void
 restart( )
 {
-
-#ifdef STANDALONE
 	signal( SIGHUP, SIG_IGN);
 
 #ifdef USE_VIRTUAL_HOSTS
@@ -411,8 +428,7 @@ restart( )
 #endif
 	if ( *wnlogfile) {
 		if ( (logfp = freopen( wnlogfile, "a", logfp)) == NULL ) {
-			fprintf( stderr, "Can't reopen logfile: %s\n",
-					wnlogfile);
+			PRINT_ERROR( err_m[0], wnlogfile);
 			exit( 2);
 		}
 	}
@@ -422,8 +438,7 @@ restart( )
 			errlogfp = logfp;
 		else if ( (errlogfp = 
 				freopen( errlogfile, "a", errlogfp)) == NULL ){
-			fprintf( stderr, "Can't reopen error logfile: %s\n",
-					errlogfile);
+			PRINT_ERROR( err_m[0], errlogfile);
 			exit( 2);
 		}
 	}
@@ -431,8 +446,8 @@ restart( )
 	start_log( TRUE);
 	init_mime();
 	signal( SIGHUP, restart);
-#endif /* STANDALONE */
 }
+#endif /* STANDALONE */
 
 
 void
@@ -448,8 +463,7 @@ char	*msg,
 	time_t	clock;
 	struct tm *ltm;
 
-	char	*rhost,
-		status[TINYLEN],
+	char	status[TINYLEN],
 		date[TINYLEN];
 
 
@@ -483,15 +497,12 @@ char	*msg,
 	else
 		strcpy( status, "500");
 
-	rhost = ( *remotehost ? remotehost : remaddr);
-	rhost = ( *rhost ? rhost : "unknown");
-
 	fseek( errlogfp, 0L, 2);
-	fprintf( errlogfp, "%s - - [%s] \"%s\" %s -", rhost,
+	fprintf( errlogfp, "%.255s - - [%.32s] \"%.200s\" %.10s -", this_conp->remotehost,
 				date, this_rp->request, status);
 
 	if ( log_type == WN_VERBOSE ) {
-		fprintf( errlogfp, " <(%d/%d) %s: %s>",
+		fprintf( errlogfp, " <(%d/%d) %.100s: %.100s>",
 			this_conp->pid, this_conp->trans_cnt, msg, msg2);
 	}
 	fprintf( errlogfp, "\n");
@@ -529,7 +540,7 @@ int	error;
 #ifndef NO_SYSLOG
 	if ( (log_func == LSYSLOG) && !*errlogfile) {
 		sprintf( buf, 
-			"none - - [] \"none\" 500 0  <%s: %.100s errno=%d> ",
+			"none - - [] \"none\" 500 0  <%.100s: %.100s errno=%d> ",
 				msg, msg2, error);
 
 		if ( log_type == WN_VERBOSE ) {
@@ -628,7 +639,11 @@ char	*msg,
 		lbuf[2*SMALLLEN];
 
 	if ( outheadp && (*outheadp->status)) {
-		mystrncpy( status, outheadp->status, 4);
+		cp2 = status;
+		cp = outheadp->status;
+		while ( *cp && !isspace(*cp) && ((cp - outheadp->status) < 10))
+			*cp2++ = *cp++;
+		*cp2 = '\0';
 	}
 	else
 		strcpy( status, "200");
@@ -643,7 +658,7 @@ char	*msg,
 	cp = cp2 = this_conp->log_ptr;
 
 	authname = ( *this_rp->authuser ? this_rp->authuser : "-");
-	sprintf( lbuf, "%s [%s] \"%.200s\" %s %s", authname,
+	sprintf( lbuf, "%.100s [%.32s] \"%.200s\" %.32s %.32s", authname,
 				date, ip->request, status, bytes);
 	cp2 += strlen( lbuf);
 	if ( cp2 >= this_conp->logbuf + LOGBUFLEN) {
@@ -654,8 +669,15 @@ char	*msg,
 	cp = cp2;
 
 	if ( log_type == WN_VERBOSE ) {
-		sprintf( lbuf, " <(%d/%d) %s: %s>",
+		if ( streq( status, "206")) {
+			sprintf( lbuf, " <(%d/%d) %.32s (%.32s): %.200s>",
+				this_conp->pid, this_conp->trans_cnt, 
+				log_m[23], outheadp->range, msg2);
+		}
+		else {
+			sprintf( lbuf, " <(%d/%d) %.200s: %.200s>",
 			this_conp->pid, this_conp->trans_cnt, msg, msg2);
+		}
 		cp2 += strlen( lbuf);
 		if ( cp2 >= this_conp->logbuf + LOGBUFLEN) {
 			dump_log();
@@ -666,7 +688,7 @@ char	*msg,
 		strcpy( cp, lbuf);
 		cp = cp2;
 
-		sprintf( lbuf, " <%.500s>", inheadp->ua);
+		sprintf( lbuf, " <%.200s>", inheadp->ua);
 		cp2 += strlen( lbuf);
 		if ( cp2 >= this_conp->logbuf + LOGBUFLEN) {
 			dump_log();
@@ -687,7 +709,7 @@ char	*msg,
 
 		sprintf( lbuf, " <%.500s>", inheadp->cookie);
 		cp2 += strlen( lbuf);
-		if ( cp2 >= this_conp->logbuf + (LOGBUFLEN - 32)) {
+		if ( cp2 >= this_conp->logbuf + (LOGBUFLEN - 64)) {
 			dump_log();
 			cp = cp2 = this_conp->logbuf;
 		}
@@ -695,12 +717,15 @@ char	*msg,
 		cp = cp2;
 
 #ifdef USE_VIRTUAL_HOSTS
-		sprintf( cp, " <%d>", interface_num);
+		if ( (interface_num > 0) && *vhostlist[interface_num -1][3] )
+			sprintf( cp, " <%.32s>", vhostlist[interface_num - 1][3] );
+		else
+			sprintf( cp, " <%d>", interface_num);
 		cp += strlen( cp);
 #endif
 	}
 	else if ( log_type == NCSA_LOG) {
-		sprintf( lbuf, " \"%s\"", inheadp->referrer);
+		sprintf( lbuf, " \"%.500s\"", inheadp->referrer);
 		cp2 += strlen( lbuf);
 		if ( cp2 >= this_conp->logbuf + LOGBUFLEN) {
 			dump_log();
@@ -709,7 +734,7 @@ char	*msg,
 		strcpy( cp , lbuf);
 		cp = cp2;
 
-		sprintf( lbuf, " \"%s\"", inheadp->ua);
+		sprintf( lbuf, " \"%.200s\"", inheadp->ua);
 		cp2 += strlen( lbuf);
 		if ( cp2 >= this_conp->logbuf + LOGBUFLEN) {
 			dump_log();
@@ -760,7 +785,6 @@ char	*msg,
 {
 	char	*authname,
 		*rfc931p,
-		*rhost,
 		bytes[TINYLEN],
 		status[TINYLEN];
 
@@ -777,18 +801,25 @@ char	*msg,
 
 	get_remote_info();
 
-	rhost = remotehost;
-
-	syslog( priority,"%s %s %s [] \"%s\" %s %s  <%s: %s>\n", 
-		rhost, rfc931p, authname, ip->request, status, 
+	syslog( priority,
+		"%.100s %.32s %.32s [] \"%.100s\" %.10s %.9s  <%.100s: %.100s>\n", 
+		this_conp->remotehost, rfc931p, authname, ip->request, status, 
 		bytes, msg, msg2);
 
 	if ( log_type == WN_VERBOSE ) {
 #ifdef USE_VIRTUAL_HOSTS
-		syslog( priority, "<%s> <%s> <%s> <%d>\n", inheadp->ua,
-			inheadp->referrer, inheadp->cookie, interface_num);
+		char			label[SMALLLEN];
+
+		if ( (interface_num > 0 ) && *vhostlist[interface_num - 1][3] )
+			sprintf( label, " <%.64s>", 
+					vhostlist[interface_num - 1][3]);
+		else
+			sprintf( label, " <%d>", interface_num );
+
+		syslog( priority, "<%.100s> <%.100s> <%.100s> <%.100s>\n",
+			inheadp->ua, inheadp->referrer, inheadp->cookie, label);
 #else
-		syslog( priority, "<%s> <%s> <%s>\n",
+		syslog( priority, "<%.100s> <%.100s> <%.100s>\n",
 			inheadp->ua, inheadp->referrer, inheadp->cookie);
 #endif
 	}
@@ -821,11 +852,11 @@ struct tm *ltm;
 	sprintf( cp, " %c%02ld%02ld", sign, tz/3600, tz % 3600);
 }
 
+#ifdef STANDALONE
 static void
 start_log( restart)
 int	restart;
 {
-#ifdef STANDALONE
 	time_t	clock;
 	struct tm *ltm;
 	char	*cp,
@@ -841,14 +872,15 @@ int	restart;
 			startdate, cp, VERSION, port, getpid() );
 
 	(void) fflush(logfp);
-#endif
 }
+#endif
 
 
 void
 wn_abort( )
 {
 
+	this_conp->keepalive = FALSE;
 	this_rp->type = RTYPE_FINISHED;
 	this_rp->status |= (WN_ABORTED + WN_ERROR);
 	
@@ -858,17 +890,35 @@ void
 wn_exit( status)
 int	status;
 {
+	int	n;
+	char	buf[MIDLEN];
 
+	if ( this_conp->chunk_status & WN_USE_CHUNK)
+		end_chunking();
 	flush_outbuf();
-	if ( *inheadp->tmpfile)
+	if ( *inheadp->tmpfile) {
 		unlink( inheadp->tmpfile);
+		*inheadp->tmpfile = '\0';
+	}
 
-	shutdown( fileno( stdin), 0);
-
+	shutdown( fileno( stdin), 1);
 	dump_log();
+
+	if ( status != 0) {
+		/* error condition */
+		signal( SIGALRM, SIG_DFL);
+		alarm( 3);  	
+		/* Give ourselves at most 3 seconds to shutdown */
+		while ( (n = WN_read( (fileno( stdin)), buf, MIDLEN)) > 0 ) {
+			; /* Collect stuff from client and throw it away */
+		}
+	}
+
+	close( fileno( stdin));
 	exit( status);
 }
 
+
 static void
 dump_log()
 {
@@ -876,7 +926,6 @@ dump_log()
 			*cp2;
 
 	char		*rfc931p,
-			*rhost,
 			logline[MIDLEN];
 
 #ifndef NO_FLOCK
@@ -899,9 +948,6 @@ dump_log()
 #endif
 
 	rfc931p = ( *this_conp->rfc931name ? this_conp->rfc931name : "-");
-	rhost = ( *remotehost ? remotehost : remaddr);
-	rhost = ( *rhost ? rhost : "unknown");
-
 	fseek( logfp, 0L, 2);
 
 	cp = this_conp->logbuf;
@@ -911,7 +957,8 @@ dump_log()
 		*cp2 = *cp;
 		if ( *cp2 == '\n') {
 			*++cp2 = '\0';
-			fprintf( logfp, "%s %s %s", rhost, rfc931p, logline);
+			fprintf( logfp, "%s %s %s", this_conp->remotehost, 
+							rfc931p, logline);
 			cp2 = logline;
 			cp++;
 			continue;
