@@ -1,7 +1,7 @@
 /*
     Wn: A Server for the HTTP
     File: wn/misc.c
-    Version 1.18.0
+    Version 2.0.4
 */
 
 /*
@@ -302,7 +302,7 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 			break;
 
 		case 'c':	/* appropriate date and time representation */
-			sprintf(tbuf, "%s %s %2d %02d:%02d:%02d %d",
+			sprintf(tbuf, "%.3s %.3s %2d %02d:%02d:%02d %d",
 				days_a[timeptr->tm_wday],
 				months_a[timeptr->tm_mon],
 				timeptr->tm_mday,
@@ -365,7 +365,7 @@ strftime(char *s, size_t maxsize, const char *format, const struct tm *timeptr)
 			break;
 
 		case 'x':	/* appropriate date representation */
-			sprintf(tbuf, "%s %s %2d %d",
+			sprintf(tbuf, "%.3s %.3s %2d %d",
 				days_a[timeptr->tm_wday],
 				months_a[timeptr->tm_mon],
 				timeptr->tm_mday,
