@@ -1,7 +1,7 @@
 /*
     Wn: A Server for the HTTP
     File: wn/parse.c
-    Version 1.18.7
+    Version 2.0.4
     
     Copyright (C) 1996, 1997  <by John Franks>
 
@@ -23,9 +23,7 @@
 
 #include "../config.h"
 #include <stdio.h>
-#ifdef __STDC__
 #include <stdlib.h>
-#endif
 #include <string.h>
 #include "wn.h"
 #include "parse.h"
@@ -75,7 +73,10 @@ char * WN_CONST parserr_m[] = {
 	/* 9 */ "Can't open Conditional text file",
 	/* 10 */ "Redirect attempted after text sent",
 	/* 11 */ "Missing quote mark in redirect URL",
-	/* 12 */ "If-Elif-Else nested too deeply"
+	/* 12 */ "If-Elif-Else nested too deeply",
+	/* 13 */ "If-Elif condition syntax error",
+	/* 14 */ "Parse construct too long",
+	/* 15 */ "Unexpected end of file"
 };
 
 /*
@@ -106,7 +107,7 @@ unsigned	show;
 	pdata.currline = 0;
 
 	/*
-	 * ip->do_wrap_1st_time gets reinitialized in dolocation()
+	 * ip->do_wrap_1st_time gets reinitialized in dolocation ()
 	 * and in keepalive loop in wn.c.
 	 */
 	if ( ip->do_wrap_1st_time) {
@@ -127,7 +128,7 @@ unsigned	show;
 			if ( !(ip->filetype & WN_TEXT)) {
 				senderr( DENYSTATUS, err_m[34],
 					ip->relpath);
-				wn_exit( 0);
+				wn_exit( 0); /* senderr: DENYSTATUS */
 			}
 
 			wrap_place = in_file;
@@ -161,14 +162,14 @@ unsigned	show;
 		/* Error logged in openfp */
 
 		if ( ip->attrib2 & WN_LIST_INCL)
-			ip->inclptr = ip->includes; /* safe empty spot */
+			ip->inclptr = wn_empty; /* safe empty spot */
 		else
 			ip->inclptr = cp;
 		return;
 	}
 	else {
 		if ( ip->attrib2 & WN_LIST_INCL)
-			ip->inclptr = ip->includes; /* safe empty spot */
+			ip->inclptr = wn_empty; /* safe empty spot */
 		else
 			ip->inclptr = cp;
 
@@ -202,21 +203,30 @@ File_or_Pipe	frp_st;
 Parsedata	*pdp;
 {
 	char	linebuf[BIGLEN],
+		svbuf[BIGLEN],
 		insert_param[SMALLLEN];
 
 	int	token;
 
 	pdp->currline = 0;
-	while ( fgets( linebuf, BIGLEN, frp_st.fp)) {
+	svbuf[0] = '\0';
+	while ( get_parse_line( linebuf, svbuf, BIGLEN, frp_st.fp)) {
 		pdp->currline++;
 		token = get_parse_token( linebuf, insert_param, pdp);
 
+		if ( token == EMPTY_TOKEN ) {
+			if ( !(ip->status & WN_PROLOGSENT) )
+				continue;
+			else
+				token = TEXT_TOKEN;
+		}
+
 		/* ignore if using section but not between start and end */
 		if ( (pdp->show & (SECT_MSK)) && !(pdp->show & (IN_SECT_MSK))
 						&& ( token != START))
 			continue;
 
-		if ( ( token == NULLTOKEN) || (token == COMMENT)) {
+		if ( ( token == TEXT_TOKEN) || (token == COMMENT)) {
 			if ( (pdp->show & SHOW_IT) == SHOW_IT)
 				send_text_line( linebuf);
 			continue;
@@ -245,6 +255,7 @@ Parsedata	*pdp;
 {
 
 	char		linebuf[BIGLEN],
+			svbuf[BIGLEN],
 			insert_param[SMALLLEN];
 
 	int		token;
@@ -255,16 +266,24 @@ Parsedata	*pdp;
 	}
 
 	pdp->currline = 0;
-	while ( fgets( linebuf, BIGLEN, ip->fp)) {
+	svbuf[0] = '\0';
+	while ( get_parse_line( linebuf, svbuf, BIGLEN, ip->fp)) {
 		pdp->currline++;
 		token = get_parse_token( linebuf, insert_param, pdp);
 
+		if ( token == EMPTY_TOKEN ) {
+			if ( !(ip->status & WN_PROLOGSENT) )
+				continue;
+			else
+				token = TEXT_TOKEN;
+		}
+
 		/* ignore if using section but not between start and end */
 		if ( (pdp->show & (SECT_MSK)) && !(pdp->show & (IN_SECT_MSK))
 						&& ( token != START))
 			continue;
 
-		if ( ( token == NULLTOKEN) || (token == COMMENT)) {
+		if ( ( token == TEXT_TOKEN) || (token == COMMENT)) {
 			if ( (pdp->show & SHOW_IT) == SHOW_IT)
 				send_text_line( linebuf);
 			continue;
@@ -300,7 +319,6 @@ char		*insert_ptr;
 	case FIELD:				/* field# */
 		if ( (pdp->show & SHOW_IT) == SHOW_IT ) {
 			mystrncpy( buf, ip->field[atoi(insert_ptr)], MIDLEN);
-			strcat( buf, "\n");
 			send_text_line( buf);
 		}
 		break;
@@ -310,7 +328,6 @@ char		*insert_ptr;
 			if ( (cp = getenv(insert_ptr)) == NULL)
 				break;
 			mystrncpy( buf, cp, MIDLEN);
-			strcat( buf, "\n");
 			send_text_line( buf);
 		}
 		break;
@@ -335,9 +352,12 @@ char		*insert_ptr;
 				logerr( parserr_m[10], pdp->currfile);
 				break;
 			}
-			dolocation( outheadp->redirect, ip);
+			this_conp->keepalive = FALSE;
+			dolocation( outheadp->redirect, ip, 301);
+			wn_exit( 0);  /* after redirect dolocation; UGLY */
 		}
 		break;
+
 	case SECTION:
 		show2 = pdp->show;
 		if ( !(show2 & SECT_MSK)) {
@@ -363,14 +383,12 @@ char		*insert_ptr;
 	case QUERY:
 		if ( (pdp->show & SHOW_IT) == SHOW_IT ) {
 			mystrncpy( buf, ip->query, MIDLEN);
-			strcat( buf, "\n");
 			send_text_line( buf);
 		}
 		break;
 	case TITLE:
 		if ( (pdp->show & SHOW_IT) == SHOW_IT ) {
 			mystrncpy( buf, ip->title, MIDLEN);
-			strcat( buf, "\n");
 			send_text_line( buf);
 		}
 		break;
@@ -518,7 +536,7 @@ unsigned	show;
 
 
 	char	linebuf[MIDLEN],
-		linebuf2[MIDLEN],
+		svbuf[MIDLEN],
 		bigbuf[3*MIDLEN],
 		insert_param[SMALLLEN],
 		*astart,
@@ -551,17 +569,25 @@ unsigned	show;
 
 	if ( off1 > off2) {
 			senderr( CLIENT_ERR, err_m[38], "");
-			wn_exit( 2);
+			wn_exit( 2); /* senderr: CLIENT_ERR */
 	}
 
-	bigbuf[0] = '\0';
-
-	while ( fgets( linebuf, MIDLEN, ip->fp)) {
+	svbuf[0] = bigbuf[0] = '\0';
+	
+	while ( get_parse_line( linebuf, svbuf, MIDLEN, ip->fp)) {
 		pdata.currline++;
 		i++;
 
 		token = get_parse_token( linebuf, insert_param, &pdata);
-		if ( ( token == NULLTOKEN) || (token == COMMENT)) {
+
+		if ( token == EMPTY_TOKEN ) {
+			if ( !(ip->status & WN_PROLOGSENT) )
+				continue;
+			else
+				token = TEXT_TOKEN;
+		}
+
+		if ( ( token == TEXT_TOKEN) || (token == COMMENT)) {
 			if ( !((pdata.show & SHOW_IT) == SHOW_IT) )
 				continue;
 		}
@@ -579,15 +605,13 @@ unsigned	show;
 		if ( i == markline) {
 			s3 = bigbuf + strlen(bigbuf);
 			mystrncpy( s3, linebuf, MIDLEN);
-			mystrncpy( linebuf2, linebuf, MIDLEN);
-			strlower( linebuf2);
-			if ( off2 <= strlen( linebuf2) ) {
+			if ( off2 <= strlen( s3) ) {
 				begin = s3 + off1;
 				end = s3 + off2;
 			}
 			else {
 				senderr( CLIENT_ERR, err_m[38], "");
-				wn_exit( 2);
+				wn_exit( 2); /* senderr: CLIENT_ERR */
 			}
 
 
@@ -595,7 +619,7 @@ unsigned	show;
 			if ( (astart = in_anchor( bigbuf, begin))) {
 				*astart = '\0';
 				send_text_line( bigbuf);
-				sprintf( linebuf, "<a name=\"%s\">&#160;</a>",
+				sprintf( linebuf, "<a name=\"%.32s\">&#160;</a>",
 						WN_HTML_MARK);
 				send_text_line( linebuf);
 					
@@ -682,6 +706,7 @@ Parsedata	*pdp;
 
 	switch ( token) {
 	case IF_ERR:
+		parse_html_err( IF_ERR, pdp);
 		pdp->show |= ERR_MSK;
 		pdp->show &= ~SHOW_MSK;
 		break;
@@ -800,9 +825,12 @@ Parsedata	*pdp;
 
 	if ( pdp->currfile == NULL)
 		pdp->currfile = "unknown";
-	sprintf( buf, "file=%s, line=%d", pdp->currfile, pdp->currline);
+	sprintf( buf, "file=%.200s, line=%d", pdp->currfile, pdp->currline);
 
 	switch ( token) {
+		case IF_ERR:
+			logerr( parserr_m[13], buf);
+			break;
 		case ELSE:
 		case ELIF_TRUE:
 		case ELIF_FALSE:
@@ -821,7 +849,7 @@ Parsedata	*pdp;
 		case PARSE_TOO_DEEP:
 			logerr( parserr_m[12], buf);
 			break;
-		case NULLTOKEN:
+		case TEXT_TOKEN:
 			logerr( parserr_m[5], buf);
 			break;
 	}
@@ -833,7 +861,8 @@ char		*s,
 		*param_ptr;
 Parsedata	*pdp;
 {
-	char		buf[SMALLLEN];
+	char		*if_exprp,
+			buf[SMALLLEN];
 	int		sgmlprocess,
 			oldprocess;
 
@@ -844,6 +873,9 @@ Parsedata	*pdp;
 	while ( isspace( *cp))
 		cp++;
 
+	if ( !*cp)
+		return EMPTY_TOKEN;
+
 	sgmlprocess = (strncasecmp( cp, "<?wn", 4) == 0);
 	oldprocess = (strncmp( cp, "<!--", 4) == 0 );
 
@@ -853,7 +885,7 @@ Parsedata	*pdp;
 			if ( !(this_rp->status & WN_PROLOGSENT) )
 				http_prolog();
 		}
-		return (NULLTOKEN);
+		return (TEXT_TOKEN);
 	}
 
 	cp += 4;
@@ -914,14 +946,16 @@ Parsedata	*pdp;
 	}
 
 	if ( strncasecmp( cp, "if ", 3) == 0 ) {
-		return (eval_if( cp + 3, pdp));
+		if_exprp = cp + 3;
+		return (do_ifexpr( &if_exprp, pdp));
 
 	}
 
 	if ( strncasecmp( cp, "elif ", 5) == 0 ) {
 		int	val;
 
-		val = eval_if( cp + 5, pdp);
+		if_exprp = cp + 5;
+		val = do_ifexpr( &if_exprp, pdp);
 		switch (val) {
 		case IF_TRUE:
 			return	ELIF_TRUE;
@@ -999,7 +1033,7 @@ Parsedata	*pdp;
 	mystrncpy( buf, s, SMALLLEN);
 	chop( buf);
 	logerr( parserr_m[5], buf);
-	return (NULLTOKEN);
+	return (TEXT_TOKEN);
 }
 
 void
@@ -1026,6 +1060,7 @@ Request	*ip;
 
 {
 	char	linebuf[BIGLEN],
+		svbuf[BIGLEN],
 		insert_param[SMALLLEN],
 		*wrapfile = NULL;
 	int	token;
@@ -1053,9 +1088,18 @@ Request	*ip;
 		}
 	}
 
-	while ( fgets( linebuf, BIGLEN, swrapfp_st.fp)) {
+	svbuf[0] = '\0';
+	while ( get_parse_line( linebuf, svbuf, BIGLEN, swrapfp_st.fp)) {
 		pdp->currline++;
 		token = get_parse_token( linebuf, insert_param, pdp);
+
+		if ( token == EMPTY_TOKEN ) {
+			if ( !(ip->status & WN_PROLOGSENT) )
+				continue;
+			else
+				token = TEXT_TOKEN;
+		}
+
 		if ( token == INCLUDE) {
 			if (sw_status == SWSTAT_NO_INCL ) {
 				sw_status = SWSTAT_INCL;
@@ -1066,7 +1110,7 @@ Request	*ip;
 			}
 		}
 
-		if ( ( token == NULLTOKEN) || (token == COMMENT)) {
+		if ( ( token == TEXT_TOKEN) || (token == COMMENT)) {
 			if ( (pdp->show & SHOW_IT) == SHOW_IT)
 				send_text_line( linebuf);
 			continue;
@@ -1103,21 +1147,23 @@ Request	*ip;
  */
 
 void
-dolocation( path, ip)
+dolocation( path, ip, status)
 char	*path;
 Request	*ip;
+int	status;
 {
 	static int	num = 0;
 
 	char	*cp,
 		*lhost,
+		*stat_msg,
 		buf[2*MIDLEN];
 
 	num++;
 
 	if ( num > MAX_REDIRECT) {
 		senderr( SERV_ERR, err_m[55], "");
-		wn_exit(2);
+		wn_exit(2); /* senderr: SERV_ERR */
 	}
 
 	if ( strncasecmp( path, "<null>", 6) == 0) {
@@ -1134,9 +1180,21 @@ Request	*ip;
 		mystrncpy( ++cp, ip->query, MIDLEN);
 	/* ends with '?' so add the ip->query to it */
 
+	switch ( status) {
+	case 301:
+		stat_msg = "301 Moved Permanently";
+		break;
+	case 303:
+		stat_msg = "303 See Other";
+		break;
+	case 302:
+	default:
+		stat_msg = "302 Found";
+	}
+
 	if ( ((cp = strchr( buf, ':')) != NULL) && (*(cp+1) == '/'))
 		{ /* URL rediretion: it has "http://" or equivalent  */
-		sendredirect( ip, "302 Redirection", buf);
+		sendredirect( ip, stat_msg, buf);
 		return;
 	}
 
@@ -1144,25 +1202,25 @@ Request	*ip;
 		/* Relative URL with anchor designator requires redirect */
 		/* and so does any relative URL with a '/'  */
 
-		char	loc[BIGLEN];   /* BIGLEN = 2*SMALLLEN */
+		char	loc[BIGLEN];   /* BIGLEN = 2*MIDLEN */
 
 		if ( (port == STANDARD_PORT) ||
 					( strchr( lhost, ':') != NULL) )
-			sprintf( loc, "%s://%.100s", this_conp->scheme, lhost);
+			sprintf( loc, "%.32s://%.100s", this_conp->scheme, lhost);
 		else
-			sprintf( loc, "%s://%.100s:%d", this_conp->scheme, 
+			sprintf( loc, "%.32s://%.100s:%d", this_conp->scheme, 
 					lhost, port);
 
 		if ( *buf == '/') {
 			mystrncat( loc, buf, MIDLEN);
-			sendredirect( ip, "302 CGI redirection", loc);
+			sendredirect( ip, stat_msg, loc);
 			return;
 		}
 
 		mystrncat( loc, inheadp->url_path, MIDLEN);
 		cp = strrchr( loc, '/');
 		mystrncpy( ++cp, buf, MIDLEN - SMALLLEN);
-		sendredirect( ip, "302 Redirection", loc);
+		sendredirect( ip, stat_msg, loc);
 		return;
 
 	}
@@ -1176,16 +1234,18 @@ Request	*ip;
 		*(outheadp->redirect) = '\0';
 		ip->do_wrap_1st_time = TRUE;
 
+		inheadp->method = GET;
+		/* get is only method allowed for internal redirect */
+
 		process_url( ip, sloc);
 	}
 
-	if ( ip->attributes & (WN_PARSE + WN_DYNAMIC + WN_FILTERED) ) {
-		wn_exit(0);
-	/*
-	 * We aren't doing keepalive.  The problematic place
-	 * for return is in dolocation call in parse.c
-	 */
+	if ( (ip->attributes & (WN_PARSE + WN_DYNAMIC +WN_FILTERED))
+				&& ( inheadp->protocol == HTTP1_0) ) {
+		this_conp->keepalive = FALSE;
 	}
+
+	return;
 }
 
 
@@ -1206,7 +1266,7 @@ char	*line,
 	while ( line < begin )
 		*cp++ = *line++;
 	if (! inanch)
-		sprintf( cp, "<b><a name=\"%s\">", WN_HTML_MARK);
+		sprintf( cp, "<b><a name=\"%.32s\">", WN_HTML_MARK);
 	else
 		sprintf( cp, "<b>");
 
@@ -1231,3 +1291,82 @@ char	*line,
 	send_text_line( buf);
 }
 
+char *
+get_parse_line( linebuf, savebuf, size, fp)
+char	*linebuf,
+	*savebuf;
+int	size;
+FILE	*fp;
+{
+	char		save;
+	int		isstart,
+			len;
+
+	register char	*cp;
+
+	if ( !*savebuf) {
+		if ( ! fgets( savebuf, size, fp))
+			return NULL;
+	}
+	cp = savebuf;
+	while ( *cp) {
+		if ( (cp = strchr( cp, '<')) == NULL) {
+			mystrncpy( linebuf, savebuf, size);
+			*savebuf = '\0';
+			return linebuf;
+		}
+		isstart = (strncasecmp( cp, "<?wn", 4) == 0)
+				|| (strncmp( cp, "<!--", 4) == 0 );
+		
+		if ( isstart)
+			break;
+		else
+			cp++;
+	}
+
+	/* found start of token in savebuf; cp points to it */
+
+	if ( cp != savebuf ) {  /* token not at start of savebuf */
+		*cp = '\0';
+		mystrncpy( linebuf, savebuf, size);
+		*cp = '<';
+		mystrncpy( savebuf, cp, size);
+		return linebuf;
+	}
+
+	else {     /* token starts at start of savebuf */
+		cp = savebuf;
+		while ( *cp ) {
+			if ( (cp = strchr( cp, '>')) != NULL) {
+				if ( *(cp - 1) == '\\') {
+					mystrncpy( cp - 1, cp, size);
+					continue;
+				}
+				cp++;
+				save = *cp;
+				*cp = '\0';
+				mystrncpy( linebuf, savebuf, size);
+				*cp = save;
+				mystrncpy( savebuf, cp, size);
+				return linebuf;
+			}
+			else {
+				len = strlen( savebuf);
+				if ( len > size - 5 ) {
+					logerr( parserr_m[14], savebuf);
+					mystrncpy( linebuf, savebuf, size);
+					*savebuf = '\0';
+					return linebuf;
+				}			
+				if ( !fgets( savebuf + len, size - len, fp)) {
+					logerr( parserr_m[15], savebuf);
+					mystrncpy( linebuf, savebuf, size);
+					*savebuf = '\0';
+					return linebuf;
+				}
+				cp = savebuf;
+			}
+		}
+		return NULL;  /* not reached ? */
+	}
+}
