@@ -1,7 +1,7 @@
 /*
     Wn: A Server for the HTTP
     File: wn/parse.h
-    Version 1.07
+    Version 1.19.5
     
     Copyright (C) 1995  <by John Franks>
 
@@ -22,7 +22,7 @@
 */
 
 
-#define NULLTOKEN		(0)
+#define TEXT_TOKEN		(0)
 #define END			(1)
 #define ELSE			(2)
 #define ENDIF			(3)
@@ -57,6 +57,9 @@
 #define PARSE_TOO_DEEP		(32)
 #define LIST_INCLUDE		(33)
 #define LIST_SECTION		(34)
+#define EMPTY_TOKEN		(35)
+#define MATCH_LANGUAGE		(36)
+#define MATCH_BASIC_USER	(37)
 
 
 typedef struct Parsedata {
@@ -103,6 +106,16 @@ typedef struct Parsedata {
 #define SWSTAT_INCL	(1)
 #define SWSTAT_ERROR	(2)
 
+/* return values for gettoken */
+
+#define IFCLAUSE_TERM	(1)
+#define IFCLAUSE_END	(2)
+#define IFCLAUSE_AND	(3)
+#define IFCLAUSE_OR	(4)
+#define IFCLAUSE_ERR	(5)
+
+
+
 extern
 WN_CONST 
 char * WN_CONST parserr_m[];
