@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/prequest.c
-    Version 1.18.3
+    Version 2.0.4
 
-    Copyright (C) 1996  <by John Franks>
+    Copyright (C) 1996-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@
 extern char	*inet_ntoa();
 
 #ifdef USE_VIRTUAL_HOSTS
-extern char	*vhostlist[][3];
+extern char	*vhostlist[][4];
 #endif
 
 #ifndef S_IROTH
@@ -76,14 +76,33 @@ char	*url_path;
 	char		redirect[MIDLEN],
 			path[MIDLEN];
 
+	ip->type = RTYPE_UNCHECKED; /* we don't know type yet */
+
+	if ( inheadp->method ==  TRACE) {
+		/* nothing to do */
+		ip->type = RTYPE_FINISHED;
+		return;
+	}
+
+	if ( inheadp->method ==  OPTIONS) {
+		if ( *url_path == '*') {
+			ip->status &= ~(WN_HAS_BODY);
+			mystrncpy( outheadp->list, "Content-length: 0\r\n",30);
+			mystrncat( outheadp->list, 
+				"Allow: GET, HEAD, TRACE, OPTIONS\r\n", 64); 
+			http_prolog( );
+			writelog( ip, log_m[22], "");
+			ip->type = RTYPE_FINISHED;
+			return;
+		}
+	}
+
 	if ( (inheadp->protocol ==  HTTP1_1) &&	(!*inheadp->host_head) ) {
 		/* Missing Host: header in 1.1 request is an error */
 		senderr(CLIENT_ERR, err_m[15], "");
 		return;
 	}
 
-	ip->type = RTYPE_UNCHECKED; /* we don't know type yet */
-
 	/*
 	 * Here are the initial steps in correct order:
 	 *
@@ -132,6 +151,7 @@ char	*url_path;
 	else
 		mystrncpy( path, url_path, MIDLEN);
 
+
 	www_unescape( path, '+');
 
 	if ( path[1] == '\0') {
@@ -144,7 +164,7 @@ char	*url_path;
 		/* path =  "/;something" */
 		strcpy( path, DEFAULT_URI);
 		ip->filetype |= WN_DEFAULT_DOC;
-		mystrncat( path, &url_path[1], MIDLEN);
+		mystrncat( path, &url_path[1], MIDLEN - sizeof(DEFAULT_URI));
 	}
 
 	tilde( ip, path);
@@ -250,10 +270,10 @@ char	*url_path;
 			}
 			if ( (port == STANDARD_PORT) ||
 					( strchr( lhost, ':') != NULL) )
-				sprintf( redirect, "%s://%s%s/", 
+				sprintf( redirect, "%.32s://%.100s%.1536s/", 
 					this_conp->scheme, lhost, url_path);
 			else
-				sprintf( redirect, "%s://%s:%d%s/",
+				sprintf( redirect, "%.32s://%.100s:%d%.1536s/",
 						this_conp->scheme, lhost,
 						port, url_path);
 			sendredirect( ip, "301 Moved Permanently", redirect);
@@ -277,6 +297,8 @@ char	*url_path;
 	}
 	else if ( trailslash) {
 			/* iswndir( ip) is false; it's not a directory */
+			/* put slash back on path for DENYHANDLER --ckd */
+			strcat ( ip->filepath, "/");  
 			ip->type = RTYPE_DENIED;
 			return;
 	}
@@ -401,7 +423,7 @@ void
 get_stat( ip)
 Request	*ip;
 {
-	struct stat stat_buf;
+	struct stat	stat_buf;
 
 
 #ifndef S_ISREG
@@ -429,7 +451,6 @@ Request	*ip;
 	if ( !(stat_buf.st_mode & S_IROTH))
 		ip->filetype |= WN_NOT_WORLD_READ;
 
-
 	if ( S_ISDIR( stat_buf.st_mode)) {
 		if ( ip->filetype & WN_DEFAULT_DOC) {
 			logerr( err_m[88], ip->filepath);
@@ -440,6 +461,7 @@ Request	*ip;
 		return;
 	}
 
+
 	if ( !(S_ISREG(stat_buf.st_mode))) {
 		logerr( err_m[115], ip->filepath);
 		ip->type = RTYPE_DENIED;
@@ -456,7 +478,7 @@ void
 set_etag( sbp)
 struct stat *sbp;
 {
-	sprintf( this_rp->etag, "%lx-%lx-%lx", (long) sbp->st_mtime,
+	sprintf( this_rp->etag, "%lx=%lx=%lx", (long) sbp->st_mtime,
 				(long) sbp->st_ino, (long) sbp->st_size);
 }
 
@@ -473,8 +495,10 @@ char	*path;
 	*(ip->pathinfo) = '\0';
 	if ( (cp = strstr( path, CGI_EXT)) != NULL) {
 		cp += (sizeof( CGI_EXT) - 1);
-		if ( (*cp == '/') || (*cp == '\0')) {
+		if ( (*cp == '/') || (*cp == '\0') || (*cp == ';')) {
 			ip->type = RTYPE_CGI;
+			ip->attributes |= WN_NOSEARCH;
+			ip->allowed |= WN_M_POST;
 			mystrncpy( ip->pathinfo, cp, MIDLEN);
 			*cp = '\0';
 			path_security( ip, ip->pathinfo, TRUE);
@@ -490,7 +514,10 @@ char	*path;
 		cp += sizeof( CGI_BIN);
 		if ( *cp == '/') {
 			cp++;
+			ip->attrib2 |= WN_ISACGIBIN;
 			ip->type = RTYPE_CGI;
+			ip->attributes |= WN_NOSEARCH;
+			ip->allowed |= WN_M_POST;
 			if ( (cp = strchr( cp, '/')) != NULL) {
 				mystrncpy( ip->pathinfo, cp, MIDLEN);
 				*cp = '\0';
@@ -498,8 +525,12 @@ char	*path;
 			}
 			return;
 		}
+		if ( !*cp )
+			ip->attrib2 |= WN_ISACGIBIN;
 	}
 #endif
+	/* not CGI so no POST */
+	ip->allowed &= ~WN_M_POST;
 }
 
 static void
@@ -787,9 +818,13 @@ Request *ip;
 	time_t  mtime, max_mtime;
 	struct stat stat_buf;
 
-	max_mtime = 0;
+	max_mtime = (time_t) 0;
 
 	if ( ip->attrib2 & WN_WRAPPED) {
+		if ( strchr( ip->wrappers, '!')) {
+			ip->mod_time = (time_t) 0;
+			return;
+		}
 		mtime = max_mtime_includes(ip->wrappers, ip);
 		if (mtime > max_mtime) {
 			max_mtime = mtime;
@@ -797,6 +832,10 @@ Request *ip;
 	}
 
 	if ( ip->attrib2 & WN_INCLUDE ) {
+		if ( strchr( ip->includes, '!')) {
+			ip->mod_time = (time_t) 0;
+			return;
+		}
 		mtime = max_mtime_includes(ip->includes, ip);
 		if (mtime > max_mtime) {
 			max_mtime = mtime;
@@ -804,6 +843,10 @@ Request *ip;
 	}
 
 	if ( ip->attrib2 & WN_LIST_INCL ) {
+		if ( strchr( ip->list_incl, '!')) {
+			ip->mod_time = (time_t) 0;
+			return;
+		}
 		mtime = max_mtime_includes(ip->list_incl, ip);
 		if (mtime > max_mtime) {
 			max_mtime = mtime;
@@ -865,7 +908,6 @@ set_interface_root( )
 				|| streq( vhostlist[ifn][1] , lhostname))  {
 				mystrncpy( this_rp->rootdir,
 					vhostlist[ifn][2], SMALLLEN);
-				cp = vhostlist[ifn][0];
 				mystrncpy( hostname, lhostname, SMALLLEN);
 				break;
 			}
@@ -900,7 +942,7 @@ set_interface_root( )
 		}
 	}
 	interface_num = ((vhostlist[ifn][2] == NULL) ? 0 : ++ifn);
-	/* Set interface to zero if nothing matched, otherwise (ifn+1) */
+	/* Set interface to zero if nothing matched, otherwise (ifn + 1) */
 	if ( interface_num == 0)
 		mystrncpy( this_rp->rootdir, rootdir, SMALLLEN);
 	/* Use default rootdir if no matches */
