@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/send.c
-    Version 1.18.4
+    Version 2.0.4
     
-    Copyright (C) 1996-7  <by John Franks>
+    Copyright (C) 1996-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include "../config.h"
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <memory.h>
 #include <errno.h>
@@ -36,7 +37,9 @@
 #include "parse.h"
 #include "reg.h"
 
-#ifndef USE_WN_WRITE
+#ifdef USE_WN_WRITE
+extern int WN_write();
+#else
 #define WN_write(a,b,c)		write(a,b,c)
 #endif
 
@@ -50,8 +53,6 @@ static void	filter_open(),
 
 static char	*enter_range();
 
-static int	valid_range();
-
 extern long	atol();
 
 
@@ -66,11 +67,23 @@ char	*status,
 
 	get_remote_info( );
 
-	sprintf( outheadp->status, "%s %s", status, msg);
+	sprintf( outheadp->status, "%.32s %.200s", status, msg);
 	strcpy( outheadp->list, "Content-type: text/html\r\n");
 
-	if ( streq( status, SERV_ERR))
+	if ( *status == '4') {
+		/* it's a 4xx error; a client error */
+		if ( streq( status, "416") ) {
+			sprintf( outheadp->range, "*/%ld", this_rp->datalen);
+		}
+		else {
+			this_conp->keepalive = FALSE;
+		}
+	}
+
+	if ( streq( status, SERV_ERR)) {
+		this_conp->keepalive = FALSE;
 		logerr( msg, file);
+	}
 	else
 		writelog( this_rp, msg, file);
 
@@ -82,17 +95,20 @@ char	*status,
 		set_interface_root( );
 
 		sprintf( buf,
-		"<html>\n<head>\n<title>%s %s</title>\n</head>\n<body>\n", 
+		"<html>\n<head>\n<title>%.32s %.150s</title>\n</head>\n<body>\n", 
 					status, msg);
 		sprintf( buf + strlen( buf), 
-				"<h2>Error code %s</h2>\n%s\n", status, msg);
+				"<h2>Error code %.32s</h2>\n%.150s\n", status, msg);
 		sprintf( buf + strlen( buf),
-			"\n<hr>\n<address>%s</address>\n", VERSION);
+			"\n<hr>\n<address>%.32s</address>\n", VERSION);
 		sprintf( buf + strlen(buf), "\n</body>\n</html>\n");
 		sprintf( this_rp->length, "%d", strlen(buf));
 
+		if (inheadp->method != HEAD)
+			this_rp->status |= WN_HAS_BODY;
 		http_prolog( );
-		send_text_line( buf);
+		if (inheadp->method != HEAD)
+			send_text_line( buf);
 	}
 
 	this_rp->status |= WN_ERROR;
@@ -107,6 +123,7 @@ Request	*ip;
 
 {
 	char	*cp,
+		*maintainer,
 		owner[MIDLEN],
 		len[TINYLEN],
 		con[SMALLLEN],
@@ -127,12 +144,13 @@ Request	*ip;
 
 	mystrncpy( con, ip->content_type, SMALLLEN);
 	ip->content_type = "text/html";
+	this_rp->status |= WN_HAS_BODY;
 	http_prolog( );
 
 	sprintf(buf, "<html>\n<head>\n<title>URL information </title>\n");
 	send_text_line( buf);
-	cp = ( *dir_p->dir_owner ? dir_p->dir_owner : MAINTAINER);
-	sprintf( owner, "<link rev=\"made\" href=\"%s\">\n", cp);
+	maintainer = ( *dir_p->dir_owner ? dir_p->dir_owner : MAINTAINER);
+	sprintf( owner, "<link rev=\"made\" href=\"%.64s\">\n", maintainer);
 	sprintf(buf,"%.256s</head>\n<body>\n<h2>URL information</h2>\n",
 					owner);
 	send_text_line( buf);
@@ -156,13 +174,19 @@ Request	*ip;
 			send_text_line( buf);
 		}
 	}
+
+	if ( *ip->etag) {
+		sprintf(buf, "<dt><b>ETag:</b>\n<dd>%.256s\n", ip->etag);
+		send_text_line( buf);
+	}
+
 	if ( ip->expires && *ip->expires) {
 		sprintf(buf, "<dt><b>Expires:</b>\n<dd>%.64s\n", ip->expires);
 		send_text_line( buf);
 	}
 
 	if ( *len ) {
-		sprintf(buf, "<dt><b>Size:</b>\n<dd>%s\n", len);
+		sprintf(buf, "<dt><b>Size:</b>\n<dd>%.64s\n", len);
 		send_text_line( buf);
 	}
 	sprintf(buf, "<dt><b>Content-type:</b>\n<dd>%.128s\n", con);
@@ -173,17 +197,22 @@ Request	*ip;
 		send_text_line( buf);
 	}
 
-	if ( ip->mod_time) {
-		gmt = gmtime(&ip->mod_time);
-		strftime( buf, SMALLLEN,
-		"<dt><b>Last-modified:</b>\n<dd> %a, %d %h %Y %T GMT\n", gmt);
+	if ( outheadp->md5 &&  (cp = strchr( outheadp->md5, ':')) != NULL ) {
+		sprintf(buf, "<dt><b>Content-MD5:</b>\n<dd>%.128s\n", ++cp);
 		send_text_line( buf);
 	}
 
-	sprintf(buf, "<dt><b>Maintainer:</b>\n<dd>%.256s\n", cp);
+	if ( ip->mod_time) {
+		gmt = gmtime(&ip->mod_time);
+		strftime( buf, SMALLLEN,
+		"<dt><b>Last-Modified:</b>\n<dd> %a, %d %h %Y %T GMT\n", gmt);
+		send_text_line( buf);
+	}
+
+	sprintf(buf, "<dt><b>Maintainer:</b>\n<dd>%.256s\n", maintainer);
 	send_text_line( buf);
 
-	sprintf(buf, "</dl>\n<hr>\n<address>%s</address>\n", VERSION);
+	sprintf(buf, "</dl>\n<hr>\n<address>%.32s</address>\n", VERSION);
 	send_text_line( buf);
 
 	send_text_line( "\n</body>\n</html>\n");
@@ -212,7 +241,7 @@ Request *ip;
 		check_perm( ip, ip->filepath);
 		if ( (ip->fp = fopen( ip->filepath, "r")) == (FILE *) NULL ) {
 			senderr( DENYSTATUS, err_m[1], ip->filepath);
-			wn_exit( 2);
+			wn_exit( 2); /* senderr: DENYSTATUS */
 		}
 		ip->fptype = FRP_FILE;
 		return;
@@ -225,7 +254,7 @@ Request *ip;
 		if ((ip->fp = popen( dir_p->filemod, "r"))
 					== (FILE *) NULL ) {
 			senderr( SERV_ERR, err_m[39], dir_p->filemod);
-			wn_exit( 2);
+			wn_exit( 2); /* senderr: SERV_ERR */
 		}
 		ip->fptype = FRP_PIPE;
 	}
@@ -265,7 +294,7 @@ Request *ip;
 	}
 	if ( (ip->fp = popen( commandbuf, "r")) == (FILE *) NULL ) {
 		senderr( SERV_ERR, err_m[52], commandbuf);
-		wn_exit( 2);
+		wn_exit( 2); /* senderr: SERV_ERR */
 	}
 	ip->fptype = FRP_PIPE;
 }
@@ -322,7 +351,8 @@ Request	*ip;
 
 	char	buf[OUT_BUFFSIZE];
 
-	int	n;
+	int	n,
+		invalid;
 
 
 	if ( ip->attributes & WN_FILTERED )
@@ -342,7 +372,7 @@ Request	*ip;
 	}
 
 
-	if ( ip->attributes & WN_PARSE ) { 
+	if ( (ip->attributes & WN_PARSE) && (ip->filetype & WN_ISHTML) ){ 
 		dontlog++;
 		/* Don't do http_prolog() until later */
 		do_wrap( ip, SHOW_IT);
@@ -361,18 +391,16 @@ Request	*ip;
 			char	*cp;
 
 			cp = ip->range;
-			enter_range( cp, &startline, &endline);
+			enter_range( cp, &startline, &endline, &invalid);
 			*ip->length ='\0';
 			http_prolog( );
-			if ( endline == -1 )
-				endline = ip->datalen;
 			if ( startline == -1 )
 				logerr( err_m[93], "");
 			for ( n = 1; n <= endline; n++) {
 				if ( fgets( buf, OUT_BUFFSIZE, ip->fp) == NULL)
 					break;
 			/*
-			   Note if a line is longer than OUT_BUFFSIZE sothe
+			   Note if a line is longer than OUT_BUFFSIZE so the
 			   only a partial line is read, then the line count 
                            here will be wrong.  This is a bug.
 			 */
@@ -419,7 +447,7 @@ char	*status,
 	num++;
 	if ( num > MAX_REDIRECT) {
 		senderr( SERV_ERR, err_m[55], err_m[64]);
-		wn_exit( 2);
+		wn_exit( 2); /* senderr: SERV_ERR */
 	}
 
 	if ( strncasecmp( location, "<null>", 6) == 0) {
@@ -439,14 +467,11 @@ char	*status,
 	writelog( ip, log_m[9], location);
 
 
-	if ( ip->attributes & (WN_PARSE + WN_DYNAMIC + WN_FILTERED) )
-		wn_exit(0);
-	/*
-	 * Exit since we aren't doing keepalive.  The problematic place
-	 * for return is in dolocation call in parse.c
-	 */
-
-	/* If we are doing keepalive then return */
+	if ( (ip->attributes & (WN_PARSE + WN_DYNAMIC +WN_FILTERED))
+				&& ( inheadp->protocol == HTTP1_0) ) {
+		this_conp->keepalive = FALSE;
+	}
+	ip->type = RTYPE_FINISHED;
 	return;
 }
 
@@ -456,15 +481,15 @@ Request	*ip;
 {
 	strcpy( outheadp->status, "204 No Response");
 
+	ip->status &= ~(WN_HAS_BODY);
 	http_prolog();
 
 	writelog( ip, log_m[16], "");
-	if ( ip->attributes & (WN_PARSE + WN_DYNAMIC +WN_FILTERED) )
-		wn_exit(0);
-	/*
-	 * We aren't doing keepalive.  The problematic place
-	 * for return is in dolocation call in parse.c
-	 */
+	if ( (ip->attributes & (WN_PARSE + WN_DYNAMIC +WN_FILTERED))
+				&& ( inheadp->protocol == HTTP1_0) ) {
+		this_conp->keepalive = FALSE;
+	}
+	ip->type = RTYPE_FINISHED;
 	return;
 }
 
@@ -484,20 +509,31 @@ send_byterange( )
 		file_len,
 		len_sent;
 
-	int	multi = FALSE,
-		firsttime = TRUE;
+	int	invalid,
+		multi = FALSE,
+		first_one = TRUE;
 
 	len_sent = 0;
 	file_len = this_rp->datalen;
 	nextrange = this_rp->range;
 
-	if ( !valid_range( nextrange, &multi)) {
-		senderr( "416", err_m[113], "");
-		wn_exit( 2); /* senderr: 416 */
-	}
+	if ( strchr( nextrange, ',') != NULL)
+		multi = TRUE;
 
 	while ( nextrange ) {
-		nextrange = enter_range( nextrange, &startbyte, &endbyte);
+		nextrange = enter_range( nextrange, &startbyte, &endbyte, &invalid);
+		if ( invalid) {
+			if (first_one && (nextrange == NULL)) {
+				if ( invalid == 416 )
+					senderr( "416", err_m[113], "");
+				if ( invalid == 400 )
+					senderr( CLIENT_ERR, err_m[121], "");
+				return;
+			}
+			else
+				continue;
+		}
+
 		if ( startbyte == -1 ) {
 			temp_len = endbyte;
 			endbyte = file_len - 1;
@@ -512,21 +548,27 @@ send_byterange( )
 					"206 Partial Content", SMALLLEN);
 
 		if ( multi && (this_rp->filetype & WN_RFC_BYTERANGE) ) {
-			if ( firsttime) {
-				firsttime = FALSE;
+			if ( first_one) {
+				first_one = FALSE;
 				*(this_rp->length) = '\0';
 				srand( this_conp->pid);
 				sprintf( sep, "=%x=%x=%x=",
 						rand(), rand(), rand());
 				sprintf( buf,
-				"multipart/x-byteranges; boundary=\"%s\"",
+				"multipart/x-byteranges; boundary=\"%.64s\"",
 						sep);
 				save_content = this_rp->content_type;
 				this_rp->content_type = buf;
 				http_prolog( );
+				if ( this_conp->chunk_status
+					 & (WN_START_CHUNK + WN_IN_CHUNK)) {
+						senderr( SERV_ERR, err_m[120], "");
+						wn_exit( 2); /* senderr: SERV_ERR */
+				}
 				this_rp->content_type = save_content;
 			}
-			sprintf( buf, "\r\n--%s\r\n", sep);
+			sprintf( buf, "\r\n--%.64s\r\n", sep);
+
 
 			send_text_line( buf);
 			len_sent += strlen( buf);
@@ -546,7 +588,7 @@ send_byterange( )
 			len_sent += (endbyte - startbyte + 1);
 
 			if ( nextrange == (char *)NULL) {
-				sprintf( buf, "\r\n--%s--\r\n", sep);
+				sprintf( buf, "\r\n--%.64s--\r\n", sep);
 				send_text_line( buf);
 				len_sent += strlen( buf);
 				sprintf( this_rp->length, "%ld", len_sent);
@@ -558,10 +600,8 @@ send_byterange( )
 		else {
 			sprintf( this_rp->length, "%ld",
 						endbyte - startbyte + 1);
-
 			if ( this_rp->filetype & WN_RFC_BYTERANGE ) {
-				sprintf( outheadp->range,
-					"Content-Range: bytes %ld-%ld/%ld\r\n",
+				sprintf( outheadp->range, "%ld-%ld/%ld",
 						startbyte, endbyte, file_len);
 			}
 			http_prolog( );
@@ -621,10 +661,11 @@ long	start,
 
 
 static char 
-*enter_range(  value, start, end )
+*enter_range(  value, startp, endp, errp )
 char	*value;
-long	*start,
-	*end;
+long	*startp,
+	*endp;
+int	*errp;
 {
 	register char	*cp,
 			*cp2;
@@ -632,9 +673,8 @@ long	*start,
 	char		*next;
 
 	/* if ip->param_value is "123-234" it is a file range from
-	 * byte or line 123 to 234.  Put 123 in start and 234
-	 * in end. For 123- use -1 for end range and for -456 use
-	 * -1 for start.
+	 * byte  123 to 234.  Put 123 in *startp and 234
+	 * in *endp. For 123- use -1 for endp range
 	 */
 
 	cp = value;
@@ -648,16 +688,38 @@ long	*start,
 
 	if ( (cp2 = strchr( cp, '-')) == NULL) {
 		logerr( err_m[93], cp);
-		return (char *)NULL;
+		*errp = 400;
+		return (next);
 	}
 
+
 	*cp2++ = '\0';
-	*start = ( *cp ? atol( cp ) : (-1));
-	*end = ( *cp2 ? atol( cp2 ) : (-1));
+
+	if ( strchr( cp, '-')  || strchr( cp2, '-') ) {
+		logerr( err_m[93], cp);
+		*errp = 400;
+		return (next);
+	}
+
+	*startp = ( *cp ? atol( cp ) : (-1));
+	*endp = ( *cp2 ? atol( cp2 ) : this_rp->datalen);
+	if ( *endp > this_rp->datalen)
+		*endp = this_rp->datalen;
+
+	if ( *startp > this_rp->datalen)
+		*errp = 416;
+	else if ( *startp > *endp)
+		*errp = 400;
+	else
+		*errp = 0;
 	return (next);
 }
 
 
+
+
+#define CHUNKPAD	(6)	/* number of bytes for chunk size */
+				/* CHUNKPAD = 4 for digits + 2 for CRLF */
 void
 send_out_fd( fd )
 int	fd;
@@ -667,15 +729,47 @@ int	fd;
 			len;
 
 	remlen = this_conp->outbuf + OUT_BUFFSIZE - this_conp->out_ptr;
+
 	while ( TRUE) {
-		len = read( fd, this_conp->out_ptr, remlen);
-		if ( (len == -1) && (errno == EINTR))
-			continue;
-		if ( len <= 0 )
-			break;
+		if ( this_conp->chunk_status & WN_START_CHUNK) {
+			/* set up chunking */
+			this_conp->chunksize = 0;
+			if ( remlen < OUT_BUFFSIZE/4 ) {
+				flush_outbuf();
+				remlen = OUT_BUFFSIZE;
+			}
+
+			this_conp->chunksize_ptr = this_conp->out_ptr;
+			this_conp->out_ptr += CHUNKPAD;
+			len = read( fd, this_conp->out_ptr, remlen);
+
+			if ( len > 0 ) {
+				this_conp->chunk_status &= ~(WN_START_CHUNK);
+				this_conp->chunk_status |= WN_IN_CHUNK;
+				this_rp->logcount += CHUNKPAD;
+			}
+			else {  /* len <= 0 */
+				this_conp->out_ptr -= CHUNKPAD;
+				/* undo this */
+				if ( (len == -1) && (errno == EINTR))
+					continue;
+				else
+					break;
+			}
+		}
+		else {
+			len = read( fd, this_conp->out_ptr, remlen);
+			if ( (len == -1) && (errno == EINTR))
+				continue;
+			if ( len <= 0 )
+				break;
+		}
 
 		this_rp->logcount += (long) len;
 		this_conp->out_ptr += len;
+		if ( this_conp->chunk_status & WN_IN_CHUNK) {
+			this_conp->chunksize += (long)len;
+		}
 		if ( len == remlen ) {  /* buffer is full */
 			flush_outbuf();
 			remlen = OUT_BUFFSIZE;
@@ -701,17 +795,30 @@ int	len;
 			*end;
 
 	int		remlen;
-
 	end = this_conp->outbuf + OUT_BUFFSIZE;
 	cp = buf;
 
 	this_rp->logcount += (long) len;
 	while ( len > 0 ){
+		if ( this_conp->chunk_status & WN_START_CHUNK) {
+			/* set up chunking */
+			if ( (end - this_conp->out_ptr) < OUT_BUFFSIZE/4 ) {
+				flush_outbuf();
+			}
+			this_conp->chunksize = 0;
+			this_conp->chunk_status &= ~(WN_START_CHUNK);
+			this_conp->chunk_status |= WN_IN_CHUNK;
+			this_conp->chunksize_ptr = this_conp->out_ptr;
+			this_conp->out_ptr += CHUNKPAD;
+			this_rp->logcount += CHUNKPAD;
+		}
 		cp2 = this_conp->out_ptr;
 		if ( end > cp2 + len ) {
 			/* it all fits in buffer */
 			memcpy( cp2, cp, len);
 			this_conp->out_ptr += len;
+			if ( this_conp->chunk_status & WN_IN_CHUNK)
+				this_conp->chunksize += (long)len;
 			if ( this_rp->attributes & WN_UNBUFFERED)
 				flush_outbuf();
 			return;
@@ -721,6 +828,10 @@ int	len;
 					+ OUT_BUFFSIZE - cp2;
 			len -= remlen;
 			memcpy( cp2, cp, remlen);
+
+			if ( this_conp->chunk_status & WN_IN_CHUNK)
+				this_conp->chunksize += (long)remlen;
+
 			cp += remlen;
 			this_conp->out_ptr = end;
 			flush_outbuf();
@@ -739,11 +850,32 @@ flush_outbuf( )
 
 	register char	*cp;
 
+
+	if ( this_conp->out_ptr == this_conp->outbuf)
+		return;
+
+	if ( this_conp->chunk_status & WN_IN_CHUNK) {
+		cp = this_conp->chunksize_ptr;
+
+		sprintf( cp, "%.4lX\r", this_conp->chunksize);
+		/* 4 in %.4lX is CHUNKPAD - 2 */
+
+		cp += strlen(cp);
+		*cp = '\n';
+		this_conp->chunksize = 0;
+		*this_conp->out_ptr++ = '\r';
+		*this_conp->out_ptr++ = '\n';
+		*this_conp->out_ptr = '\0';
+		this_conp->chunk_status &= ~(WN_IN_CHUNK);
+		this_conp->chunk_status |= WN_START_CHUNK;
+	}
+
+
 	cp = this_conp->outbuf;
 	len = this_conp->out_ptr - cp;
-	if ( len == 0 )
-		return;
+
 	fdstdout = fileno( stdout);
+
 	while ( (n = WN_write( fdstdout, cp, len)) < len) {
 		if ( n == -1 && errno == EINTR) 
 			continue;
@@ -770,46 +902,60 @@ flush_outbuf( )
 	this_conp->out_ptr = this_conp->outbuf;
 }
 
+/* Chunking is automatically done by buffer routines (flush_outbuf,
+ * send_out_fd() and send_out_mem() ) whenever chunk_status & WN_USE_CHUNK
+ * is true.  There are two disjoint states for chunking WN_START_CHUNK and
+ * WN_IN_CHUNK.  With the first we are ready to start a new chunk (any
+ * previous one has been completed.  In this state when one of the send_out_*
+ * functions is called space is reserved in the buffer to place the
+ * chunk length,  WN_IN_CHUNK is set, an a chunksize count is started.
+ * When in the state WN_IN_CHUNK, any calls to send_out_* put bytes
+ * in the buffer and update the chunksize.  When the buffer is full
+ * flush_outbuf() is called.  It completes a chunk by filling in the
+ * the chunk size (at chunksize_ptr), terminating the chunk with
+ * CRLF, and setting the state to WN_START_CHUNK, before the buffer is
+ * flushed.  Finally when the document being sent is complete, a
+ * call to end_chunking() completes the current chunk if necessary,
+ * i.e. if WN_IN_CHUNK, appends the zero size chunk indicating the
+ * end of chunking and reinitializes chunk_status, turning off all
+ * bits.
+ * Currently chunk size is formatted as %.4lX, e.g. 02FF.  The maximum
+ * chunk size is OUT_BUFFSIZE which must be less than FFFF hex.
+ */
 
-static int
-valid_range(  str, multip )
-char	*str;
-int	*multip;
-
+void
+end_chunking()
 {
 	register char	*cp,
-			*cp2,
-			*cp3;
-	long		start,
-			end;
+			*cp2;
 
-	char		*next;
-
-	next = str;
-	*multip = FALSE;
-
-	while ( next ) {
-		cp = next;
-		if ( (cp3 = strchr( cp, ',')) != NULL) {
-			*cp3++ = '\0';
-			next = cp3;
-			cp3--;
-			*multip = TRUE;
-		}
-		else
-			next = (char *)NULL;
-
-		if ( (cp2 = strchr( cp, '-')) == NULL)
-			return FALSE;
-
-		*cp2++ = '\0';
-		start = ( *cp ? atol( cp ) : (-1));
-		end = ( *cp2 ? atol( cp2 ) : (-1));
-		*--cp2 = '-';
-		if ( next )
-			*cp3 = ',';
-		if ( ( start <= 0 ) || ( (end != -1) && ( start > end )))
-			return FALSE;
+	if ( !this_conp->chunk_status & WN_USE_CHUNK) {
+		logerr( err_m[116], "");
+		return;
 	}
-	return TRUE;
+	cp = this_conp->out_ptr;
+	if ( this_conp->chunk_status & WN_IN_CHUNK) {
+		/* finish the chunk we're doing */
+		cp2 = this_conp->chunksize_ptr;
+
+		sprintf( cp2, "%.4lX\r", this_conp->chunksize);
+		/* 4 in %.4lX is CHUNKPAD - 2 */
+
+		cp2 += strlen(cp2);
+		*cp2 = '\n';
+
+		*cp++ = '\r';
+		*cp++ = '\n';
+	}
+	this_conp->chunksize = 0;
+	this_conp->chunk_status = 0;
+	/* add last (zero size) chunk */
+	*cp++ = '0';
+	*cp++ = '\r';
+	*cp++ = '\n';
+	/* "trailer" (if any) should go here */
+	*cp++ = '\r';
+	*cp++ = '\n';
+	*cp = '\0';
+	this_conp->out_ptr = cp;
 }
