@@ -1,7 +1,7 @@
 /*
     Wn: A Server for the HTTP
     File: wn/standalone.c
-    Version 1.18.0
+    Version 2.0.4
     
     Copyright (C) 1996  <by John Franks>
 
@@ -23,6 +23,8 @@
 
 #include "../config.h"
 #include <stdio.h>
+#include <stdlib.h>
+#include <grp.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
@@ -71,9 +73,7 @@ do_standalone()
 	FILE	*pid_fp;
 	char	mbuf[2*SMALLLEN];
 
-	int 	user_id = USERID,
-		group_id = GROUPID,
-		sockdes,
+int		sockdes,
 		sd,
 		len,
 		pid,
@@ -206,10 +206,9 @@ do_standalone()
 	}
 
         len = sizeof(sa_client);
-        if ( (sockdes = accept( sd, (struct sockaddr *) &sa_client,
-							&len)) == -1 ) {
-		if ( errno != EINTR &&  errno != ECONNABORTED )
-			daemon_logerr( err_m[28], "", errno);
+        if ( (sockdes = accept( sd, (struct sockaddr *) &sa_client, &len)) == -1 ) {
+		daemon_logerr( err_m[28], "", errno);
+		exit( 2);
 	}
 
 
@@ -241,7 +240,14 @@ do_standalone()
 
 	        while ( (sockdes = accept( sd,
 				(struct sockaddr *) &sa_client, &len)) < 0 ) {
-			if ( errno != EINTR &&  errno != ECONNABORTED ) {
+			switch ( errno) {
+			case EINTR:
+			case ECONNABORTED:
+			case ECONNRESET:
+			case ETIMEDOUT:
+			case EHOSTUNREACH:
+				break;
+			default:
 				daemon_logerr( err_m[28], "", errno);
 			}
 		}
@@ -471,7 +477,7 @@ init_mime( )
  * takes the line and reformats it to the form 
  * "mime_type\0suf1.suf2.suf3\0".  I.e. mime_type null terminated
  * followed by suffixes separated by '.'.  Only suffixes not in
- * mime list are used.
+ * mime list are used.  Returns the the number of suffixes found.
  */
 
 static int
@@ -482,8 +488,6 @@ char	*line;
 	char	*cp,
 		*cp2;
 
-
-
 	if ( (cp = strchr( line, '#')) != NULL)
 		*cp = '\0';
 
