@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/util.c
-    Version 1.18.5
+    Version 2.0.4
 
-    Copyright (C) 1996-7  <by John Franks>
+    Copyright (C) 1996-8  <by John Franks>
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 1, or (at your option)
@@ -21,6 +21,7 @@
 
 #include "../config.h"
 #include <stdio.h>
+#include <unistd.h>
 #include <ctype.h>
 #include <string.h>
 #include <sys/types.h>
@@ -52,7 +53,7 @@ extern void		rfc931();
 
 /*
  * get_remote_ip() gets IP address of client via getpeername call and
- * puts it in global variable remaddr.
+ * puts it in  this_conp->remaddr.
  */
 
 void
@@ -63,15 +64,15 @@ get_remote_ip( )
 
 	size = sizeof(saddr);
 	if ( getpeername(fileno(stdin), (struct sockaddr *) &saddr, &size)< 0){
-		*remaddr = '\0';
-		*remotehost = '\0';
+		*this_conp->remaddr = '\0';
+		*this_conp->remotehost = '\0';
 		if (!isatty(fileno(stdin)))
 			logerr(  err_m[48], "");
 		return;
 	}
 
 	ip_address = saddr.sin_addr;
-	mystrncpy(remaddr, inet_ntoa(ip_address), 20);
+	mystrncpy(this_conp->remaddr, inet_ntoa(ip_address), 20);
 
 #ifdef RFC931_TIMEOUT
 	remsin = &saddr;
@@ -82,12 +83,10 @@ get_remote_ip( )
 
 /*
  * get_remote_info() does DNS lookup of remotehost and places host name
- * in global variable remotehost.  
- * It also does RFC931 lookup if RFC931_TIMEOUT
- * is defined.  If possible the call to this function happens after the
- * entire transaction is complete so the user doesn't have to wait for these
- * lookups to happen.  This delay is not possible for CGI or when doing
- * authentication. (RFC931 stuff thanks to Chris Davis).
+ * in this_conp->remotehost. If possible the call to this function
+ * happens after the entire transaction is complete so the user
+ * doesn't have to wait for these lookups to happen.  This delay
+ * is not possible for CGI or when doing authentication. 
  * 
  */
 
@@ -95,36 +94,37 @@ void
 get_remote_info( )
 {
 #ifndef NO_DNS_HOSTNAMES
-	char    dot_name[MAXHOSTNAMELEN + 1];
+	char    dot_name[2*SMALLLEN];
 	struct hostent	*hostentp = NULL;
 
-	if ( *remotehost )
+	if ( *this_conp->remotehost )
 		return;
-	if ( !*remaddr ) {
-		strcpy( remotehost, "unknown");
+	if ( !*this_conp->remaddr ) {
+		strcpy( this_conp->remotehost, "unknown");
 		return;
 	}
 	
-	/* if (*remotehost) we have already got info; if (!*remaddr)
-	   then there is no hope -- we can't even get IP address */
+	/* if (*this_conp->remotehost) we have already got info;
+	   if (!*this_conp->remaddr) then there is no hope
+	    -- we can't even get IP address */
 
 	hostentp = gethostbyaddr((char *) &ip_address.s_addr,
 			sizeof (ip_address.s_addr), AF_INET);
 	if (hostentp) {
-		mystrncpy( remotehost, hostentp->h_name, MAXHOSTNAMELEN);
+		mystrncpy( this_conp->remotehost, hostentp->h_name, MAXHOSTNAMELEN);
 
 		/* Check that the name has this address listed. */
 #ifndef CHECK_DNS_HOSTNAMES
 		if ( (!dir_p->accessfile) || (!*dir_p->accessfile)) {
-			strlower( remotehost);
+			strlower( this_conp->remotehost);
 			return;
 		}
 			/* Assume ok unless used for access control */
-#endif /* CHECK_DNS_HOSTNAMES */
-		if (strlen( remotehost) >= MAXHOSTNAMELEN) {
-			hostentp = gethostbyname( remotehost);
+#endif /* DON'T CHECK_DNS_HOSTNAMES */
+		if (strlen( this_conp->remotehost) >= MAXHOSTNAMELEN) {
+			hostentp = gethostbyname( this_conp->remotehost);
 		} else {
-			sprintf(dot_name, "%s.", remotehost);
+			sprintf(dot_name, "%.300s.", this_conp->remotehost);
 			hostentp = gethostbyname( dot_name);
 		}
 
@@ -133,20 +133,20 @@ get_remote_info( )
 			for (ap = hostentp->h_addr_list; *ap; ap++) {
 				if (!memcmp( (char *) &ip_address.s_addr,
 						*ap, hostentp->h_length)) {
-					/* Value in remotehost is ok.*/
-					strlower( remotehost);
+					/*  this_conp->remotehost is ok.*/
+					strlower( this_conp->remotehost);
 					return;
 				}
 			}
 		}
 		/* No good name found */
-		*remotehost = '\0';
+		*this_conp->remotehost = '\0';
 	}
 #endif /* NO_DNS_HOSTNAMES */
-	if ( !*remotehost )
-		strcpy( remotehost, remaddr);
-	if ( !*remotehost )
-		strcpy( remotehost, "unknown");
+	if ( !*this_conp->remotehost )
+		strcpy( this_conp->remotehost, this_conp->remaddr);
+	if ( !*this_conp->remotehost )
+		strcpy( this_conp->remotehost, "unknown");
 
 }
 
@@ -155,12 +155,12 @@ get_remote_info( )
 void
 get_rfc931()
 {
-	if (remsin && mysin && (*this_conp->rfc931name == '\0'))
+	if ( this_conp->con_status & WN_CON_TIMEDOUT)
+		return;
+	if ( *this_conp->rfc931name != '\0')
+		return;
+	if ( remsin && mysin )
 		rfc931(remsin, mysin, this_conp->rfc931name);
-	signal( SIGALRM, SIG_DFL);
-	alarm( 20);  	/* Give ourselves 20 seconds to get remote DNS data
-			 * and write log entry.
-			 */
 }
 #endif
 
@@ -330,7 +330,7 @@ char	*command,
 	else
 		mystrncpy( buf, command, BIGLEN - SMALLLEN);
 
-	if ( *inheadp->tmpfile != '\0') {
+	if ( (inheadp->method == POST) 	&& (*inheadp->tmpfile != '\0')) {
 		strcat( buf, " < ");
 		strcat( buf, inheadp->tmpfile);
 	}
