@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/vhost.c
-    Version 1.18.0
+    Version 2.0.0
     
-    Copyright (C) 1996-7  <by John Franks>
+    Copyright (C) 1996-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 */
 
 #include <errno.h>
+#include <stdlib.h>
 #include <string.h>
 #include "../config.h"
 #include "wn.h"
@@ -40,13 +41,11 @@
 #include "vhost.h"
 #else
 
-char *vhostlist[MAXVHOSTS][3] = 
+char *vhostlist[MAXVHOSTS][4] = 
 {
-	{ NULL, NULL, NULL }
+	{ NULL, NULL, NULL, NULL }
 };
 
-extern char	*malloc();
-
 char	vhostfile[SMALLLEN] = VIRTUAL_HOSTS_FILE;
 
 void
@@ -116,6 +115,20 @@ load_virtual()
 			break;
 		}
 
+		while ( *cp && !isspace( *cp))
+			cp++;
+		if ( *cp )
+			*cp++ = '\0';
+
+		while ( *cp && isspace( *cp))
+			cp++;
+
+		if ( *cp )
+			vhostlist[i][3] = cp;
+		else {
+			vhostlist[i][3] = wn_empty;
+		}
+
 		while ( *cp && !isspace( *cp))
 			cp++;
 		if ( *cp )
@@ -127,7 +140,8 @@ load_virtual()
 			return;
 		}
 	}
-	vhostlist[i][0] = vhostlist[i][1] = vhostlist[i][2] = NULL;
+	vhostlist[i][0] = vhostlist[i][1] = vhostlist[i][2] 
+			= vhostlist[i][3] = NULL;
 	fclose( vhostfp);
 }
 
