@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/wn.c
-    Version 1.18.7
+    Version 2.0.4
     
-    Copyright (C) 1996-98  <by John Franks>
+    Copyright (C) 1996-97  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+    GNU General Public License for more deatails.
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
@@ -25,6 +25,8 @@
 #include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <errno.h>
 
 #ifdef AIX
 #include <sys/select.h>
@@ -36,23 +38,15 @@
 #include <unistd.h>
 #endif
 
-#ifndef STANDALONE
-#include <sys/stat.h>
-#endif
-
 #include "wn.h"
 #include "version.h"
 #include "cgi.h"
 
-#ifndef USE_WN_READ
-#define WN_read(a,b,c)		read(a,b,c)
-#endif
-
 #define MAX_POST		(5 * 1024 * 1024)
-#define KEEP_ALIVE_TIMEOUT	(20)
+#define KEEPALIVE_TIMEOUT	(20)
 #define BROWSER_BUGS
-#define AUTHENT_TIMEOUT		(30)
-#define KEEP_ALIVE_MAX		(10)
+#define AUTHENT_TIMEOUT		(20)
+#define KEEPALIVE_MAX		(25)
 
 #define AWAIT_REQUEST		(1)
 #define AWAIT_HEADER		(2)
@@ -61,14 +55,22 @@
 extern int	daemon_init();
 extern long	atol();
 extern void	parse_request(),
-		write_debug();
+		write_debug(),
+		start_chunking(),
+		end_chunking();
 
 extern time_t	time();
 
 static void	do_post(),
-		get_headers(),
-		chk_method(),
+		do_trace(),
+		do_options(),
+		do_allow_header(),
+		get_header(),
+		end_headers(),
 		wn_timeout(),
+		send_keepalive(),
+		chk_method(),
+		remk_postdir(),
 		client_closed();
 
 static Methodtype parse_header( );
@@ -78,8 +80,10 @@ static int	chk_continue(),
 		reset_buf(),
 		load_inbuf(),
 		wn_getc(),
+		get_chunk(),
+		etag_match(),
 		await_state,
-		await_timeout = KEEP_ALIVE_TIMEOUT;
+		await_timeout = KEEPALIVE_TIMEOUT;
 
 Request		*this_rp = NULL;
 
@@ -90,6 +94,7 @@ Connection	*this_conp = NULL;
 
 int		port = 0;
 
+int
 main( argc, argv)
 int	argc;
 char	*argv[];
@@ -128,6 +133,7 @@ do_connection()
 
 	thiscon.keepalive = FALSE;
 	thiscon.trans_cnt = 0;
+	thiscon.remotehost[0] = thiscon.remaddr[0] = '\0';
 	thiscon.rfc931name[0] = thiscon.logbuf[0] = '\0';
 	thiscon.pid = getpid();
 	readbuf.bcp = NULL;
@@ -142,7 +148,6 @@ do_connection()
 	this_conp->scheme = "http";
 	this_conp->con_status = 0;
 
-
 	 /* Start a debug entry if debugging enabled. */
 
 	if ( debug_log) {
@@ -150,6 +155,7 @@ do_connection()
 	}
 
 	get_remote_ip( );
+
 	signal( SIGALRM, wn_timeout );
 	signal( SIGPIPE, client_closed );
 
@@ -158,15 +164,14 @@ do_connection()
 
 	WN_EXTRA_SETUP
 
-	while ( this_conp->trans_cnt < KEEP_ALIVE_MAX) {
-		alarm( await_timeout);
+	while ( this_conp->trans_cnt < KEEPALIVE_MAX) {
 		this_conp->trans_cnt++;
 		await_state = AWAIT_REQUEST;
 		request[0] = '\0';
 		clear_req( );
 		bzero( (char *) inheadp, sizeof( Inheader));
 		bzero( (char *) outheadp, sizeof( Outheader));
-
+		alarm( await_timeout);
 		if ( get_input( request, TRUE) == NULL )
 			client_closed(); /* no one there? */
 
@@ -183,13 +188,11 @@ do_connection()
 		mystrncpy( thisreq.request, request, BIGLEN);
 
 		if ( parse_header( request) == UNKNOWN ) {
-			wn_exit( 0);
+			wn_exit( 2); /* parse_header() == UNKNOWN */
 		}
 		await_state = AWAIT_TRANSACTION;
-		await_timeout = KEEP_ALIVE_TIMEOUT;
+		await_timeout = KEEPALIVE_TIMEOUT;
 		/* await_timeout may have been set for authentication */
-		/* reset it here */
-		alarm( TRANSACTION_TIMEOUT);
 
 #ifdef NO_KEEPALIVE
 		this_conp->keepalive = FALSE;
@@ -210,10 +213,12 @@ do_connection()
 			write_debug( 1, "Keep-Alive: ", buf);
 		}
 
+		alarm( TRANSACTION_TIMEOUT);
 		process_url(&thisreq, inheadp->url_path);
 
+
 		if ( this_rp->status & WN_ABORTED)
-			wn_exit( 2);
+			wn_exit( 2);  /* WN_ABORTED */
 
 		if ( !(this_conp->keepalive)) {
 			break;
@@ -233,6 +238,7 @@ wn_timeout( )
 {
 	char	buf[SMALLLEN];
 
+	this_conp->con_status |= WN_CON_TIMEDOUT;
 	signal( SIGALRM, SIG_DFL);
 	alarm( 20);  	/* Give ourselves 20 seconds to get remote DNS data
 			 * and write log entry.
@@ -244,7 +250,7 @@ wn_timeout( )
 			this_conp->pid, await_state);
 		senderr( "408", err_m[60], buf);
 	}
-	wn_exit( 2);
+	wn_exit( 0);  /* wn_timeout */
 }
 
 
@@ -263,7 +269,7 @@ client_closed( )
 		writelog( this_rp, log_m[18], (await_state == AWAIT_HEADER 
 			? "header" : "transaction"));
 	}
-	wn_exit( 0);
+	wn_exit( 0);  /* client_closed */
 }
 
 void
@@ -274,9 +280,10 @@ char	*url_path;
 	Reqtype	itemtype;
 
 
-	/* These may need clearing if we came from redirect */
 	ip->attributes = ip->filetype = ip->status = 0;
+	/* These may need clearing if we came from redirect */
 
+	ip->status |= WN_HAS_BODY; /* assume it has a body */
 	parse_request( ip, url_path);
 
 	if ( ip->type != RTYPE_FINISHED)
@@ -288,53 +295,75 @@ char	*url_path;
 				&& (itemtype != RTYPE_NOACCESS)
 				&& (itemtype != RTYPE_REDIRECT) 
 				&& (itemtype != RTYPE_FINISHED)) {
+
 		switch (inheadp->method) {
 		case HEAD:
-			itemtype = RTYPE_HEADER;
+			chk_method( WN_M_HEAD);
+			itemtype = RTYPE_HEAD;
 			break;
+
+		case OPTIONS:
+			chk_method( WN_M_OPTIONS);
+			itemtype = RTYPE_OPTIONS;
+			break;
+
+		case GET:
+			chk_method( WN_M_GET);
+			break;
+
 		case CONDITIONAL_GET:
+			inheadp->method = GET;
+			chk_method( WN_M_GET);
+
 			if ( ip->attributes & WN_DYNAMIC) {
-				inheadp->method = GET;
 				break;
 			}				
-			inheadp->method = GET;
-
-			if ( (inheadp->conget & IFMODSINCE)
-				&& !date_cmp( ip, inheadp->inmod_date, TRUE))
-				itemtype = RTYPE_NOT_MODIFIED;
-
-			if ( (inheadp->conget & IFUNMODSINCE)
-				&& date_cmp( ip, inheadp->inmod_date, TRUE)) {
-				itemtype = RTYPE_PRECON_FAILED;
-				break;
-			}
 
 			if ( inheadp->conget & IFNMATCH) {
-				if ( streq( inheadp->etag, ip->etag))
+				if ( etag_match( ))
 					itemtype = RTYPE_NOT_MODIFIED;
+				break;
 			}
 
 			if ( inheadp->conget & IFMATCH) {
-				if ( !streq( inheadp->etag, ip->etag))
+				if ( !etag_match( ))
 					itemtype = RTYPE_PRECON_FAILED;
+				break;
 			}
 
 			if ( (inheadp->conget & IFRANGE) && *inheadp->etag) {
-				if ( !streq( inheadp->etag, ip->etag))
+				if ( !etag_match( ))
 					ip->filetype &= ~(WN_RFC_BYTERANGE
 						+ WN_BYTERANGE
 						+ WN_LINERANGE);
+				break;
 			}
 			else if ( (inheadp->conget & IFRANGE) && 
 				(date_cmp( ip, inheadp->inmod_date, TRUE))) {
 				ip->filetype &= ~(WN_RFC_BYTERANGE
 						+ WN_BYTERANGE
 						+ WN_LINERANGE);
+				break;
+			}
+			if ( (inheadp->conget & IFUNMODSINCE)
+				&& date_cmp( ip, inheadp->inmod_date, TRUE)) {
+				itemtype = RTYPE_PRECON_FAILED;
+				break;
+			}
+
+			if ( (inheadp->conget & IFMODSINCE) 
+				&& !date_cmp( ip, inheadp->inmod_date, TRUE)) {
+				itemtype = RTYPE_NOT_MODIFIED;
+
 			}
 			break;
 
 		case POST:
+			chk_method( WN_M_POST);
+			do_post( inheadp);
+			break;
 		case PUT:
+			chk_method( WN_M_PUT);
 			do_post( inheadp);
 			break;
 		default:
@@ -346,7 +375,6 @@ char	*url_path;
 		case RTYPE_FINISHED:
 			break;
 		case RTYPE_FILE:
-			chk_method( GET);
 			if ( ip->attributes & WN_CGI )
 				cgi_env( ip, FALSE);
 
@@ -356,7 +384,6 @@ char	*url_path;
 				sendbin( ip);
 			break;
 		case RTYPE_MARKLINE:
-			chk_method( GET);
 			if ( ip->attributes & WN_CGI )
 				cgi_env( ip, FALSE);
 
@@ -375,7 +402,6 @@ char	*url_path;
 		case RTYPE_GSEARCH:
 		case RTYPE_CONTEXTSEARCH:
 		case RTYPE_LINESSEARCH:
-			chk_method( GET);
 			ip->attrib2 |= WN_ISSEARCH;
 			sendgrep( ip);
 			break;
@@ -383,48 +409,49 @@ char	*url_path;
 		case RTYPE_KSEARCH:
 		case RTYPE_TKSEARCH:
 		case RTYPE_FIELDSEARCH:
-			chk_method( GET);
 			ip->attrib2 |= WN_ISSEARCH;
 			cache_search( ip);
 			break;
 		case RTYPE_ISEARCH:
-			chk_method( GET);
 			ip->attrib2 |= WN_ISSEARCH;
 			if ( ip->attributes & WN_CGI )
 				cgi_env( ip, FALSE);
 			send_isearch( ip);
 			break;
 		case RTYPE_LISTSEARCH:
-			chk_method( GET);
 			ip->attrib2 |= WN_ISSEARCH;
 			list_search( ip);
 			break;
 		case RTYPE_IMAGEMAP:
-			chk_method( GET);
 			image( );
 			break;
 		case RTYPE_INFO:
-			chk_method( GET);
 			sendinfo( ip);
 			break;
-		case RTYPE_HEADER:
+		case RTYPE_HEAD:
+			ip->status &= ~(WN_HAS_BODY);
 			http_prolog( );
 			writelog( ip, log_m[6], "");
 			break;
+		case RTYPE_OPTIONS:
+			do_options( ip);
+			break;
 		case RTYPE_REDIRECT:
 			sendredirect( ip, "301 Moved Permanently", 
 				outheadp->redirect);
 			break;
 		case RTYPE_NOT_MODIFIED:
 			strcpy( outheadp->status, "304 Not Modified");
+			ip->status &= ~(WN_HAS_BODY);
 			http_prolog( );
 			strcpy( ip->length, "0");
 			writelog( ip, log_m[5], "");
 			break;
 		case RTYPE_NO_AUTH:
 			break;
+
 		case RTYPE_PRECON_FAILED:
-			senderr( PRECON_STATUS, out_m[5], ip->filepath);
+			senderr( PRECON_FAILED_STATUS, out_m[5], ip->filepath);
 			ip->type = RTYPE_FINISHED;
 			break;
 
@@ -432,19 +459,20 @@ char	*url_path;
 			if ( *(dir_p->noaccess_url) && 
 				*dir_p->noaccess_url &&
 				!streq( ip->relpath, dir_p->noaccess_url)) {
-				sendredirect( ip, "302 Moved Temporarily", 
+				sendredirect( ip, "301 Moved Permanently", 
 					dir_p->noaccess_url);
 				break;
-			/* Falls through */
 			}
+			/* Falls through */
 		case RTYPE_DENIED:
 			if ( 	*dir_p->cantstat_url &&
 				!streq( ip->relpath, dir_p->cantstat_url)) {
-				sendredirect( ip, "302 Moved Temporarily", 
+				sendredirect( ip, "301 Moved Permanently", 
 					dir_p->cantstat_url);
 				break;
-			/* Falls through */
 			}
+			/* Falls through */
+
 		case RTYPE_UNCHECKED:
 		default:
 #ifdef DENYHANDLER
@@ -457,9 +485,8 @@ char	*url_path;
 					send_out_fd( fileno( fp));
 					pclose( fp);
 					sprintf( outheadp->status,
-						"%s %s", DENYSTATUS, out_m[0]);
-					writelog( this_rp,
-						out_m[0], ip->filepath);
+						"%.16s %.100s", DENYSTATUS, out_m[0]);
+					writelog( this_rp, out_m[0], ip->filepath);
 					ip->type = RTYPE_FINISHED;
 					ip->status |= WN_ERROR;
 					return;
@@ -473,6 +500,14 @@ char	*url_path;
 			senderr( DENYSTATUS, out_m[0], ip->filepath);
 			ip->type = RTYPE_FINISHED;
 	}
+
+	if ( this_conp->chunk_status & WN_USE_CHUNK) {
+		/* if chunking end end it */
+		end_chunking();
+	}
+
+
+	ip->type = RTYPE_FINISHED;
 	return;
 
 	/*
@@ -509,20 +544,30 @@ char	*req;
 			cp++;
 	}
 
-	if ( !*cp )
+	if ( !*cp ) {
 		ih->protocol =  HTTP0_9;
-	else if ( strncasecmp( cp, "HTTP/1.1", 8) == 0 )
-		ih->protocol =  HTTP1_1;
-	else
-		ih->protocol =  HTTP1_0;
-
-
+		this_conp->keepalive = FALSE;
+	}
+	else if ( strncasecmp( cp, "HTTP/1.", 7) == 0 ) {
+		if ( *(cp + 7) == '0' ) {
+			ih->protocol =  HTTP1_0;
+			this_conp->keepalive = FALSE;
+		}
+		else {
+			ih->protocol =  HTTP1_1;
+			this_conp->keepalive = TRUE;
+		}
+	}
+	else {
+		senderr( "505", err_m[125], cp);
+		wn_exit( 2);  /* 505 HTTP version not supported */
+	}
 	mystrncpy( ih->url_path, req, BIGLEN);
 
 	if ( streq( method, "GET")) {
 		ih->method = GET;
 		if ( ih->protocol != HTTP0_9 ) 
-			get_headers( ih);
+			get_header( ih);
 		return	(ih->method);
 
 	}
@@ -532,7 +577,7 @@ char	*req;
 			return	(ih->method = UNKNOWN);
 		}
 		else {
-			get_headers( ih);
+			get_header( ih);
 			return	(ih->method = POST);
 		}
 	}
@@ -542,36 +587,78 @@ char	*req;
 			return	(ih->method = UNKNOWN);
 		}
 		else {
-			get_headers( ih);
+			get_header( ih);
 			return	(ih->method = PUT);
 		}
 	}
 	if ( streq( method, "HEAD") ) {
-		get_headers( ih);
+		get_header( ih);
 		return	(ih->method = HEAD);
 	}
-	else {
-		senderr( "501", err_m[116], "");
-		return	(ih->method = UNKNOWN);
+
+	if ( streq( method, "OPTIONS") ) {
+		get_header( ih);
+		return	(ih->method = OPTIONS);
 	}
+
+	if ( streq( method, "TRACE") ) {
+		do_trace();
+		return	(ih->method = TRACE);
+	}
+
+
+	senderr( "501", err_m[119], "");
+	return	(ih->method = UNKNOWN);
 }
 
 
 
 static void
-get_headers( ih)
+do_trace( )
+{
+	register char	*cp;
+	char	headerline[BIGLEN];
+
+	this_conp->keepalive = FALSE;
+	this_rp->content_type = "message/http";
+
+	http_prolog();
+
+	send_text_line( this_rp->request);
+	send_text_line( "\r\n");
+
+	while ( get_input( headerline, TRUE) != NULL) {
+		if ( !*headerline) {	/* Blank line, end of headers */
+			send_text_line( "\r\n");
+			break;
+		}
+		if ( strncasecmp( headerline, "Host:", 5) == 0 ) {
+			cp = headerline + 5;
+			while ( isspace( *cp))
+				cp++;
+			mystrncpy( inheadp->host_head,  cp, SMALLLEN);
+		}
+		send_text_line( headerline);
+		send_text_line( "\r\n");
+	}
+
+	writelog( this_rp, log_m[21], inheadp->host_head);
+	return;
+}
+
+static void
+get_header( ih)
 Inheader	*ih;
 {
-	register char	*cp, *cp2;
+	register char	*cp;
 	char	headerline[BIGLEN];
 	int	acceptlen,
 		cookielen;
 
 	acceptlen = cookielen = 0;
-	this_conp->keepalive = FALSE;
 
 	while ( get_input( headerline, FALSE) != NULL) {
-		if ( !*headerline)	/* Blank line, end of header */
+		if ( !*headerline)	/* Blank line, end of headers */
 			return;
 
 		if ( strncasecmp( headerline, "Accept:", 7) == 0 ) {
@@ -629,19 +716,12 @@ Inheader	*ih;
 		}
 
 		if ( strncasecmp( headerline, "Authorization:", 14) == 0 ) {
-			/* For digest use the URI from URI header */
-			if ( ( cp = strstr( headerline, "uri=\"")) ||
-					(cp = strstr( headerline, "URI=\""))) {
-				cp2 = ih->url_path;
-				cp += 5;
-				while( *cp && ( *cp != '"'))
-					*cp2++ = *cp++;
-				*cp2 = '\0';
-			}
 			cp = headerline + 14;
 			while ( isspace( *cp))
 				cp++;
 			mystrncpy( ih->authorization, cp, MIDLEN);
+			mystrncpy( ih->auth_url_path, ih->url_path, MIDLEN);
+
 			continue;
 		}
 
@@ -666,6 +746,15 @@ Inheader	*ih;
 			mystrncpy( ih->encoding, cp, SMALLLEN);
 			continue;
 		}
+		if ( strncasecmp( headerline, "Transfer-encoding:", 18) == 0 ){
+			cp =  headerline + 18;
+			while ( isspace( *cp))
+				cp++;
+			strlower( cp);
+			if ( streq( cp, "chunked"))
+				ih->attrib |= INPUT_CHUNKED;
+			continue;
+		}
 		if ( strncasecmp( headerline, "Host:", 5) == 0 ) {
 			cp = headerline + 5;
 			while ( isspace( *cp))
@@ -680,9 +769,14 @@ Inheader	*ih;
 			mystrncpy( ih->referrer,  cp, MIDLEN);
 			continue;
 		}
-		if ( strncasecmp( headerline, 
-				 "Connection: Keep-Alive", 22) == 0 ) {
-			this_conp->keepalive = TRUE;
+		if ( strncasecmp( headerline, "Connection:", 11) == 0 ) {
+			cp = headerline + 11;
+			while ( isspace( *cp))
+				cp++;
+			if ( strncasecmp( cp, "keep-alive", 10) == 0 )
+				this_conp->keepalive = TRUE;
+			if ( strncasecmp( cp, "close", 5) == 0 )
+				this_conp->keepalive = FALSE;
 			continue;
 		}
 		if ( strncasecmp( headerline, "from:", 5) == 0 ) {
@@ -708,8 +802,20 @@ Inheader	*ih;
 			mystrncpy( ih->ua, cp, SMALLLEN); 
 			continue;
 		}
+		if ( strncasecmp( headerline, "Expect:", 7) == 0 ) {
+			cp = headerline + 7;
+			while ( isspace( *cp))
+				cp++;
+			if ( strncasecmp( cp, "100-Continue", 12) == 0 ) {
+				send_text_line( HTTPVERSION);
+				send_text_line( "100 Continue\r\n\r\n");
+				flush_outbuf();
+				alarm( KEEPALIVE_TIMEOUT);
+			}
+			continue;
+		}
 		if ( strncasecmp( headerline, "If-Modified-Since:", 18) == 0) {
-						/* it's a Fielding wart */
+						/* it's a wart */
 			if ( ih->method == POST)
 				continue;
 			cp = headerline + 18;
@@ -723,7 +829,7 @@ Inheader	*ih;
 
 		if ( strncasecmp( headerline, "If-Unmodified-Since:", 20)
 									== 0) {
-						/* another Fielding wart */
+						/* another wart */
 			if ( ih->method == POST)
 				continue;
 			cp = headerline + 20;
@@ -778,52 +884,213 @@ Inheader	*ih;
 }
 
 
+/*
+ * static int etag_match( ) returns TRUE if this_rp->etag matches anything in
+ * the comma separated list of quoted strings inheap->etag.  Otherwise
+ * returns FALSE.
+ */
+
+static int /*bool*/
+etag_match( )
+{
+	register char	*cp,
+			*cp2,
+			*cp3;
+
+	cp2 = cp = inheadp->etag;
+	while ( (cp = strchr( cp2, ',')) != NULL) {
+		*cp++ = '\0';
+		while ( isspace( *cp2)) /* skip leading LWS */
+			cp2++;
+
+		if ( *cp2 == '"')	/* skip leading quote */
+			cp2++;
+		if ( (cp3 = strchr( cp2, '"')) != NULL)
+			*cp3 = '\0';	/* stop before ending quote */
+		if ( streq( this_rp->etag, cp2))
+			return TRUE;
+
+		cp2 = cp;
+	}
+	while ( isspace( *cp2)) /* skip leading LWS */
+		cp2++;
+
+	if ( streq( cp2, "*"))
+		return TRUE;
+
+	if ( *cp2 == '"')	/* skip leading quote */
+		cp2++;
+	if ( (cp3 = strchr( cp2, '"')) != NULL)
+		*cp3 = '\0';	/* stop before ending quote */
+	if ( streq( this_rp->etag, cp2))
+		return TRUE;
+	else
+		return FALSE;
+}
+
 static void
 do_post( ih)
 Inheader	*ih;
 {
 #ifndef FORBID_CGI
 	long	len;
-	int	c;
+	int	n,
+		c,
+		error = 0;
+
 	FILE	*fp;
+
 	static int	num = 0;
+
 #ifndef STANDALONE
 	umask( 077);
 #endif
-	len = (long) 0;
-	sprintf( ih->tmpfile, "%.200s/WNpost-%x-%x-%x",
+	len = 0;
+	sprintf( ih->tmpfile, "%.200s/WNpost-%lx-%x-%x",
 				wn_tmpdir, time( NULL), this_conp->pid, num++);
 
 	if ( (fp = fopen( ih->tmpfile, "w")) == (FILE *) NULL ) {
-		senderr( SERV_ERR, err_m[53], ih->tmpfile);
-		wn_exit( 2); /* senderr: SERV_ERR */
-	}
-
-
-	if ( *(ih->length))
-		len = atol( ih->length);
-	else {
-		senderr( "403", err_m[57], "");
-		wn_exit( 0);
-	}
-
-	if ( (len > MAX_POST) || (len < 0)) {
-		senderr( "403", err_m[74], "");
-		wn_exit( 2);
+		remk_postdir();
+		if ( (fp = fopen( ih->tmpfile, "w")) == (FILE *) NULL ) {
+			senderr( SERV_ERR, err_m[53], ih->tmpfile);
+			wn_exit( 2); /* senderr: SERV_ERR */
+		}
 	}
 
 	/* Timeout after TRANSACTION_TIMEOUT seconds */
 	alarm( TRANSACTION_TIMEOUT);
 
-	while ( len && (c = wn_getc( )) != EOF) {
-		len--;
-		putc( c, fp);
+	if ( ih->attrib & INPUT_CHUNKED ) {
+		while ( (n = get_chunk( fp))) {
+			len += n;
+			if ( len > MAX_POST) {
+				senderr( REQ_TOO_LONG_STATUS, err_m[126], "");
+				wn_exit( 2); /* senderr: REQ_TOO_LONG_STATUS */
+			}
+		}
+		sprintf( ih->length, "%ld", len);
 	}
+	else if ( *ih->length) {
+		len = atol( ih->length);
+		if ( len > MAX_POST) {
+			senderr( REQ_TOO_LONG_STATUS, err_m[126], "");
+			wn_exit( 2); /* senderr: 413 REQ_TOO_LONG_STATUS */
+		}
+
+		while ( (len > 0) && (c = wn_getc( )) != EOF) {
+			len--;
+			putc( c, fp);
+		}
+
+		if ( len < 0 )
+			error++;
+	}
+	else {
+		error++;
+	}
+
+	if ( error) {
+		senderr( CLIENT_ERR, err_m[57], "");
+		wn_exit( 2);  /* senderr: CLIENT_ERR */
+	}
+
+	if ( len > MAX_POST) {
+		senderr( "413", err_m[74], "");
+		wn_exit( 2);  /* senderr: 413 */
+	}
+
 	fclose( fp);
 #endif
 }
 
 
+static void
+remk_postdir( )
+{
+#ifndef FORBID_CGI
+	struct stat	stat_buf;
+	uid_t		my_id;
+
+	my_id = getuid();
+	if ( lstat( wn_tmpdir, &stat_buf) == -1) {
+		if ( (errno != ENOENT) || (mkdir( wn_tmpdir, 0700) != 0) ) {
+			senderr( SERV_ERR,  err_m[122], wn_tmpdir);
+			wn_exit( 2);  /* senderr: SERV_ERR */
+		}
+	}
+	else if ( (!S_ISDIR(stat_buf.st_mode))
+				|| (stat_buf.st_uid != my_id)
+				|| (chmod( wn_tmpdir, 0700) != 0) ) {
+		senderr( SERV_ERR,  err_m[122], wn_tmpdir);
+		wn_exit( 2);  /* senderr: SERV_ERR */
+	}
+#endif
+}
+
+static int
+get_chunk( fp)
+FILE	*fp;
+{
+	int	len = -1,
+		c,
+		n,
+		error = 0;
+
+	char	buf[TINYLEN],
+		trailer[BIGLEN];
+
+	buf[TINYLEN - 1] = '\0';
+	for ( n = 0; n < TINYLEN - 1; n++) {
+		c = wn_getc( );
+		if ( (c  == EOF)) {
+			error++;
+			break;
+		}
+		if ( c == '\r') {
+			if ( (c = wn_getc( )) != '\n') {
+				error++;
+				break;
+			}
+			buf[n] = '\0';
+			break;
+		}
+		buf[n] = (char) c;
+	}
+
+	if ( !error)
+		sscanf( buf, "%x", &len);
+
+	if ( len > MAX_POST) {
+		senderr( "413", err_m[74], "");
+		wn_exit( 2);  /* senderr: 413 */
+	}
+
+	n = len;
+	while ( (n > 0) && (c = wn_getc( )) != EOF) {
+		n--;
+		putc( c, fp);
+	}
+
+	if ( len == 0 ) {	/* get any trailer lines */
+		while ( get_input( trailer, FALSE) != NULL) {
+			if ( !*trailer)/* Blank line, end of headers */
+				break;
+		}
+	}
+	else {
+		if ( (c = wn_getc( )) != '\r')
+			error++;
+		if ( (c = wn_getc( )) != '\n')
+			error++;
+	}
+	if ( error || (len < 0)) {
+		senderr( "400", err_m[118], "");
+		wn_exit( 2);  /* senderr: 406 */
+	}
+	return len;
+}
+
+
 static int
 wn_getc()
 {
@@ -848,22 +1115,24 @@ wn_getc()
 /*
  * static char *get_input()
  * Returns NULL if no input (client quit), otherwise places next line
- * of input in "line".  Any CRLF are removed.  Thus *line == '\0' indicates
- * end of headers.  This function reads lines from a buffer thiscon.bufp.
+ * of input in "line" and returns "line".  Any CRLF are removed.
+ * Thus *line == '\0' indicates end of headers.
+ *
+ * The function get_input() reads lines from a buffer thiscon.bufp.
  * If the buffer is empty or does not contain a complete line it reloads
  * it.  It looks ahead to see if the line continues (i.e. next line starts
  * with white space.  The lookahead is only done for header lines
  * (rline = FALSE) not for the request line.
- * The only possibilities for "line" are request and headerline.
- * Both have size BIGLEN.
+ * The only possibilities for "line" are request, trailer, and headerline.
+ * All have size BIGLEN.
  */
 
 static char *
-get_input( line, is_rline)
+get_input( line, no_continuation)
 char	*line;
-int	is_rline; 
+int	no_continuation; 
 {
-	int	llen,
+	int	llen = 0,
 		n = 0;
 	Inbuffer	*bp;
 
@@ -874,9 +1143,9 @@ int	is_rline;
 		bp->bcp = bp->buffer;
 
 	*line = '\0';
-	llen = 0;
+
 	while ( TRUE) {
-		if ( reset_buf( bp) == FALSE ) {  /* the buffer is empty */
+		if ( reset_buf( bp) == FALSE ) {  /* buffer is empty */
 			n = load_inbuf( bp);
 			if ( n <= 0) {
 				if ( debug_log) {
@@ -892,20 +1161,29 @@ int	is_rline;
 		if ( ( cp = strchr( bp->bcp, '\n')) == NULL) {
 			/* an incomplete header has been read */
 			/* put what there is into line */
-			/* if header contains nulls, stuff will be discarded */
-
 			bp->buffer[bp->cur_sz] = '\0';
 			mystrncat( line, bp->bcp, BIGLEN - llen);
 			llen = strlen( line);
-			bp->cur_sz = 0;
-			bp->bcp = bp->buffer;
-			bp->buffer[0] ='\0';
+
+			cp = bp->bcp;
+			while ( *cp)
+				cp++;
+
+			if ( (cp < &(bp->buffer)[bp->cur_sz])
+							&& (cp > bp->bcp) ) {
+				bp->bcp = ++cp;
+			}
+			else {
+				bp->cur_sz = 0;
+				bp->bcp = bp->buffer;
+				bp->buffer[0] ='\0';
+			}
 			/* then read more */
 
 			n = load_inbuf( bp);
 			if ( n <= 0 ) {
-				writelog( this_rp,  err_m[66], line);
-				wn_exit( 2);
+				senderr( CLIENT_ERR, err_m[66], line); 
+				wn_exit( 2); /* senderr: CLIENT_ERR */
 			}
 			continue;
 		}
@@ -913,9 +1191,15 @@ int	is_rline;
 		/* cp now points to next NL in the in_bufer */
 		*cp++ = '\0';
 
+
 		mystrncpy( line + llen, bp->bcp, BIGLEN - llen);
 		llen = strlen( line);
 
+		if ( llen >= BIGLEN-3 ) {
+			senderr( URI_TOO_LONG_STATUS, err_m[124], line); 
+			wn_exit( 2); /* senderr: 414 URI too long */
+		}
+
 		if ( (llen > 0) && (line[llen-1] == '\r'))
 			line[--llen] = '\0';
 
@@ -926,9 +1210,10 @@ int	is_rline;
 		if ( debug_log)
 			write_debug(1, " -> ", line);
 
-		if ( !*line || is_rline)
+		if ( (!*line) || no_continuation) {
 			return ( line);
 			/* Don't allow continuation for request line */
+		}
 
 
 		if ( ! chk_continue( bp) ) 
@@ -964,9 +1249,12 @@ Inbuffer	*bp;
 			size = ( size < 0 ? 0 : size);
 			mymemcpy( base, cp, size);
 			bp->bcp = cp = base;
+			bp->cur_sz = size;
 		}
+
 		n = WN_read( (fileno( stdin)), (base + size), 
 				((INBUFFSIZE - 4) - size));
+
 		if ( n > 0 ) {
 			size += n;
 			bp->cur_sz = size;
@@ -977,6 +1265,7 @@ Inbuffer	*bp;
 			bp->bcp = bp->buffer;
 			*base ='\0';
 		}
+
 		return n;
 }
 
@@ -1018,7 +1307,6 @@ static int
 reset_buf( bp)
 Inbuffer	*bp;
 {
-
 	if ( bp->bcp - bp->buffer >= bp->cur_sz) {
 		/* it's empty */
 		bp->bcp = bp->buffer;
@@ -1055,12 +1343,13 @@ http_prolog( )
 			clock2;
 	Request		*ip;
 	char		buf[CACHELINE_LEN],
+			mod_date[SMALLLEN],	
+					/* Last-Modified HTTP header line */
 			datebuf[2*TINYLEN];
 	unsigned	unbuffered;
 
 
 	ip = this_rp;
-
 	if ( ip->status & WN_PROLOGSENT )
 		return FALSE;
 	ip->status |= WN_PROLOGSENT;
@@ -1072,16 +1361,18 @@ http_prolog( )
 		return TRUE;
 	}
 
-	if ( *outheadp->status)
-		sprintf( buf, "%s %s\r\n",
-				HTTPVERSION, outheadp->status);
-	else {
-		strcpy( buf, HTTPVERSION);
-		strcat( buf, " 200 OK\r\n");
+	send_text_line( HTTPVERSION);
+	if ( *outheadp->status) {
+		send_text_line(  outheadp->status);
+		send_text_line(  "\r\n");
 	}
+	else {
+		send_text_line( "200 OK\r\n");
+	}
+
+	sprintf( buf, "Server: %.32s\r\n", VERSION);
 	send_text_line( buf);
-	sprintf( buf, "Server: %s\r\n", VERSION);
-	send_text_line( buf);
+
 		/* Find date and serve the HTTP Date header */
 	time(&clock);
 	gmt = gmtime(&clock);
@@ -1089,37 +1380,49 @@ http_prolog( )
 			"Date: %a, %d %h %Y %T GMT\r\n", gmt);
 	send_text_line( datebuf);
 	if ( strncmp( outheadp->status, "204", 3) == 0 ) {
-		send_text_line( "Content-length: 0\r\n\r\n");
+		/* 204 No Response (null action) */
+		end_headers();
 		return TRUE;
 	}
 
+	if  ((ip->attrib2 & (WN_ISSEARCH + WN_FILEMOD)) ||
+			(ip->attributes & 
+				(WN_CGI + WN_PARSE + WN_FILTERED + WN_DYNAMIC))) {
+		/* If any of these hold we don't really know the length */
+		*ip->length = *outheadp->md5 = '\0';
+	}
+
 	if ( strncmp( outheadp->status, "401", 3) == 0 ) {
 		send_text_line( outheadp->list);
-		if ( this_conp->keepalive)
-			send_text_line( "Connection: Keep-Alive\r\n");
+		send_keepalive();
 		await_timeout = AUTHENT_TIMEOUT;
-		sprintf( buf, "Content-length: %s\r\n\r\n", ip->length);
-		send_text_line( buf);
+		if ( *ip->length) {
+			sprintf( buf, "Content-length: %.64s\r\n", ip->length);
+			send_text_line( buf);
+		}
+		end_headers();
 		return TRUE;
 	}
 
 	if ( *outheadp->redirect) {
 		send_text_line( "Content-type: text/html\r\n");
-		if ( this_conp->keepalive)
-			send_text_line( "Connection: Keep-Alive\r\n");
+		send_keepalive();
 		send_text_line( "Location: ");
 		send_text_line( outheadp->redirect);
-		send_text_line( "\r\nContent-length: 0\r\n\r\n");
+		send_text_line( "\r\n"); /* end of redirect */
+		end_headers();
 		return TRUE;
 	}
 
-	if ( ip->attributes & (WN_DYNAMIC)) {
+
+	if ( *outheadp->allow)
+		send_text_line( outheadp->allow);
+
+	if ( ip->attributes & WN_DYNAMIC) {
 		ip->mod_time = (time_t) 0;
-		*ip->mod_date = '\0';
-		*ip->length = '\0';
 	}
 
-	if ( ip->attributes & (WN_NOCACHE)) {
+	if ( ip->attributes & WN_NOCACHE) {
 		send_text_line( "Pragma: no-cache\r\n");
 		if ( inheadp->protocol ==  HTTP1_1)
 			send_text_line( "Cache-control: no-cache\r\n");
@@ -1127,20 +1430,22 @@ http_prolog( )
 		
 	if ( ip->mod_time) {
 		gmt = gmtime(&ip->mod_time);
-		strftime( ip->mod_date, SMALLLEN,
-				"Last-modified: %a, %d %h %Y %T GMT\r\n", gmt);
+		strftime( mod_date, SMALLLEN,
+				"Last-Modified: %a, %d %h %Y %T GMT\r\n", gmt);
+		send_text_line( mod_date);
 
-#if 0
-		if ( (!(ip->attributes & (WN_FILTERED + WN_PARSE)))
-					((clock - ip->mod_time) > 2))  {
-			sprintf( buf, "ETag: \"%s\"\r\n", ip->etag);
+		/* No ETag for filtered, parsed, or info documents */
+
+		if ( ip->filetype & (WN_BYTERANGE + WN_LINERANGE)) 
+			; /* do nothing */
+		else if ( ip->attributes & (WN_FILTERED + WN_PARSE))
+			; /* do nothing */
+		else if ( (ip->type == RTYPE_INFO) || !*ip->length )
+			; /* do nothing */
+		else if ((clock - ip->mod_time) > 2 )  {
+			sprintf( buf, "ETag: \"%.64s\"\r\n", ip->etag);
 			send_text_line( buf);
 		}
-#endif
-	}
-
-	if ( *ip->mod_date) {
-		send_text_line( ip->mod_date);
 	}
 
 	if ( ip->maxage && *ip->maxage) {
@@ -1159,14 +1464,14 @@ http_prolog( )
 		}
 		else {
 			clock2 = clock + atol( ip->maxage);
-			sprintf( buf, "Cache-Control: max-age=%s\r\n",
+			sprintf( buf, "Cache-Control: max-age=%.64s\r\n",
 					ip->maxage);
 			send_text_line( buf);
 		}
 	}
 
 	if ( ip->expires && *ip->expires) {
-		sprintf( buf, "Expires: %s\r\n", ip->expires);
+		sprintf( buf, "Expires: %.64s\r\n", ip->expires);
 		send_text_line( buf);
 	}
 	else if ( *outheadp->expires) {
@@ -1181,56 +1486,82 @@ http_prolog( )
 
 	if ( inheadp->method == CONDITIONAL_GET) {
 		/* It's not modified */
-		if ( this_conp->keepalive)
-			send_text_line( "Connection: Keep-Alive\r\n");
-		send_text_line( "\r\n");
+		send_keepalive();
+		end_headers();
 		return TRUE;
 	}
 
 	if ( ip->content_type) {
-		sprintf(buf, "Content-type: %s\r\n", ip->content_type);
+		sprintf(buf, "Content-type: %.256s\r\n", ip->content_type);
 		send_text_line( buf);
 	}
 
-	if ( *ip->length && !(ip->attributes 
-				& (WN_PARSE + WN_FILTERED + WN_DYNAMIC))) {
-		sprintf( buf, "Content-length: %s\r\n", ip->length);
-		send_text_line( buf);
-		if ( *outheadp->range )
-			send_text_line( outheadp->range);
+	if ( *outheadp->md5 && *ip->length && (!*outheadp->range) ) {
+		if ( dir_p->cmod_time <= ip->mod_time ) {
+			*outheadp->md5 = '\0';
+			logerr( err_m[123], ip->cachepath);
+		}
+		else
+			send_text_line( outheadp->md5);
+	}
 
-		if ( !(ip->filetype & WN_BYTERANGE) && !*(outheadp->status)) {
-			/* !*(outheadp->status)) means status 200 */
-			send_text_line( "Accept-Ranges: bytes\r\n"); 
+	if  ( (inheadp->method == HEAD) && *ip->length ) {
+		send_text_line( "Accept-Ranges: bytes\r\n"); 
+		sprintf( buf, "Content-length: %.64s\r\n", ip->length);
+		send_text_line( buf);
+	}
+	else if ( (ip->status & WN_HAS_BODY) ) {
+		if ( !*ip->length ) {		/* chunk if HTTP/1.1 */
+			if (inheadp->protocol ==  HTTP1_0) {
+				this_conp->keepalive = FALSE;
+				/* can't chunk so close connection */
+			}
+			else if ( !(ip->filetype & WN_RFC_BYTERANGE)) {
+				send_text_line(
+					"Transfer-Encoding: chunked\r\n");
+				this_conp->chunk_status |= WN_USE_CHUNK;
+			}
+
+		}
+		else {	/* not dynamic and has length */
+			sprintf( buf, "Content-length: %.64s\r\n", ip->length);
+			send_text_line( buf);
+			if ( *outheadp->range ) {
+				send_text_line( "Content-Range: bytes ");
+				send_text_line( outheadp->range);
+				send_text_line( "\r\n");
+			}
+			if ( !(ip->filetype & WN_BYTERANGE)
+					&& !*(outheadp->status)) {
+
+				/* !*(outheadp->status) means status 200 */
+				send_text_line( "Accept-Ranges: bytes\r\n"); 
+			}
 		}
 	}
-	else if ( inheadp->method != HEAD)
-		this_conp->keepalive = FALSE;
 
-	if ( this_conp->keepalive) {
-		send_text_line( "Connection: Keep-Alive\r\n");
-	}
+	send_keepalive();
 			
 	if ( ip->encoding && *ip->encoding) {
-		sprintf( buf, "Content-encoding: %s\r\n",
+		sprintf( buf, "Content-encoding: %.64s\r\n",
 			ip->encoding);
 		send_text_line( buf);
 	}
 
 #ifdef USE_TITLE_KEYWORD_HEADERS
 	if ( ip->title && *ip->title) {
-		sprintf( buf, "Content-description: %s\r\n", ip->title);
-		/* formally Title: */
+		sprintf( buf, "Content-description: %.500s\r\n", ip->title);
+		/* formerly Title: */
 		send_text_line( buf);
 	}
 
 	if ( ip->keywords && *ip->keywords) {
-		sprintf( buf, "Keywords: %s\r\n", ip->keywords);
+		sprintf( buf, "Keywords: %.500s\r\n", ip->keywords);
 		send_text_line( buf);
 	}
 #endif
 	if ( dir_p->dir_owner && *dir_p->dir_owner) {
-		sprintf( buf,"Link: <%s>; rev=\"Made\"\r\n",
+		sprintf( buf,"Link: <%.100s>; rev=\"Made\"\r\n",
 			dir_p->dir_owner);
 		send_text_line( buf);
 	}
@@ -1238,25 +1569,86 @@ http_prolog( )
 	if ( *outheadp->list ) {
 		send_text_line( outheadp->list);
 	}
-	send_text_line( "\r\n");
-
 	ip->attributes |= unbuffered;
+
+	end_headers();
 	return TRUE;
 }
 
 
 static void
-chk_method( meth)
-Methodtype meth;
+end_headers()
+{
+	send_text_line( "\r\n");
+	if ( this_conp->chunk_status & WN_USE_CHUNK)
+		this_conp->chunk_status |= WN_START_CHUNK;
+		/* start chunking */
+}
+
+static void
+send_keepalive()
 {
 
-	if ( inheadp->method != meth) {
-		if ( (meth == GET) && ( this_rp->attributes & WN_POST_OK) &&
-				((inheadp->method == POST) 
-				|| (inheadp->method == PUT)) )
+	if ( (this_conp->keepalive) && (inheadp->protocol ==  HTTP1_0) )
+		send_text_line( "Connection: Keep-Alive\r\n");
+
+	if ( (!this_conp->keepalive) && (inheadp->protocol ==  HTTP1_1))
+		send_text_line( "Connection: close\r\n");
+
+}
+
+
+static void
+chk_method( allow)
+unsigned	allow;
+{
+
+	if ( this_rp->allowed & allow )
 			return;		/* PUT or POST ok */
 
+	else {
+		do_allow_header( );
 		senderr( "405", err_m[117], "");
 		wn_exit( 2);  /* senderr: 405 method not allowed */
 	}
 }
+
+static void
+do_options( ip)
+Request	*ip;
+{
+	do_allow_header( );
+	mystrncpy( outheadp->list, "Content-length: 0\r\n", 30);
+
+	ip->status &= ~(WN_HAS_BODY);
+	http_prolog( );
+	writelog( ip, log_m[22], "");
+}
+
+
+static void
+do_allow_header( )
+{
+#ifdef FORBID_CGI
+	this_rp->allowed &= ~(WN_M_PUT + WN_M_POST);
+#else
+	if (serv_perm & WN_FORBID_EXEC)
+		this_rp->allowed &= ~(WN_M_PUT + WN_M_POST);
+#endif
+	mystrncat( outheadp->allow, "Allow: TRACE", 32); 
+
+	if ( this_rp->allowed & WN_M_HEAD)
+		mystrncat(outheadp->allow, ", HEAD", 10); 
+	if ( this_rp->allowed & WN_M_GET)
+		mystrncat(outheadp->allow, ", GET", 10); 
+	if ( this_rp->allowed & WN_M_POST)
+		mystrncat(outheadp->allow, ", POST", 10); 
+	if ( this_rp->allowed & WN_M_OPTIONS)
+		mystrncat(outheadp->allow, ", OPTIONS", 10); 
+	if ( this_rp->allowed & WN_M_PUT)
+		mystrncat(outheadp->allow, ", PUT", 10); 
+	mystrncat( outheadp->allow, "\r\n", 5);
+}
+
+
+
