@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wn/wn.h
-    Version 1.18.3
+    Version 2.0.2
     
-    Copyright (C) 1996, 1997  <by John Franks>
+    Copyright (C) 1996-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -37,7 +37,6 @@
 #include "common.h"
 #include "extra.h"
 
-#define HTTPVERSION "HTTP/1.0"
 #define WN_HTML_MARK	"WN_mark"
 #define	ACCEPTLEN (2048)
 #define MAXDIRLEN (256)
@@ -54,10 +53,12 @@
 #define MAXHOSTNAMELEN	(256)
 #endif
 
-#define CLIENT_ERR	"400"
-#define DENYSTATUS	"404"
-#define PRECON_STATUS	"412"
-#define SERV_ERR	"500"
+#define CLIENT_ERR		"400"
+#define DENYSTATUS		"404"
+#define PRECON_FAILED_STATUS	"412"
+#define REQ_TOO_LONG_STATUS	"413"
+#define URI_TOO_LONG_STATUS	"414"
+#define SERV_ERR		"500"
 
 extern FILE	*safer_popen();
 
@@ -72,6 +73,7 @@ char * WN_CONST	out_m[];
 
 
 extern char	rootdir[],
+		wn_empty[],
 		wn_tmpdir[],
 		wnlogfile[],
 		errlogfile[],
@@ -79,15 +81,13 @@ extern char	rootdir[],
 		vhostfile[],
 		cfname[],
 		afname[],
-		empty[],
 		hostname[],
-		remotehost[],
-		remaddr[],
 		**mtypelist,
 		**suflist,
 		*mystrncpy(),
 		*mystrncat(),
 		*mymemcpy(),
+		*get_parse_line(),
 		*strlower();
 
 extern unsigned	alarm(),
@@ -99,6 +99,8 @@ extern int	errno,
 		port,
 		admin_mode,
 		debug_log,
+		user_id,
+		group_id,
 		chop(),
 		wild_match(),
 		modified(),
@@ -111,7 +113,7 @@ extern int	errno,
 		set_show(),
 		get_parse_token(),
 		myisalnum(),
-		eval_if(),
+		do_ifexpr(),
 		read_cache_file();
 
 extern void	wn_init(),
@@ -129,7 +131,6 @@ extern void	wn_init(),
 		chk_cntrl(),
 		check_perm(),
 		exec_ok(),
-		exit(),
 		clear_req(),
 		do_standalone(),
 		do_connection(),
@@ -194,7 +195,8 @@ typedef enum {
 	RTYPE_FIELDSEARCH,
 	RTYPE_LISTSEARCH,
 	RTYPE_INFO,
-	RTYPE_HEADER,
+	RTYPE_HEAD,
+	RTYPE_OPTIONS,
 	RTYPE_REDIRECT,
 	RTYPE_NOT_MODIFIED,
 	RTYPE_NO_AUTH,
@@ -237,7 +239,6 @@ typedef struct Request {
 		param_field[SMALLLEN],	/* Stuff after ';' before '=' in URL */
 		*param_value,		/* Stuff after '=' before '?' in URL */
 		pathinfo[MIDLEN],	/* PATH_INFO for CGI */
-		mod_date[SMALLLEN],	/* Last-Modified HTTP header line */
 		authuser[SMALLLEN],     /* username with Basic auth */
 		range[RANGELEN],	/* range from range header */
 		etag[TINYLEN],		/* ETag in ASCII calculated locally */
@@ -255,11 +256,9 @@ typedef struct Request {
 
 	unsigned	attributes,
 			attrib2,
+			allowed,
 			status,
-			filetype,
-
-			cache_uid,
-			cache_gid;
+			filetype;
 
 	int		do_wrap_1st_time;
 
@@ -270,13 +269,22 @@ extern Request		*this_rp;
 
 /* Bits in the Request attributes  and attrib2 are in common.h */
 
+/* Bits in request allowed */
+#define	WN_M_GET		(1)
+#define	WN_M_HEAD		(2)
+#define	WN_M_TRACE		(4)
+#define	WN_M_OPTIONS		(8)
+#define	WN_M_POST		(16)
+#define	WN_M_PUT		(32)
+
 /* Bits in the request status */
 #define	WN_CANT_STAT		(1)
 #define	WN_HAS_BODY		(2)
 #define WN_PROLOGSENT		(4)
 #define WN_CGI_SET		(8)
 #define WN_ABORTED		(16)
-#define WN_ERROR		(32)
+#define WN_MATCH_SENT		(32)
+#define WN_ERROR		(64)
 
 
 /* Bits in the Request filetype */
@@ -321,8 +329,12 @@ typedef struct Dir_info {
 		noaccess_url[MIDLEN/2];
 
 	unsigned	attributes,
-			defattributes;
+			defattributes,
 
+			cache_uid,
+			cache_gid;
+
+	time_t		cmod_time;	/* Cache file modification time */
 } Dir_info;
 
 extern Dir_info	*dir_p;
@@ -342,6 +354,8 @@ typedef struct Cache_entry {
 		*field[NUMFIELDS],	/* user defined fields */
 		*content,
 		*encoding,	/* MIME content-transfer-encoding */
+		*status,
+		*md5,
 		*includes,	/* comma separated include files */
 		*wrappers,	/* comma separated wrapper files */
 		*list_incl,	/* comma separated include list */
@@ -351,8 +365,6 @@ typedef struct Cache_entry {
 		*handler,
 		*maxage,                /* Maxage in ascii seconds */
 		*expires,
-		*dynamic,	/* doc should not be cached if "true" */
-		*cgi,		/* Create CGI environment, "true" or "false" */
 		*url,		/* URL link to remote object */
 		*redirect,	/* URL link to redirected object */
 		*end;
@@ -362,12 +374,15 @@ typedef struct Cache_entry {
 			filetype;
 } Cache_entry;
 
+
 typedef enum { 
 	GET,
 	CONDITIONAL_GET,
 	POST,
 	PUT,
 	HEAD,
+	TRACE,
+	OPTIONS,
 	UNKNOWN
 } Methodtype;
 
@@ -383,9 +398,10 @@ typedef struct Inheader {
 		charset[ACCEPTLEN/4],
 		lang[ACCEPTLEN/4],
 		url_path[BIGLEN],
+		auth_url_path[MIDLEN],
 		content[SMALLLEN],
-		encoding[SMALLLEN],	/* MIME content-transfer-encoding */
-		length[SMALLLEN],
+		encoding[SMALLLEN],	/* MIME content-encoding */
+		length[TINYLEN],
 		referrer[MIDLEN],
 		ua[SMALLLEN],
 		from[SMALLLEN],
@@ -394,18 +410,22 @@ typedef struct Inheader {
 		inmod_date[SMALLLEN],
 		etag[SMALLLEN],
 		range[RANGELEN],
-		tmpfile[MIDLEN];
+		tmpfile[SMALLLEN];
 
 	Methodtype	method;
 	Prottype	protocol;
 
-	unsigned	conget;
+	unsigned	attrib,
+			conget;
 
 } Inheader;
 
 extern Inheader	*inheadp;
 
 
+/* Bits in inheadp->attrib */
+#define	INPUT_CHUNKED	(1)
+
 /* Bits in inheadp->conget */
 
 #define	IFNMATCH	(1)
@@ -421,7 +441,9 @@ typedef struct Outheader {
 		list[BIGLEN],
 		redirect[MIDLEN],
 		expires[SMALLLEN],
+		md5[2*TINYLEN],
 		range[SMALLLEN],
+		allow[SMALLLEN],
 		status[SMALLLEN];
 } Outheader;
 
@@ -446,15 +468,21 @@ typedef struct Connection {
 			trans_cnt;
 
 	char		logbuf[LOGBUFLEN],
-			outbuf[OUT_BUFFSIZE],
+			outbuf[OUT_BUFFSIZE + TINYLEN],
+			remotehost[MAXHOSTNAMELEN],
+			remaddr[20],
 			rfc931name[SMALLLEN],
 			*out_ptr,
+			*chunksize_ptr, /* pointer into outbuf to place */
+                                        /* where chunksize must be inserted */
 			*log_ptr,
 			*scheme;	/* "http" or something else */
 
-	long		bytecount;
+	long		chunksize,
+			bytecount;
 
-	unsigned	con_status;
+	unsigned	con_status,
+			chunk_status;
 
 	Inbuffer	*bufp;
 } Connection;
@@ -462,6 +490,15 @@ typedef struct Connection {
 extern Connection	*this_conp;
 
 
+/* Bits in the Connection con_status */
+#define WN_CON_CGI_SET		(1)
+#define WN_CON_TIMEDOUT		(2)
+
+/* Bits in the Connection chunk_status */
+#define WN_USE_CHUNK		(1)
+#define WN_START_CHUNK		(2)
+#define WN_IN_CHUNK		(4)
+
 
 extern unsigned	serv_perm;
 
