@@ -1,7 +1,7 @@
 /*
     Wn: A Server for the HTTP
     File: wndex/err.h
-    Version 1.18.2
+    Version 1.19.0
     
     Copyright (C) 1996-7  <by John Franks>
 
@@ -52,6 +52,8 @@
 #define ERRMSG29 "\tOut of directory name storage space\n"
 #define ERRMSG30 "\tIgnoring unknown option: \"-%c %s\"\n\n"
 #define ERRMSG31 "\tServeall directive not allowed.\"\n\n"
+#define ERRMSG32 "\tWarning: MD5 digest of parsed, CGI or dynamic document\n\
+\tis forbidden; file is %s\n\n"
 
 #define MSG1	"Wrote cache file %s\n"
 #define MSG2	"Writing index.html file file %s\n"
