@@ -1,7 +1,7 @@
 /*
     Wn: A Server for the HTTP
     File: wndex/init.c
-    Version 1.17.3
+    Version 2.0.3
     
     Copyright (C) 1996, 1997  <by John Franks>
 
@@ -22,10 +22,11 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "wndex.h"
-#include "../wn/version.h"
+#include "version.h"
 
 int	recurse = FALSE,
 	strong_serveall = FALSE,
@@ -50,7 +51,7 @@ char	*argv[];
 	int	c,
 		dflg = FALSE,
 		errflg = FALSE;
-	char	*dir;
+	char	*dir = NULL;
 
 	umask( 033);
 	strcpy( cntlfname, CONTROLFILE_NAME);
