@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wndex/wndex.c
-    Version 1.18.5
+    Version 2.0.3
     
-    Copyright (C) 1995-7  <by John Franks>
+    Copyright (C) 1995-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include "wndex.h"
 #include "reg.h"
 #include "regi.h"
+#include "version.h"
 
 
 extern int	errno;
@@ -55,6 +56,7 @@ static int	is_filedir();
 Entry		top;
 
 
+int
 main( argc, argv)
 int	argc;
 char	*argv[];
@@ -75,8 +77,8 @@ clear( ep)
 Entry	*ep;
 {
 	ep->content[0] = ep->cacheline[0] = ep->url[0] = ep->subdirs[0]
-			= ep->file[0] = ep->title[0] = '\0';
-	ep->flag = ep->attributes = 0;
+			= ep->file[0] = ep->md5[0] = ep->title[0] = '\0';
+	ep->flag = ep->attributes = ep->md5_attrib = 0;
 	ep->isindexfile = ep->foundtitle = ep->foundkey 
 			= ep->foundexp = FALSE;
 	ep->attributes = ep->defattributes = 0;
@@ -88,7 +90,7 @@ Entry	*ep;
 {
 	FILE	*cntlfp,
 		*cachefp,
-		*htmlfp;
+		*htmlfp = NULL;
 
 	char	*cp,
 		*text,
@@ -105,7 +107,7 @@ Entry	*ep;
 	ep->firsttime = TRUE;
 	ep->serveall = FALSE;
 	clear_slist();
-	strcpy( ep->defaultcontent, DEFAULT_CONTENT_TYPE);
+	mystrncpy( ep->defaultcontent, DEFAULT_CONTENT_TYPE, SMALLLEN);
 	ep->doindex = ep->inlist = FALSE;
 	
 	mystrncpy( tmpfpath, ep->cachefpath, MIDLEN - 20);
@@ -158,9 +160,9 @@ Entry	*ep;
 			mystrncpy( ep->file, text, SMALLLEN);
 			/* used by getcontent() */
 
-			strcpy( htmlpath, ep->cachefpath);
+			mystrncpy( htmlpath, ep->cachefpath, MIDLEN - SMALLLEN);
 			cp = strrchr( htmlpath, '/');
-			strcpy( ++cp, text);
+			mystrncpy( ++cp, text, SMALLLEN);
 
 			add_to_slist( text);
 			addpair( "file", text, ep);
@@ -180,8 +182,10 @@ Entry	*ep;
 			mystrncpy( ep->file, text, SMALLLEN);
 			add_to_slist( text);
 			addpair( "file", text, ep);
-			if ( *bufp == 'l')	/* It's Link= */
+			if ( *bufp == 'l') {	/* It's Link= */
 				ep->flag |= WN_ISLINK;
+				ep->md5_attrib |= NO_MD5;
+			}
 			continue;
 		}
 
@@ -193,6 +197,7 @@ Entry	*ep;
 				writeitem( cachefp, htmlfp, ep);
 			addpair( "url", text, ep);
 			strcpy( ep->url, text);
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -339,12 +344,20 @@ Entry	*ep;
 			continue;
 		}
 
+		if ( streq( bufp, "httpstatus")) {
+			if ( !is_filedir( bufp, ep))
+				continue;
+			addpair( "status", text, ep);
+			continue;
+		}
+
 		if ( streq( bufp, "refresh")) {
 			if ( !is_filedir( bufp, ep))
 				continue;
 			strcpy( buf, "Refresh: ");
 			strcat( buf, text);
 			addpair( "header", buf, ep);
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -361,6 +374,7 @@ Entry	*ep;
 			if ( !is_filedir( bufp, ep))
 				continue;
 			ep->attributes |= WN_PARSE;
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -368,6 +382,7 @@ Entry	*ep;
 			if ( !is_filedir( bufp, ep))
 				continue;
 			addpair( "redirect", text, ep);
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -414,6 +429,7 @@ Entry	*ep;
 				continue;
 			despace( text);
 			addpair( "includes", text, ep);
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -422,6 +438,7 @@ Entry	*ep;
 				continue;
 			despace( text);
 			addpair( "list", text, ep);
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -430,6 +447,7 @@ Entry	*ep;
 				continue;
 			despace( text);
 			addpair( "wrappers", text, ep);
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -439,6 +457,7 @@ Entry	*ep;
 			else {
 				addpair( "swrapper", text, ep);
 			}
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -446,11 +465,13 @@ Entry	*ep;
 			if ( !is_filedir( bufp, ep))
 				continue;
 			addpair( "nomatchsub", text, ep);
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
 		if ( streq( bufp, "nomatchsub")) {
 			addpair( "nomatchsub", text, ep);
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -459,6 +480,7 @@ Entry	*ep;
 				continue;
 			ep->attributes |= WN_FILTERED;
 			addpair( "filter", text, ep);
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -467,6 +489,7 @@ Entry	*ep;
 				continue;
 			ep->attributes |= WN_CGI;
 			addpair( "handler", text, ep);
+			ep->md5_attrib |= NO_MD5;
 			continue;
 		}
 
@@ -612,7 +635,7 @@ Entry	*ep;
 	}
 	if ( ep->serveall) {
 		strcpy( buf, ep->cachefpath);
-		if ( cp = strrchr( buf, '/'))
+		if ( (cp = strrchr( buf, '/')))
 			*++cp = '\0';
 		do_serveall( buf, cachefp, htmlfp, ep);
 	}
@@ -728,6 +751,9 @@ Entry	*ep;
 		((!ep->foundexp) || (!ep->foundtitle) || (!ep->foundkey)))
 		getkeytitle( ep);
 
+	if ( ep->md5_attrib & (DO_MD5 + DEF_DO_MD5) )
+		getmd5( ep);
+
 	if ( !*ep->title ) {
 		sprintf( buf, "File %s", ep->file);
 		addpair ("title", buf, ep);
@@ -735,7 +761,7 @@ Entry	*ep;
 	}
 
 	if ( ep->attributes) {
-		sprintf( buf, "%ld", ep->attributes);
+		sprintf( buf, "%d", ep->attributes);
 		addpair( "attributes", buf, ep);
 	}
 
@@ -782,7 +808,7 @@ Entry	*ep;
 
 	if ( (which_subdirs != WNDEX_NONE) && !*(ep->subdirs)) {
 		strcpy( buf, ep->cachefpath);
-		if ( cp = strrchr( buf, '/'))
+		if ( (cp = strrchr( buf, '/')))
 			*cp = '\0';
 		mksubd_list( buf, ep);
 		if ( *(ep->subdirs))
@@ -795,7 +821,7 @@ Entry	*ep;
 
 
 	if ( ep->defattributes) {
-		sprintf( buf, "%ld", ep->defattributes);
+		sprintf( buf, "%d", ep->defattributes);
 		addpair( "defattributes", buf, ep);
 	}
 
@@ -922,14 +948,17 @@ FILE	*fp;
 	char	*bufp,
 		extrabuf[MIDLEN];
 
-	while ( bufp = fgets( buf, MIDLEN/2, fp)) {
+	while ( (bufp = fgets( buf, MIDLEN/2, fp))) {
 		chop( bufp);
 
-		if ( (cp = strchr( bufp, '#')) != NULL) {
+		cp = bufp;
+		while ( (cp = strchr( cp, '#')) != NULL) {
 			if ( ( cp != bufp) && (*(cp-1) == '\\'))
 				strcpy( cp-1, cp);
-			else
+			else {
 				*cp = '\0';
+				break;
+			}
 		}
 
 		cp = buf + strlen(buf) - 1;
@@ -1212,7 +1241,15 @@ unsigned	*attrib;
 			continue;
 		}
 		if ( streq( word, "post") ) {
-			*attrib |= WN_POST_OK;
+			*attrib |= WN_POST_ONLY;
+			continue;
+		}
+		if ( streq( word, "nopost") ) {
+			*attrib |= WN_NO_POST;
+			continue;
+		}
+		if ( streq( word, "put") ) {
+			*attrib |= WN_PUT_OK;
 			continue;
 		}
 		if ( streq( word, "noparse") ) {
@@ -1227,6 +1264,20 @@ unsigned	*attrib;
 			*attrib |= WN_UNBUFFERED;
 			continue;
 		}
+		if ( streq( word, "md5") ) {
+			if ( attrib == &(ep->attributes) )
+				ep->md5_attrib |= DO_MD5;
+			else if ( attrib == &(ep->defattributes) )
+				ep->md5_attrib |= DEF_DO_MD5;
+			continue;
+		}
+		if ( streq( word, "nomd5") ) {
+			if ( attrib == &(ep->attributes) )
+				ep->md5_attrib |= UNDO_MD5;
+			else if ( attrib == &(ep->defattributes) )
+				ep->md5_attrib |= UNDO_MD5;
+			continue;
+		}
 		fprintf( stderr, ERRMSG19, word);
 			continue;
 	}
