@@ -1,9 +1,9 @@
 /*
     Wn: A Server for the HTTP
     File: wndex/wndex.h
-    Version 1.17.4
+    Version 2.0.0
     
-    Copyright (C) 1995, 1996  <by John Franks>
+    Copyright (C) 1995-8  <by John Franks>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,6 +51,12 @@
 #define WN_ISLINK	(8)
 #define WN_ISURL	(16)
 
+/* bits of md5_attrib */
+#define DO_MD5		(1)	/* Calculate base64 of MD5 digest */
+#define DEF_DO_MD5	(2)	/* Calculate MD5 by default */
+#define UNDO_MD5	(4)	/* Choose not to do MD5 */
+#define NO_MD5		(8)	/* Can't do MD5 */
+
 #define	hasencoding(x)	(x->flag & WN_HASENCODING)
 #define hascontent(x)	(x->flag & WN_HASCONTENT)
 
@@ -61,7 +67,7 @@ extern void	init(),
 		mksubd_list(),
 		getcontent(),
 		getkeytitle(),
-		exit(),
+		getmd5(),
 		writeitem(),
 		clear_slist(),
 		add_to_slist(),
@@ -80,6 +86,7 @@ extern int	recurse,
 extern char	*get_next_line(),
 		*strlower(),
 		*mystrncpy(),
+		*md5digest(),
 		cntlfname[],
 		cachefname[];
 
@@ -88,14 +95,15 @@ typedef struct Entry {
 	char	file[SMALLLEN],
 		url[MIDLEN],
 		title[MIDLEN],
-		content[SMALLLEN],
+		content[SMALLLEN],	/* content-type */
+		md5[SMALLLEN/2],	/* base64 of MD5 hash */
 		defaultcontent[SMALLLEN],
 		owner[MIDLEN],
 		cacheline[CACHELINE_LEN],
 		subdirs[CACHELINE_LEN],
 		accessfile[SMALLLEN],
-		cntlfpath[MIDLEN],
-		cachefpath[MIDLEN];
+		cntlfpath[MIDLEN],	/* path to index file */
+		cachefpath[MIDLEN];	/* path to index.cache file */
 
 	int	foundtitle,
 		foundexp,
@@ -108,6 +116,7 @@ typedef struct Entry {
 
 
 	unsigned	flag,
+			md5_attrib,
 			attributes,
 			defattributes;
 
